/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.core;

import ipl.exceptions.XmlParseErrorException;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.regex.*;

import ipl.util.*;
import ipl.splitter.SentenceSplitter;
import ipl.nerc.regex.timex.TimexMatcher;

/**
 * <p>Description: Produces the tokens of a document.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class Tokenizer {
    
    private ArrayList<TokenNode> tokens;
    private String fileName;
    
    public Tokenizer(String fileName, String text, TimexMatcher matcher, SentenceSplitter splitter) {
        this.tokens = new ArrayList<TokenNode>();
        this.fileName = fileName;
        text = Auxiliary.removeNoisySpaces(text);
        
        XmlParseTree tree = null;
        try {
            tree = new XmlParseTree(text, fileName);
        } catch (XmlParseErrorException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
        crossTree(tree.getRoot(), false);
        
        if (splitter != null) {
            splitter.setTokenizer(this);
            splitter.setSentenceTags();
        } // if
        
        if (matcher != null) {
            matcher.setDateTimeTags(this);
        } // if
    } // constructor: Tokenizer
    
    public Tokenizer(ArrayList<TokenNode> tokens) {
        this.tokens = tokens;
    } // constructor: Tokenizer
    
    public TokenNode get(int offset) {
        return tokens.get(offset);
    } // get
    
    public int size() {
        return tokens.size();
    } // size
    
    private void crossTree(ParseTreeNode curr, boolean insideSubtitle) {
        int i = 0;
        if ((curr.getTag().equalsIgnoreCase(Constants.SUBTITLE))) {
            insideSubtitle = true;
        } // if
        else if ((curr.getTag().equalsIgnoreCase(Constants.TITLE))) {
            insideSubtitle = true;
        } // else if
        while (i < curr.getPointersSize()) {
            if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.SUBTITLE)) {
                createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 2, 0, curr.getPrevChar(i));
            } // if
            else if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.TITLE)) {
                createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 2, 0, curr.getPrevChar(i));
            } // else if
            else if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.SENTENCE)) {
                createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 0, 2, curr.getPrevChar(i));
            } // else if
            else {
                createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 0, 0, curr.getPrevChar(i));
            } // else
            crossTree(curr.getPointer(i), insideSubtitle);
            i++;
        } // while
        if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.SUBTITLE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 1, 0, curr.getPrevChar(i));
        } // if
        else if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.TITLE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 1, 0, curr.getPrevChar(i));
        } // else if
        else if (curr.getTag().equalsIgnoreCase(Constants.SUBTITLE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 3, 0, curr.getPrevChar(i));
        } // else if
        else if (curr.getTag().equalsIgnoreCase(Constants.TITLE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 3, 0, curr.getPrevChar(i));
        } // else if
        else if ((i == 0) && curr.getTag().equalsIgnoreCase(Constants.SENTENCE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 0, 1, curr.getPrevChar(i));
        } // else if
        else if (curr.getTag().equalsIgnoreCase(Constants.SENTENCE)) {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 0, 3, curr.getPrevChar(i));
        } // else if
        else {
            createTokens(curr.getText(i), curr.getType(), curr.getConfScores(), insideSubtitle, 0, 0, curr.getPrevChar(i));
        } // else
    } // crossTree
    
    // start_end_subtitle and start_end_sentence:
    //   0. neither start nor end
    //   1. both start and end
    //   2. start
    //   3. end
    private void createTokens(String text, String tagType, ArrayList<Double> confs,
            boolean insideSubtitle, int start_end_sub,
            int start_end_sentence, String prevChar) {
        int i;
        int tokensStartSize = tokens.size();
        StringTokenizer sTokenizer = new StringTokenizer(text, " \n", true);
        String tok;
        TokenNode tn;
        String grWord = "\\p{InGreek}*";
        String enWord = "[a-zA-Z]*";
        String number = "[0-9]*";
        //    (prevCharsType, currCharType)  
        //  :
        //    0. nothing
        //    1. gr letter
        //    2. en letter
        //    3. digit
        //    4. dot
        //    5. other
        int prevCharsType, currCharType;
        String prevChars;
        char currChar;
        
        if ((sTokenizer.countTokens() == 0) && (start_end_sub == 2)) {
            tn = new TokenNode("", tagType, false, insideSubtitle, prevChar);
            tokens.add(tn);
        } // if
        
        if ((sTokenizer.countTokens() == 0) && (start_end_sentence == 2)) {
            if  ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                tokens.get(tokens.size() - 1).
                        setTokenNode("", tagType, false, insideSubtitle);
                tokensStartSize--;
            } // if
            else {
                tn = new TokenNode("", tagType, false, insideSubtitle, prevChar);
                tokens.add(tn);
            } // else
        } // if
        
        while (sTokenizer.hasMoreTokens()) {
            tok = sTokenizer.nextToken();
            if (tok.equals(" ") || tok.equals("\n")) {
                prevChar = tok;
            } // if
            else {
                int pos;
                if ((pos = tok.indexOf(Constants.PARAGRAPH_START_TAG)) != -1) {
                    if ((pos != 0) || (tok.length() != Constants.PARAGRAPH_START_TAG.length())) {
                        System.err.println("UNCOUGHT CASE");
                        System.exit(1);
                    } // if
                    if (!tokens.isEmpty()) {
                        tokens.get(tokens.size() - 1).setEndOfPeriod(true);
                    } // if
                } // if
                else if ((pos = tok.indexOf(Constants.PARAGRAPH_END_TAG)) != -1) {
                    if ((pos != 0) || (tok.length() != Constants.PARAGRAPH_END_TAG.length())) {
                        System.err.println("UNCOUGHT CASE");
                        System.exit(1);
                    } // if
                    if (!tokens.isEmpty()) {
                        tokens.get(tokens.size() - 1).setEndOfPeriod(true);
                    } // if
                } // if
                //   tok     ,    tok 
                //        .
                else if (Pattern.matches(grWord, tok) || Pattern.matches(enWord, tok) ||
                        Pattern.matches(number, tok)) {
                    if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                        tokens.get(tokens.size() - 1).
                                setTokenNode(tok, tagType, false, insideSubtitle);
                        tokensStartSize--;
                    } // if
                    else {
                        tn = new TokenNode(tok, tagType, false, insideSubtitle, prevChar);
                        tokens.add(tn);
                    } // else
                } // else if
                //   tok      .  ,
                // tok="boo123",      ,  "boo"  
                // "123".
                else {
                    prevCharsType = 0;
                    prevChars = "";
                    //     tok.
                    for (i = 0; i < tok.length(); i++) {
                        currChar = tok.charAt(i);
                        //     .
                        if (isGrLetter(currChar)) {
                            currCharType = 1;
                        } // if
                        else if (isEnLetter(currChar)) {
                            currCharType = 2;
                        } // else if
                        else if (isDigit(currChar)) {
                            currCharType = 3;
                        } // else if
                        else if (currChar == '.') {
                            currCharType = 4;
                        } // else
                        else {
                            currCharType = 5;
                        } // else
                        
                        //            
                        //  ,     
                        // prevChars.        
                        //     currCharType  1 ( ) 
                        // 2 ( )  3 ().
                        if (currCharType == prevCharsType) {
                            prevChars = prevChars.concat(String.valueOf(currChar));
                        } // if
                        //             
                        //  .
                        else {
                            if (prevCharsType != 0) {
                                if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                                    tokens.get(tokens.size() - 1).
                                            setTokenNode(prevChars, tagType, false, insideSubtitle);
                                    tokensStartSize--;
                                } // if
                                else {
                                    tn = new TokenNode(prevChars, tagType, false, insideSubtitle, prevChar);
                                    tokens.add(tn);
                                } // else
                                prevChar = "";
                            } // if
                            if (currCharType == 5) {
                                if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                                    tokens.get(tokens.size() - 1).
                                            setTokenNode(String.valueOf(currChar), tagType, false, insideSubtitle);
                                    tokensStartSize--;
                                } // if
                                else {
                                    tn = new TokenNode(String.valueOf(currChar), tagType, false, insideSubtitle, prevChar);
                                    tokens.add(tn);
                                } // else
                                prevChars = "";
                                prevCharsType = 0;
                                prevChar = "";
                            } // if
                            else if (currCharType == 4) {
                                if ((i == (tok.length() - 1)) || (i == (tok.length() - 2))) {
                                    prevChar = "";
                                    if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                                        tokens.get(tokens.size() - 1).
                                                setTokenNode(String.valueOf(currChar), tagType, false, insideSubtitle);
                                        tokensStartSize--;
                                    } // if
                                    else {
                                        tn = new TokenNode(String.valueOf(currChar), tagType, false, insideSubtitle, prevChar);
                                        tokens.add(tn);
                                    } // else
                                } // if
                                else {
                                    char ch1 = tok.charAt(i + 1);
                                    char ch2 = tok.charAt(i + 2);
                                    if ((ch1 == '\\') && (ch2 == 'y')) {
                                        if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                                            tokens.get(tokens.size() - 1).
                                                    setTokenNode(String.valueOf(currChar), tagType, true, insideSubtitle);
                                            tokensStartSize--;
                                        } // if
                                        else {
                                            tn = new TokenNode(String.valueOf(currChar), tagType, true, insideSubtitle, prevChar);
                                            tokens.add(tn);
                                        } // else
                                        i += 2;
                                    } // if
                                    else {
                                        if ((ch1 == '\\') && (ch2 == 'n')) {
                                            i += 2;
                                        } // if
                                        if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                                            tokens.get(tokens.size() - 1).
                                                    setTokenNode(String.valueOf(currChar), tagType, false, insideSubtitle);
                                            tokensStartSize--;
                                        } // if
                                        else {
                                            tn = new TokenNode(String.valueOf(currChar), tagType, false, insideSubtitle, prevChar);
                                            tokens.add(tn);
                                        } // else
                                    } // else
                                } // else if
                                
                                prevChar = "";
                                prevChars = "";
                                prevCharsType = 0;
                            } // else if
                            //  currCharType   1 ( ) 
                            // 2 ( )  3 ().
                            else {
                                prevChars = String.valueOf(currChar);
                                prevCharsType = currCharType;
                            } // else
                        } // else
                    } // for
                    if (prevCharsType != 0) {
                        if ((tokens.size() != 0) && (tokens.get(tokens.size() - 1).getToken().equals(""))) {
                            tokens.get(tokens.size() - 1).
                                    setTokenNode(prevChars, tagType, false, insideSubtitle);
                            tokensStartSize--;
                        } // if
                        else {
                            tn = new TokenNode(prevChars, tagType, false, insideSubtitle, "");
                            tokens.add(tn);
                        } //else
                    } // if
                } // else
            } // else
        } // while
        
        // Update tags:
        //    endOfPeriod
        //    startOfSubtitle
        //    endOfSubtitle
        if ((tokensStartSize == 0) && (tokens.size() != 0) && (insideSubtitle)) {
            tokens.get(tokensStartSize).setStartSubtitle(true);
        } // if
        if (start_end_sub == 1) {
            if (tokensStartSize - 1 >= 0) {
                tokens.get(tokensStartSize - 1).setEndOfPeriod(true);
            } // if
            if (tokens.size() > tokensStartSize) {
                tokens.get(tokensStartSize).setStartSubtitle(true);
            } // if
            if (!tokens.isEmpty()) {
                tokens.get(tokens.size() - 1).setEndOfPeriod(true);
                tokens.get(tokens.size() - 1).setEndSubtitle(true);
            } // if
        } // if
        else if (start_end_sub == 2) {
            if (tokensStartSize - 1 >= 0) {
                tokens.get(tokensStartSize - 1).setEndOfPeriod(true);
            } // if
            if (tokens.size() > tokensStartSize) {
                tokens.get(tokensStartSize).setStartSubtitle(true);
            } // if
        } // else if
        else if (start_end_sub == 3) {
            tokens.get(tokens.size() - 1).setEndOfPeriod(true);
            tokens.get(tokens.size() - 1).setEndSubtitle(true);
        } // else if
        
        // Update tags:
        //    endOfPeriod
        if (start_end_sentence == 1) {
            if (tokensStartSize - 1 >= 0) {
                tokens.get(tokensStartSize - 1).setEndOfPeriod(true);
            } // if
            if (!tokens.isEmpty()) {
                tokens.get(tokens.size() - 1).setEndOfPeriod(true);
            } // if
        } // if
        else if (start_end_sentence == 2) {
            if (tokensStartSize - 1 >= 0) {
                tokens.get(tokensStartSize - 1).setEndOfPeriod(true);
            } // if
        } // else if
        else if (start_end_sentence == 3) {
            tokens.get(tokens.size() - 1).setEndOfPeriod(true);
        } // else if
        
        // Update conf scores
        if ((tagType.compareTo(Constants.PERSON) == 0) ||
                (tagType.compareTo(Constants.ORGANIZATION) == 0) ||
                (tagType.compareTo(Constants.LOCATION) == 0)) {
            if ((tokens.size() - tokensStartSize) == confs.size()) {
                for (int j = 0; j < confs.size(); j++) {
                    tokens.get(tokensStartSize + j).setPredictedProbability(confs.get(j));
                } // for
            } // if
        } // if
    } // createTokens
    
    private boolean isGrLetter(char ch) {
        if ( ( (ch >= '') && (ch <= '')) || ( (ch >= '') && (ch <= '')) ||
                (ch == '') || (ch == '') || (ch == '') || (ch == '') || (ch == '') ||
                (ch == '') || (ch == '') ||
                (ch == '') || (ch == '') || (ch == '') || (ch == '') || (ch == '') ||
                (ch == '') || (ch == '') ||
                (ch == '') || (ch == '') || (ch == '') || (ch == '')) {
            return true;
        } // if
        else {
            return false;
        } // else
    } // isGrLetter
    
    private boolean isEnLetter(char ch) {
        if ( ( (ch >= 'a') && (ch <= 'z')) || ( (ch >= 'A') && (ch <= 'Z'))) {
            return true;
        } // if
        else {
            return false;
        } // else
    } // isEnLetter
    
    private boolean isDigit(char ch) {
        if ( (ch >= '0') && (ch <= '9')) {
            return true;
        } // if
        else {
            return false;
        } // else
    } // isDigit
    
    
    private String getStartTag(String category, int pos) {
      
        String tag = "";
        if (category.compareTo(Constants.PERSON) == 0) {
            tag = Constants.PERSON_START_TAG_OPEN;
        } // if
        else if (category.compareTo(Constants.ORGANIZATION) == 0) {
            tag = Constants.ORGANIZATION_START_TAG_OPEN;
        } // else if
        else if (category.compareTo(Constants.LOCATION) == 0) {
            tag = Constants.LOCATION_START_TAG_OPEN;
        } // else if
        else if (category.compareTo(Constants.DATE) == 0) {
            return Constants.DATE_START_TAG;
        } // else if
        else if (category.compareTo(Constants.TIME) == 0) {
            return Constants.TIME_START_TAG;
        } // else if
        else if (category.compareTo(Constants.MONEY) == 0) {
            return Constants.MONEY_START_TAG;
        } // else if
        else if (category.compareTo(Constants.PERCENT) == 0) {
            return Constants.PERCENT_START_TAG;
        } // else if
        // category = NONE
        else {
            return "";
        } // else
        
        int counter = 0;
        boolean stop = false;
        while ((pos + counter < tokens.size()) && (tokens.get(pos + counter).tagType.compareTo(category) == 0) && !stop) {
            tag += " CONF" + Integer.toString(counter) + "=\"";
            tag += Double.toString(tokens.get(pos + counter).predicted_probability);
            tag += "\"";
            if (tokens.get(pos + counter).getEndOfPeriod()) {
                stop = true;
            } // if
            counter++;
        } // while
        tag += ">";
        
        return tag;
    } // getStartTag
    
    
    private String getEndTag(String category) {
        if (category.compareTo(Constants.PERSON) == 0) {
            return Constants.PERSON_END_TAG;
        } // if
        else if (category.compareTo(Constants.ORGANIZATION) == 0) {
            return Constants.ORGANIZATION_END_TAG;
        } // else if
        else if (category.compareTo(Constants.LOCATION) == 0) {
            return Constants.LOCATION_END_TAG;
        } // else if
        else if (category.compareTo(Constants.DATE) == 0) {
            return Constants.DATE_END_TAG;
        } // else if
        else if (category.compareTo(Constants.TIME) == 0) {
            return Constants.TIME_END_TAG;
        } // else if
        else if (category.compareTo(Constants.MONEY) == 0) {
            return Constants.MONEY_END_TAG;
        } // else if
        else if (category.compareTo(Constants.PERCENT) == 0) {
            return Constants.PERCENT_END_TAG;
        } // else if
        // category = NONE
        else {
            return "";
        } // else
    } // getEndTag
    
    public String getStartTag(int pos) {
        String category = tokens.get(pos).tagType;
        // If the current token is the first token
        // AND if first's token tag type is not NONE
        if ((pos == 0) && (category.compareTo(Constants.NONE) != 0)) {
            return getStartTag(category, pos);
        } // if
        // If the current token is the first token
        // AND if first's token tag type is NONE
        else if (pos == 0) {
            return "";
        } // else if
        
        //if pos!=0 - If current token has not the same tag type as the previous token
        else if ((category.compareTo(Constants.NONE) != 0) &&
                (category.compareTo(tokens.get(pos - 1).getTagType()) != 0)) {
            return getStartTag(category, pos);
        } // else if
        else if ((category.compareTo(Constants.NONE) != 0) &&
                (tokens.get(pos-1).getEndOfPeriod())) {
            return getStartTag(category, pos);
        } // else if
        else {
            return "";
        } // else
    } // getStartTag
    
    public String getEndTag(int pos) {
        String category = tokens.get(pos).tagType;
        if ((pos == tokens.size() - 1) && (category.compareTo(Constants.NONE) != 0)) {
            return getEndTag(category);
        } // if
        else if (pos == tokens.size() - 1) {
            return "";
        } // else if
        else if ((category.compareTo(Constants.NONE) != 0) &&
                (category.compareTo(tokens.get(pos + 1).getTagType()) != 0)) {
            return getEndTag(category);
        } // else if
        else if ((category.compareTo(Constants.NONE) != 0) &&
                (tokens.get(pos).getEndOfPeriod())) {
            return getEndTag(category);
        } // else if
        else {
            return "";
        } // else
    } // getEndTag
    
    public String getStartSubtitle(int pos) {
        if (tokens.get(pos).isStartSubtitle()) {
            return Constants.SUBTITLE_START_TAG;
        } // if
        else {
            return "";
        } // else
    } // getStartSubtitle
    
    public String getEndSubtitle(int pos) {
        if (tokens.get(pos).isEndSubtitle()) {
            return Constants.SUBTITLE_END_TAG;
        } // if
        else {
            return "";
        } // else
    } // getEndSubtitle
    
    
} // end of class Tokenizer
