/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc;

import ipl.core.Tokenizer;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import ipl.nerc.lists.ListManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;
import ipl.util.Auxiliary;
import ipl.util.Constants;
import ipl.util.file_manager.FileManager;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;

/**
 *
 * @author Xen.Vassilakos
 */
public class InputTextsManager {
    
    /** Creates a new instance of InputTextsManager */
    public InputTextsManager() {
    }
    
    
    
    /**
     * Finds decisions made by first pass classifiers, for all tokens in file saved in path "trainFilePath".
     * The hashtable array returned, contains two hashtables: <br>
     * - The first Hashtable contains the "sure decisions" and <br>
     * - The second one contains all decisions from all texts.
     *
     * More Details:<br>
     * 1.In each text, first pass classifiers can be sure enought for some classification decisions they make.
     * For each text, an array of those surelly classified tokens is used in order to know if a token found
     * in a text, was also found again in the same text (in a deferent position), and first pass classifiers
     * where sure enought about classifing it. A <b>sure decision</b>, is a decision for which it's cannonical
     * posibility is grater than the margin parameter (you may use the margin value stored in static variable
     * Constants.SURE_DEC_THRESHOLD = 0.8).<br>
     * 2.Cannonical possibilities of each classification decision, are saved in a Hashtable<String, String>.
     * Strings such as [category] + "#" + trainFilePath + "#" + position Of token text, are used as keys for cannonical possibilities
     * (a double value converted to String format) .
     *
     *
     * @param FPVectorsMangr
     *          A FeaturesManager Object for finding first pass feature values
     * @param model
     *          An svm_model object which represents the model (classifier used) as an object.
     * @param margin
     *          A margin used to define which decisions made by model, are considered sure (must be in [-1,1])
     *
     * @return The array of the 2 hashtables.
     *              <br> At position 0,  sure decisions Hashtable<String, String[]>.
     *              <br> At position 1, all decisions canonical possibilities Hashtable<String, String>.
     */
    public static String[][] findFPDecisionsForSP(
            FeaturesManager FPVectorsMangr, svm_model model, double margin, String category){
        
        try {
            
            if (margin < -1 || margin>1) {
                throw new Exception("Margin for sure decisions out of bounds. " +
                        "Margin \"m\" must be less than 1 and grater than 0");
            }
            
        } catch (Exception e){
            
            e.printStackTrace();
            
            System.err.println("Margin redefined as: " + Constants.SURE_DEC_MARGIN);
            margin = Constants.SURE_DEC_MARGIN;
            
        }
        // will include all sure decisions (canonical posibility > margin)
        ArrayList<String> sureDecs = new ArrayList<String>(FPVectorsMangr.getTokens().size()/10) ;
        String []allDecsArr = new String [FPVectorsMangr.getTokens().size()];
        
        int distanceFromStartOfEntity = -1;
        String[] attributes;
        
        int nr_class = svm.svm_get_nr_class(model);
        int []categories = new int[nr_class];
        svm.svm_get_labels(model, categories);
        double[] prob_estimates = new double[nr_class];
        
        int listNum = 0;
        
        
        // constucting array of features of FP
        if(category.equalsIgnoreCase(Constants.PERSON)){
            
            attributes = new String[Constants.PER_FEAT_NUM_FP];
            
        }else{// if(category.equalsIgnoreCase(Constants.ORGANIZATION)){
            
            attributes = new String[Constants.ORG_FEAT_NUM_FP];
            
        }
        
        while ( (attributes = FPVectorsMangr.nextFeatVec(distanceFromStartOfEntity)) != null) {
            
            int tokPos = FPVectorsMangr.getOffset() -1;
            
            // answer  and canonical probanility (-1,1) for FP decisions
            double answer, canonicProb;
            
            if (attributes.length != 1) {
                
                int numOfAttributes = attributes.length - 1;
                svm_node node[] = new svm_node[numOfAttributes];
                
                for (int i = 1; i <= numOfAttributes; i++) {
                    node[i - 1] = new svm_node();
                    node[i - 1].index = i;
                    node[i - 1].value = Auxiliary.atof(attributes[i]);
                } // for
                
                // PREDICT
                answer = svm.svm_predict_probability(model, node,
                        prob_estimates);
                
                if (categories[0] == -1) {
                    canonicProb = prob_estimates[1] - prob_estimates[0];
                } // if
                
                else {
                    canonicProb = prob_estimates[0] - prob_estimates[1];
                } // else
                
            } // if
            
            else {// cat by safe rules
                answer = -1;
                canonicProb = -1;
            } // else
            
            FPVectorsMangr.tokenAt(tokPos).setPredictedProbability(canonicProb);
            
            allDecsArr[tokPos] = String.valueOf(canonicProb);
            
            if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
                
                distanceFromStartOfEntity = 1;
                String currTok = FPVectorsMangr.tokenAt(tokPos).getToken();
                
                if (currTok.length() >= 3 && canonicProb > margin){
                    listNum++;
                    sureDecs.add(currTok.toLowerCase());
                } // if
                
            } // if
            else if (answer == 1) {
                
                distanceFromStartOfEntity++;
                String currTok = FPVectorsMangr.tokenAt(
                        FPVectorsMangr.getOffset() - 1).getToken();
                
                if (currTok.length() >= 3 && canonicProb > margin){
                    listNum++;
                    sureDecs.add(currTok.toLowerCase());
                } // if
                
            } // else if
            
            else if (answer == -1){
                distanceFromStartOfEntity = -1;
            } // else if
            
        } // while
        
        
        // sorting firtst pass list
        String[] sureDecsArr = new String[sureDecs.size()];
        sureDecs.toArray(sureDecsArr);
        Arrays.sort(sureDecsArr);
        
        
        // to be returned
        String [][]toReturn = new String[2][1];
        toReturn[0] = sureDecsArr;
        toReturn[1] = allDecsArr;
        
        return toReturn;
        
    }// findFPDecisionsForSP
    
    /**Returns tokenizer for text of file at filePath. First Pass Classifiers used*/
    public static Tokenizer tokenizerForFP(
            String filePath, TimexMatcher matcher,
            SentenceSplitter sentSplitter, String personModelFileNameFP,
            String orgModelFileNameFP, ListManager personListManagerFP,
            ListManager orgListManagerFP) throws Exception {
        
        svm_model perModel = svm.svm_load_model(personModelFileNameFP);
        svm_model orgModel = svm.svm_load_model(orgModelFileNameFP);
        String text = FileManager.read(filePath);
        
        PersonFeaturesManager perVecManagerFP = new PersonFeaturesManager(filePath, text,
                personListManagerFP, matcher, sentSplitter);
        OrganizationFeaturesManager orgVecManagerFP = new OrganizationFeaturesManager(filePath, text,
                orgListManagerFP, matcher, sentSplitter);
        
        Tokenizer tokens = perVecManagerFP.getTokens();
        
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        int perCategories[] = new int[per_nr_class];
        int orgCategories[] = new int[org_nr_class];
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        String FPPerAttr[];
        String FPOrgAttr[];
        int distanceFromStartOfPer = -1;
        int distanceFromStartOfOrg = -1;
        
        while ((FPPerAttr = perVecManagerFP.nextFeatVec(distanceFromStartOfPer))!= null &&
                (FPOrgAttr = orgVecManagerFP.nextFeatVec(distanceFromStartOfOrg)) != null) {
            
            double perAnswer, orgAnswer, perProbability, orgProbability;
            if (FPPerAttr.length != 1) {
                int numOfperAttributes = Constants.PER_FEAT_NUM_FP;
                svm_node nodes[] = new svm_node[numOfperAttributes];
                for (int i = 0; i < numOfperAttributes; i ++) {
                    nodes[i] = new svm_node();
                    nodes[i].index = i + 1;
                    nodes[i].value = Auxiliary.atof(FPPerAttr[i + 1]);
                }
                perAnswer  = svm.svm_predict_probability(perModel, nodes, per_prob_estimates);
                if (perCategories[0] == -1) {
                    perProbability = per_prob_estimates[1] - per_prob_estimates[0];
                } else {
                    perProbability = per_prob_estimates[0] - per_prob_estimates[1];
                }
            } else {
                perAnswer = -1;
                perProbability = -1;
            }
            if (FPOrgAttr.length != 1) {
                int numOforgAttributes = Constants.ORG_FEAT_NUM_FP;
                svm_node nodes[] = new svm_node[numOforgAttributes];
                for (int i = 0; i < numOforgAttributes; i ++) {
                    nodes[i] = new svm_node();
                    nodes[i].index = i + 1;
                    nodes[i].value = Auxiliary.atof(FPOrgAttr[i + 1]);
                }
                orgAnswer  = svm.svm_predict_probability(orgModel, nodes, org_prob_estimates);
                if (orgCategories[0] == -1) {
                    orgProbability = org_prob_estimates[1] - org_prob_estimates[0];
                } else {
                    orgProbability = org_prob_estimates[0] - org_prob_estimates[1];
                }
            } else {
                orgAnswer = -1;
                orgProbability = -1;
            }
            if (perProbability > 0 && orgProbability < 0) {
                perAnswer = 1.0;
                orgAnswer = -1.0;
            } else if (orgProbability > 0 && perProbability < 0) {
                orgAnswer = 1.0;
                perAnswer = -1.0;
            } else if (orgProbability > perProbability && perProbability >= 0) {
                orgAnswer = 1.0;
                perAnswer = -1.0;
            } else if (perProbability > orgProbability && orgProbability >= 0) {
                orgAnswer = -1.0;
                perAnswer = 1.0;
            } else if (orgProbability < 0 && perProbability < 0) {
                orgAnswer = -1.0;
                perAnswer = -1.0;
            } else if (perProbability == orgProbability) {
                if ((Math.ceil(Math.random()) * 10) % 2 == 0) {
                    perAnswer = -1.0;
                    orgAnswer = 1.0;
                } else {
                    orgAnswer = -1.0;
                    perAnswer = 1.0;
                }
            }
            
            tokens.get(perVecManagerFP.getOffset() - 1).setPredictedProbability(perProbability);
            tokens.get(orgVecManagerFP.getOffset() - 1).setPredictedProbability(orgProbability);
            
            if (perAnswer == 1) {
                tokens.get(perVecManagerFP.getOffset() - 1).setTagType(Constants.PERSON);
            }
            if (orgAnswer == 1) {
                tokens.get(orgVecManagerFP.getOffset() - 1).setTagType(Constants.ORGANIZATION);
            }
            if (perAnswer == 1 && distanceFromStartOfPer == -1) {
                distanceFromStartOfOrg = -1;
                distanceFromStartOfPer = 1;
            } else if (perAnswer == 1) {
                distanceFromStartOfOrg = -1;
                distanceFromStartOfPer ++;
            } else if (perAnswer == -1 && distanceFromStartOfPer != -1) {
                distanceFromStartOfPer = -1;
            }
            if (orgAnswer == 1 && distanceFromStartOfOrg == -1) {
                distanceFromStartOfPer = -1;
                distanceFromStartOfOrg = 1;
            } else if (orgAnswer == 1) {
                distanceFromStartOfPer = -1;
                distanceFromStartOfOrg ++;
            } else if (orgAnswer == -1 && distanceFromStartOfOrg != -1) {
                distanceFromStartOfOrg = -1;
            }
        }
        
        return tokens;
        
    }
    
    /**Returns tokenizer for text of file at filePath. Second Pass Classifiers used*/
    public static Tokenizer tokenizerForSP(
            String filePath, TimexMatcher matcher,
            SentenceSplitter sentSplitter,
            String personModelFileNameFP, String personModelFileNameSP,
            String orgModelFileNameFP, String orgModelFileNameSP,
            ListManager personListManagerFP,
            ListManager personListManagerSP,
            ListManager orgListManagerFP,
            ListManager orgListManagerSP) throws IOException  {
        
        svm_model perModel = svm.svm_load_model(personModelFileNameFP);
        svm_model orgModel = svm.svm_load_model(orgModelFileNameFP);
        String text = FileManager.read(filePath);
        
        PersonFeaturesManager perVecManagerFP =
                new PersonFeaturesManager(filePath,text, personListManagerFP, matcher, sentSplitter);
        PersonFeaturesManager perVecManagerSP =
                new PersonFeaturesManager(filePath, text, personListManagerSP, matcher, sentSplitter);
        OrganizationFeaturesManager orgVecManagerFP =
                new OrganizationFeaturesManager(filePath, text, orgListManagerFP, matcher, sentSplitter);
        OrganizationFeaturesManager orgVecManagerSP =
                new OrganizationFeaturesManager(filePath, text, orgListManagerSP, matcher, sentSplitter);
        Tokenizer tokens = perVecManagerSP.getTokens();
        
        Hashtable []FPDecisions = Trainer.findFPDecisionsForSP(perVecManagerFP, perModel,
                orgVecManagerFP, orgModel, Constants.SURE_DEC_MARGIN, filePath);
        Hashtable <String, String[]> sureDec = (Hashtable <String, String[]>) FPDecisions[0];
        Hashtable <String, String> allDec = (Hashtable <String, String>) FPDecisions[1];
        
        
        perModel = null;
        perModel = svm.svm_load_model(personModelFileNameSP);
        orgModel = null;
        orgModel = svm.svm_load_model(orgModelFileNameSP);
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        int perCategories[] = new int[per_nr_class];
        int orgCategories[] = new int[org_nr_class];
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        String SPPerAttr[];
        String SPOrgAttr[];
        int distanceFromStartOfPer = -1;
        int distanceFromStartOfOrg = -1;
        
        while ((SPPerAttr = perVecManagerSP.nextFeatVec(distanceFromStartOfPer, sureDec, allDec,
                filePath))!= null &&
                (SPOrgAttr = orgVecManagerSP.nextFeatVec(distanceFromStartOfOrg, sureDec, allDec,
                filePath)) != null) {
            
            double perAnswer, orgAnswer, perProbability, orgProbability;
            if (SPPerAttr.length != 1) {
                int numOfperAttributes = Constants.PER_FEAT_NUM_SP;
                svm_node nodes[] = new svm_node[numOfperAttributes];
                for (int i = 0; i < numOfperAttributes; i ++) {
                    nodes[i] = new svm_node();
                    nodes[i].index = i + 1;
                    nodes[i].value = Auxiliary.atof(SPPerAttr[i + 1]);
                }
                perAnswer  = svm.svm_predict_probability(perModel, nodes, per_prob_estimates);
                if (perCategories[0] == -1) {
                    perProbability = per_prob_estimates[1] - per_prob_estimates[0];
                } else {
                    perProbability = per_prob_estimates[0] - per_prob_estimates[1];
                }
            } else {
                perAnswer = -1;
                perProbability = -1;
            }
            if (SPOrgAttr.length != 1) {
                int numOforgAttributes = Constants.ORG_FEAT_NUM_SP;
                svm_node nodes[] = new svm_node[numOforgAttributes];
                for (int i = 0; i < numOforgAttributes; i ++) {
                    nodes[i] = new svm_node();
                    nodes[i].index = i + 1;
                    nodes[i].value = Auxiliary.atof(SPOrgAttr[i + 1]);
                }
                orgAnswer  = svm.svm_predict_probability(orgModel, nodes, org_prob_estimates);
                if (orgCategories[0] == -1) {
                    orgProbability = org_prob_estimates[1] - org_prob_estimates[0];
                } else {
                    orgProbability = org_prob_estimates[0] - org_prob_estimates[1];
                }
            } else {
                orgAnswer = -1;
                orgProbability = -1;
            }
            if (perProbability > 0 && orgProbability < 0) {
                perAnswer = 1.0;
                orgAnswer = -1.0;
            } else if (orgProbability > 0 && perProbability < 0) {
                orgAnswer = 1.0;
                perAnswer = -1.0;
            } else if (orgProbability > perProbability && perProbability >= 0) {
                orgAnswer = 1.0;
                perAnswer = -1.0;
            } else if (perProbability > orgProbability && orgProbability >= 0) {
                orgAnswer = -1.0;
                perAnswer = 1.0;
            } else if (orgProbability < 0 && perProbability < 0) {
                orgAnswer = -1.0;
                perAnswer = -1.0;
            } else if (perProbability == orgProbability) {
                if ((Math.ceil(Math.random()) * 10) % 2 == 0) {
                    perAnswer = -1.0;
                    orgAnswer = 1.0;
                } else {
                    orgAnswer = -1.0;
                    perAnswer = 1.0;
                }
            }
            if (perAnswer == 1) {
                tokens.get(perVecManagerSP.getOffset() - 1).setPredictedProbability
                        (perProbability);
                tokens.get(perVecManagerSP.getOffset() - 1).setTagType(Constants.PERSON);
            }
            if (orgAnswer == 1) {
                tokens.get(orgVecManagerSP.getOffset() - 1).setPredictedProbability
                        (orgProbability);
                tokens.get(orgVecManagerSP.getOffset() - 1).setTagType(Constants.ORGANIZATION);
            }
            if (perAnswer == 1 && distanceFromStartOfPer == -1) {
                distanceFromStartOfOrg = -1;
                distanceFromStartOfPer = 1;
            } else if (perAnswer == 1) {
                distanceFromStartOfOrg = -1;
                distanceFromStartOfPer ++;
            } else if (perAnswer == -1 && distanceFromStartOfPer != -1) {
                distanceFromStartOfPer = -1;
            }
            if (orgAnswer == 1 && distanceFromStartOfOrg == -1) {
                distanceFromStartOfPer = -1;
                distanceFromStartOfOrg = 1;
            } else if (orgAnswer == 1) {
                distanceFromStartOfPer = -1;
                distanceFromStartOfOrg ++;
            } else if (orgAnswer == -1 && distanceFromStartOfOrg != -1) {
                distanceFromStartOfOrg = -1;
            }
        }
        
        return tokens;
    
    }
    
    
    
    
}
