/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc;

import ipl.util.Constants;
import ipl.core.Tokenizer;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.Auxiliary;
import java.io.*;
import libsvm.*;
import ipl.nerc.lists.ListManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;
import ipl.util.file_manager.FileManager;
import java.util.Hashtable;

/**
 * By using this class, you may call methods for testing trained svm models. Named entity categories suported,
 * are Constants.PERSON and Constants.ORGANIZATION (Constants.PER_ORG not supported directly) .
 *
 * @see  class Constants for constant values such as c and gamma svm parameters, paths to files e.t.c.
 */
public  class Tester  {
    
    
    /**
     * Constructor for Trainer class.
     *
     * @param args
     *      An array of String arguments for the classifier,
     *      read from the commmand line such as described in the following example:
     *      "-c [category] -m [margin]" eg -c PERSON -m 0.5
     */
    public Tester(){
        
        
    } // constructor: Trainer
    
    /*
     ***************************************************************************
     *_________________________________________________________________________*
     *_________________________________________________________________________*
     ******************************TESTING METHODS******************************
     *_________________________________________________________________________*
     *_________________________________________________________________________*
     ***************************************************************************
     */
    
    /**
     * This method is invoked to test the trained SVMs for both organizations
     * and persons on a test set.
     *
     * @param matcher
     *          A date and time matcher Object
     * @param sentSplitter A sentence splitter for periods.
     *
     * @param personModelFileName
     *          Path to file where SVM model(classifier)
     *          is saved for category PERSON
     * @param personListManager
     *          Manager for lists for category PERSON
     * @param recognizedPersonsOutputFile
     *          Name for the file where organization entity names
     *          for each testing file are saved. e.g. PERSONS.txt
     * @param orgModelFileName
     *          Path to file where SVM model(classifier)
     *          is saved for category  ORGANIZATION
     * @param orgListManager
     *          Manager for lists for category  ORGANIZATION
     * @param recognizedOrgsOutputFile
     *          Name for the file where person names for each
     *          testing file are saved. e.g. ORGS.txt
     * @param perNumOfVecs
     *          Number of training vectors used, for person category (needed just for typing results)
     * @param orgNumOfVecs
     *          Number of training vectors used, for organization category (needed just for typing results)
     * @param output
     *          An output stream to print results
     * @return Time needed to perform testing
     * @see ListManager
     * @see TimexMatcher
     * @see PeriodSeparator
     */
    public static void testFP(
            TimexMatcher matcher, SentenceSplitter sentSplitter,
            File[] testFilesset, String personModelFileName,
            ListManager personListManager, String personsRecognizedOutputFile, String orgModelFileName,
            ListManager orgListManager, String orgsRecognizedOutputFile,
            int perNumOfVecs, int orgNumOfVecs, PrintStream output)  {
        
        new File(personsRecognizedOutputFile).getParentFile().mkdirs();
        new File(orgsRecognizedOutputFile).getParentFile().mkdirs();
        
        long  start = System.currentTimeMillis();
        output.println("Testing . . . ");
        
        try {
            
            
            ////////////////////////////////////////////////////////////////////
            ////////////////////////confusion matrixes//////////////////////////
            int perConfMatrix[][] = new int[2][2];//confusion matrix for person category
            Auxiliary.initConfusionMatrix(perConfMatrix);
            int orgConfMatrix[][] = new int[2][2];//confusion matrix for organization category
            Auxiliary.initConfusionMatrix(orgConfMatrix);
            //======================confusion matrixes==============================
            //======================================================================
            
            // output stream for the file where person/organization
            // entity names are saved (for each testing file ).
            FileOutputStream perOut = new FileOutputStream(personsRecognizedOutputFile);
            FileOutputStream orgOut = new FileOutputStream(orgsRecognizedOutputFile);
            ////////////////////////////////////////////////////////////////////////
            /////////////////////loading classifiers (models)///////////////////////
            svm_model perModel = svm.svm_load_model(personModelFileName);
            svm_model orgModel = svm.svm_load_model(orgModelFileName);
            //====================loading classifiers (models)======================
            //======================================================================
            
            // loading number of classies (in our case it is two : "1" and "-1")
            int per_nr_class = svm.svm_get_nr_class(perModel);
            int org_nr_class = svm.svm_get_nr_class(orgModel);
            int perCategories[] = new int[per_nr_class];
            int orgCategories[] = new int[org_nr_class];
            ///////////////////////////////////////////////
            ///////////////////////////////////////////////
            ///////////////////////////////////////////////
            svm.svm_get_labels(perModel, perCategories);
            svm.svm_get_labels(orgModel, orgCategories);
            ///////////////////////////////////////////////
            ///////////////////////////////////////////////
            ///////////////////////////////////////////////
            double[] per_prob_estimates = new double[per_nr_class];
            double[] org_prob_estimates = new double[org_nr_class];
            
            // for every file of test set
            for( File curr : testFilesset) {
                
                if (curr.isFile()) {
                    
                    // arrays where values for features are kept
                    // in string representation
                    String perAttr[];
                    String orgAttr[];
                    // distancies from start of entities found. If -1 then it is the first
                    // (could be the only one) token of a sequence of tokens belonging to
                    // the same entity such as "A. Papanikolas"
                    int distanceFromStartOfPer = -1;
                    int distanceFromStartOfOrg = -1;
                    //writing names of files
                    perOut.write((curr.getName() + "\n").getBytes());
                    orgOut.write((curr.getName() + "\n").getBytes());
                    ////////////////////////////////////////////////////////////////
                    /////////////////text retrieval from this file//////////////////
                    FileInputStream fis = new FileInputStream(curr);
                    byte []buff = new byte[(int)curr.length()];
                    fis.read(buff);
                    fis.close();
                    //context of retrieved text
                    String text = new String(buff);
                    //===============text retrieval from this file==================
                    //==============================================================
                    
                    //feature vector managers for each category
                    PersonFeaturesManager perVecManager;
                    OrganizationFeaturesManager orgVecManager;
                    
                    // PERSON
                    perVecManager =
                            new PersonFeaturesManager(curr.getName(),
                            text, personListManager, matcher, sentSplitter);
                    
                    // ORGANIZATION
                    orgVecManager =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, orgListManager, matcher, sentSplitter);
                    
                    // for each token in text...
                    while (
                            ((perAttr = perVecManager.nextFeatVec(
                            distanceFromStartOfPer))!= null)&&
                            ((orgAttr = orgVecManager.nextFeatVec(
                            distanceFromStartOfOrg)) != null)
                            ) {
                        
                        double //predicted probabilities by svm
                                perAnswer, orgAnswer,
                                //category under examination in double representation
                                //(see class.method(...)--> Auxiliary.atof(...))
                                rightperAnswer, rightorgAnswer,
                                //canonical values calculated by predicted values
                                orgProbability,perProbability;
                        
                        ////////////////////////////////////////////////////////////
                        ///////////////assigning person probability/////////////////
                        
                        //if it isn't just the category returned - in other words everythimg went
                        //as we expected and we have now the feature-table for the next token
                        //with feature-values in string representation
                        if (perAttr.length != 1) {
                            
                            int numOfperAttributes = Constants.PER_FEAT_NUM_FP;
                            // an array of nodes, each node for each feature
                            svm_node node[] = new svm_node[numOfperAttributes];
                            // a double representing "PERSON" category (perAttr[0] equals always to category)
                            rightperAnswer  = Auxiliary.atof(perAttr[0]);
                            // assigning to each node the value of each feature
                            for (int i = 0; i < numOfperAttributes; i++) {
                                
                                node[i] = new svm_node();
                                node[i].index = i+1;
                                node[i].value = Auxiliary.atof(perAttr[i+1]);
                                
                            } // for
                            
                            
                            // PREDICT
                            perAnswer  = svm.svm_predict_probability(perModel, node,
                                    per_prob_estimates );
                            
                            // making probability in [-1.0,1.0] (canonical value of probability)
                            if (perCategories[0] == -1) {
                                perProbability = per_prob_estimates [1] - per_prob_estimates [0];
                            } // if
                            else {
                                perProbability = per_prob_estimates [0] - per_prob_estimates [1];
                            } // else
                        } // if
                        
                        else {// if excluded by safe rules
                            
                            rightperAnswer  = Auxiliary.atof(perAttr[0]);
                            perAnswer  = -1;
                            perProbability = -1;
                            
                        } // else
                        
                        ////////////////////////////////////////////////////////////
                        /////////////assigning organization probability/////////////
                        
                        //if it isn't just the category returned - in other words every went
                        //as we expected and we have now the feature-table for next token
                        //with values in string representation
                        if (orgAttr.length != 1) {
                            
                            int numOforgAttributes = Constants.ORG_FEAT_NUM_FP;
                            //an array of nodes, each node for each feature
                            svm_node node[] = new svm_node[numOforgAttributes];
                            //a double representing orgson (orgAttr[0] equals always to category)
                            rightorgAnswer  = Auxiliary.atof(orgAttr[0]);
                            
                            // assigning to each node the value of each feature
                            for (int i = 0; i < numOforgAttributes; i++) {
                                
                                node[i] = new svm_node();
                                node[i].index = i+1;
                                node[i].value = Auxiliary.atof(orgAttr[i+1]);
                                
                            } // for
                            
                            // PREDICT
                            orgAnswer  = svm.svm_predict_probability(orgModel, node,
                                    org_prob_estimates );
                            if (orgCategories[0] == -1) {
                                orgProbability = org_prob_estimates [1] - org_prob_estimates [0];
                            } // if
                            else {
                                orgProbability = org_prob_estimates [0] - org_prob_estimates [1];
                            } // else
                        } // if
                        else {
                            
                            rightorgAnswer  = Auxiliary.atof(orgAttr[0]);
                            orgAnswer  = -1;
                            orgProbability = -1;
                            
                        } // else
                        
                        ///======================================================///
                        ///==============assigning probabilities=================///
                        //========================================================//
                        
                        
                        ////////////////////////////////////////////////////////////
                        /////comparing probabilities to find categories answers/////
                        
                        //if there is assigned a possitive canonical probabillity
                        //and the other one a negative probability, then we decide that it
                        //belongs to the "possitive category"
                        if ((perProbability > 0)&&( orgProbability < 0)) {
                            
                            perAnswer = 1.0;
                            orgAnswer = -1.0;
                            
                        } else if ((orgProbability > 0)&&(perProbability < 0)) {
                            
                            orgAnswer = 1.0;
                            perAnswer = -1.0;
                            ////////////////////////////////////////////////////////////
                        }
                        
                        //in case they are both positive:
                        else if (orgProbability > perProbability && perProbability >=0) {
                            
                            orgAnswer = 1.0;
                            perAnswer = -1.0;
                            
                        } else if (perProbability > orgProbability && orgProbability >=0) {
                            
                            orgAnswer = -1.0;
                            perAnswer = 1.0;
                            
                        }
                        // if both negative then the answer is no (-1) for both
                        else if ((orgProbability < 0)&&(perProbability < 0)) {
                            
                            orgAnswer = -1.0;
                            perAnswer = -1.0;
                            
                        }// in case they are not both negative and they are equal to eachother
                        else if (perProbability == orgProbability) {
                            // random decision
                            if ((Math.ceil(Math.random()) * 10) % 2 == 0){
                                
                                perAnswer = -1.0;
                                orgAnswer = 1.0;
                                
                            }else {
                                
                                orgAnswer = -1.0;
                                perAnswer = 1.0;
                                
                            }
                            
                        }
                        
                        ////////////////////////////////////////////////////////////
                        //////////////////updating confusion matrixies//////////////
                        /////////////// and possibilities assigned by svms//////////
                        perVecManager.tokenAt(
                                perVecManager.getOffset(
                                )-1).setPredictedProbability(perProbability);
                        
                        if(perAnswer==1)perVecManager.tokenAt(perVecManager.getOffset()-1).setTagType(Constants.PERSON);
                        
                        Auxiliary.updateConfusionMatrix(perConfMatrix,
                                rightperAnswer , perAnswer, output );
                        
                        orgVecManager.tokenAt(
                                orgVecManager.getOffset(
                                )-1).setPredictedProbability(orgProbability);
                        
                        if(orgAnswer==1) orgVecManager.tokenAt(orgVecManager.getOffset()-1).setTagType(Constants.ORGANIZATION);
                        
                        Auxiliary.updateConfusionMatrix(orgConfMatrix,
                                rightorgAnswer , orgAnswer , output);
                        
                        // checking for a sequence of tokens of the same person entity
                        if ((perAnswer  == 1.0) && (distanceFromStartOfPer == -1)) {
                            
                            // in case we found one category then the other's index
                            // from start of entitty must be ensured to -1
                            distanceFromStartOfOrg = -1;
                            distanceFromStartOfPer = 1;
                            String currTok =
                                    perVecManager.tokenAt(
                                    perVecManager.getOffset()-1).getToken();
                            perOut.write((currTok + " ").getBytes());
                            
                        } // if
                        
                        //in case it is categorized as person, but we have also found
                        //other tokens before it that belong to the same entity...
                        else if (perAnswer  == 1) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfOrg = -1;
                            distanceFromStartOfPer++;
                            String currTok = perVecManager.tokenAt(
                                    perVecManager.getOffset() - 1).getToken();
                            perOut.write((currTok + " ").getBytes());
                            
                        } // else if
                        
                        //if it is not categorized as person and there a sequense of
                        //tokens of a person entity before it
                        else if ((perAnswer  == -1) &&
                                (distanceFromStartOfPer != -1)) {
                            
                            distanceFromStartOfPer = -1;
                            perOut.write("\n".getBytes());//change line in file
                            
                        } // else if
                        /////////////////////////////////////////////////////////////
                        
                        
                        // checking for a sequence of tokens of the same organization entity
                        if ((orgAnswer  == 1) && (distanceFromStartOfOrg == -1)) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfPer = -1;
                            distanceFromStartOfOrg = 1;
                            String currTok = orgVecManager.tokenAt(
                                    orgVecManager.getOffset() - 1).getToken();
                            orgOut.write((currTok + " ").getBytes());
                            
                        } // if
                        
                        //in case it is categorized as organization, but we have also found
                        //other tokens before it that belong to the same entity...
                        else if (orgAnswer  == 1) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfPer = -1;
                            distanceFromStartOfOrg++;
                            String currTok = orgVecManager.tokenAt(
                                    orgVecManager.getOffset() - 1).getToken();
                            orgOut.write((currTok + " ").getBytes());
                            
                        } // else if
                        
                        //if it is not categorized as organization
                        else if ((orgAnswer  == -1) &&
                                (distanceFromStartOfOrg != -1)) {
                            
                            distanceFromStartOfOrg = -1;
                            orgOut.write("\n".getBytes());
                            
                        } // else if
                        
                    } // while-->for all tokens in text
                    
                    Tokenizer tok = perVecManager.getTokens();
                    // current test file annotated by classifiers
                    File outputAnnotatedFile = new File(
                            Constants.ENTITIES_RECONIZED_DIR_PATH + "/annotated_data",
                            curr.getName()
                            );
                    FileManager.write(tok, outputAnnotatedFile.getCanonicalPath());
                    
                    orgVecManager = null;
                    orgOut.write("\n".getBytes());
                    
                    perVecManager = null;
                    perOut.write("\n".getBytes());
                    
                } // if
                
                
            } // while--> for all files in testing set
            
            output.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
            
            
            ////////////////////////////////////////////////////////////////////////
            ///////////////////////results printing/////////////////////////////////
            output.println("\n\n"+
                    "________________________________________________");
            output.println(Constants.PERSON+" results\n");
            output.println(Constants.PERSON+" training vectors = "+perNumOfVecs);
            Auxiliary.printConfusionMatrix(perConfMatrix,output);
            Auxiliary.printPrecisionRecall(
                    Auxiliary.computePrecisionRecall(perConfMatrix), perConfMatrix.length,
                    output);
            
            output.println("\n\n"+
                    "________________________________________________");
            output.println(Constants.ORGANIZATION+" results\n");
            output.println(Constants.ORGANIZATION+" training vectors = "+orgNumOfVecs);
            Auxiliary.printConfusionMatrix(orgConfMatrix,output);
            Auxiliary.printPrecisionRecall(
                    Auxiliary.computePrecisionRecall(orgConfMatrix), orgConfMatrix.length,
                    output);
            
            
            perOut.close();
            orgOut.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        
    }// testFP()
    
    
    public static void testFP(
            TimexMatcher matcher, SentenceSplitter sentSplitter,
            String testFilesPath, String personModelFileName,
            ListManager personListManager, String personsRecognizedOutputFile, String orgModelFileName,
            ListManager orgListManager, String orgsRecognizedOutputFile,
            int perNumOfVecs, int orgNumOfVecs, PrintStream output)  {
        
        File testDir = new File(testFilesPath);
        if (!testDir.isDirectory()) return ;
        else testFP(matcher, sentSplitter, testDir.listFiles(), personModelFileName,
                personListManager, personsRecognizedOutputFile, orgModelFileName, orgListManager,
                orgsRecognizedOutputFile, perNumOfVecs, orgNumOfVecs, output);
        
        
    }
    
    /**
     * Same as TestFP, this time for second pass.
     * @see method TestFP
     */
    public static void testSP(
            TimexMatcher matcher, SentenceSplitter sentSplitter,
            File[] testFilesset, String personModelFileNameFP, ListManager personListManagerFP,
            String personModelFileNameSP, ListManager personListManagerSP, String personsRecognizedOutputFile,
            String orgModelFileNameFP, ListManager orgListManagerFP, String orgModelFileNameSP,
            ListManager orgListManagerSP, String orgsRecognizedOutputFile,
            int perNumOfVecs, int orgNumOfVecs, PrintStream output)  {
        
        
        new File(personsRecognizedOutputFile).getParentFile().mkdirs();
        new File(orgsRecognizedOutputFile).getParentFile().mkdirs();
        
        long start = System.currentTimeMillis();
        output.println("Testing . . . ");
        
        try {
            
            
            ////////////////////////////////////////////////////////////////////
            ////////////////////////confusion matrixes//////////////////////////
            int perConfMatrix[][] = new int[2][2];//confusion matrix for person category
            Auxiliary.initConfusionMatrix(perConfMatrix);
            int orgConfMatrix[][] = new int[2][2];//confusion matrix for organization category
            Auxiliary.initConfusionMatrix(orgConfMatrix);
            //======================confusion matrixes==============================
            //======================================================================
            
            // output stream for the file where person/organization
            // entity names found are saved .
            FileOutputStream perOut = new FileOutputStream(personsRecognizedOutputFile);
            FileOutputStream orgOut = new FileOutputStream(orgsRecognizedOutputFile);
            
            ////////////////////////////////////////////////////////////////////////
            /////////////////////loading classifiers (models)///////////////////////
            svm_model perModelFP = svm.svm_load_model(personModelFileNameFP);
            svm_model orgModelFP = svm.svm_load_model(orgModelFileNameFP);
            svm_model perModelSP = svm.svm_load_model(personModelFileNameSP);
            svm_model orgModelSP = svm.svm_load_model(orgModelFileNameSP);
            //====================loading classifiers (models)======================
            //======================================================================
            
            // loading number of classies (in our case it is two : "1" and "-1")
            int per_nr_class = svm.svm_get_nr_class(perModelSP);
            int org_nr_class = svm.svm_get_nr_class(orgModelSP);
            int perCategories[] = new int[per_nr_class];
            int orgCategories[] = new int[org_nr_class];
            ///////////////////////////////////////////////
            svm.svm_get_labels(perModelSP, perCategories);
            svm.svm_get_labels(orgModelSP, orgCategories);
            ///////////////////////////////////////////////
            double[] per_prob_estimates = new double[per_nr_class];
            double[] org_prob_estimates = new double[org_nr_class];
            
            // for every file of test set
            for (File curr : testFilesset) {
                
                if (curr.isFile()) {
                    
                    // arrays where values for features are kept
                    // in string representation
                    String SPPerAttr[];
                    String SPOrgAttr[];
                    // distancies from start of entities found. If -1 then it is the first
                    // (could be the only one) token of a sequence of tokens belonging to
                    // the same entity such as "A. Papanikolas"
                    int distanceFromStartOfPer = -1;
                    int distanceFromStartOfOrg = -1;
                    //writing names of files
                    perOut.write((curr.getName() + "\n").getBytes());
                    orgOut.write((curr.getName() + "\n").getBytes());
                    
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////text retrieval from this file//////////////////
                    FileInputStream fis = new FileInputStream(curr);
                    byte []buff = new byte[(int)curr.length()];
                    fis.read(buff);
                    fis.close();
                    //context of retrieved text
                    String text = new String(buff);
                    //===============text retrieval from this file==================
                    //==============================================================
                    
                    
                    
                    // Second and First Pass feature vector managers for each category
                    PersonFeaturesManager perVecManagerSP, perVecManagerFP;
                    OrganizationFeaturesManager orgVecManagerSP, orgVecManagerFP;
                    
                    // PERSON
                    perVecManagerSP =
                            new PersonFeaturesManager(curr.getName(),
                            text, personListManagerSP, matcher, sentSplitter);
                    perVecManagerFP =
                            new PersonFeaturesManager(curr.getName(),
                            text, personListManagerFP, matcher, sentSplitter);
                    
                    // ORGANIZATION
                    orgVecManagerSP =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, orgListManagerSP, matcher, sentSplitter);
                    orgVecManagerFP =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, orgListManagerFP, matcher, sentSplitter);
                    
                    // getting sure decisions list and all decisions cannonical posibilities from fp classifiers
                    Hashtable []FPDecisions =
                            Trainer.findFPDecisionsForSP(perVecManagerFP, perModelFP, orgVecManagerFP, orgModelFP, Constants.SURE_DEC_MARGIN, curr.getName());
                    
                    Hashtable <String, String[]> sureDec = (Hashtable <String, String[]> ) FPDecisions[0];
                    Hashtable <String, String> allDec = (Hashtable <String, String> ) FPDecisions[1];
                    
                    
                    // for each token in text...
                    while (
                            ((SPPerAttr = perVecManagerSP.nextFeatVec(
                            distanceFromStartOfPer, sureDec, allDec, curr.getName()))!= null) &&
                            
                            ((SPOrgAttr = orgVecManagerSP.nextFeatVec(
                            distanceFromStartOfOrg, sureDec, allDec, curr.getName())) != null)
                            ) {
                        
                        double //predicted probabilities by svm
                                perAnswer, orgAnswer,
                                //category under examination in double representation
                                //(see class.method(...)--> Auxiliary.atof(...))
                                rightperAnswer, rightorgAnswer,
                                //canonical values calculated by predicted values
                                orgProbability, perProbability;
                        
                        ////////////////////////////////////////////////////////////
                        ///////////////assigning person probability/////////////////
                        
                        //if it isn't just the category returned - in other words everythimg went
                        //as we expected and we have now the feature-table for the next token
                        //with feature-values in string representation
                        if (SPPerAttr.length != 1) {
                            
                            int numOfperAttributes = Constants.PER_FEAT_NUM_SP;
                            // an array of nodes, each nodes for each feature
                            svm_node nodes[] = new svm_node[numOfperAttributes];
                            // a double representing "PERSON" category (SPPerAttr[0] equals always to category)
                            rightperAnswer  = Auxiliary.atof(SPPerAttr[0]);
                            // assigning to each nodes the value of each feature
                            for (int i = 0; i < numOfperAttributes; i++) {
                                
                                nodes[i] = new svm_node();
                                nodes[i].index = i+1;
                                nodes[i].value = Auxiliary.atof(SPPerAttr[i+1]);
                                
                            } // for
                            
                            
                            // PREDICT
                            perAnswer  = svm.svm_predict_probability(perModelSP, nodes,
                                    per_prob_estimates );
                            
                            // making probability in [-1.0,1.0] (canonical value of probability)
                            if (perCategories[0] == -1) {
                                perProbability = per_prob_estimates [1] - per_prob_estimates [0];
                            } // if
                            else {
                                perProbability = per_prob_estimates [0] - per_prob_estimates [1];
                            } // else
                            
                        } // if
                        
                        else {// if excluded by safe rules
                            
                            rightperAnswer  = Auxiliary.atof(SPPerAttr[0]);
                            perAnswer  = -1;
                            perProbability = -1;
                            
                        } // else
                        
                        ////////////////////////////////////////////////////////////
                        /////////////assigning organization probability/////////////
                        
                        //if it isn't just the category returned - in other words every went
                        //as we expected and we have now the feature-table for next token
                        //with values in string representation
                        if (SPOrgAttr.length != 1) {
                            
                            int numOforgAttributes = Constants.ORG_FEAT_NUM_SP;
                            //an array of nodes, each nodes for each feature
                            svm_node nodes[] = new svm_node[numOforgAttributes];
                            //a double representing orgson (SPOrgAttr[0] equals always to category)
                            rightorgAnswer  = Auxiliary.atof(SPOrgAttr[0]);
                            
                            // assigning to each nodes the value of each feature
                            for (int i = 0; i < numOforgAttributes; i++) {
                                
                                nodes[i] = new svm_node();
                                nodes[i].index = i+1;
                                nodes[i].value = Auxiliary.atof(SPOrgAttr[i+1]);
                                
                            } // for
                            
                            // PREDICT
                            orgAnswer  = svm.svm_predict_probability(orgModelSP, nodes,
                                    org_prob_estimates );
                            if (orgCategories[0] == -1) {
                                orgProbability = org_prob_estimates [1] - org_prob_estimates [0];
                            } // if
                            else {
                                orgProbability = org_prob_estimates [0] - org_prob_estimates [1];
                            } // else
                            
                        } // if
                        else {
                            
                            rightorgAnswer  = Auxiliary.atof(SPOrgAttr[0]);
                            orgAnswer  = -1;
                            orgProbability = -1;
                            
                        } // else
                        
                        ///======================================================///
                        ///==============assigning probabilities=================///
                        //========================================================//
                        
                        
                        ////////////////////////////////////////////////////////////
                        /////comparing probabilities to find categories answers/////
                        
                        //if there is assigned a possitive canonical probabillity
                        //and the other one a negative probability, then we decide that it
                        //belongs to the "possitive category"
                        if ((perProbability > 0) && ( orgProbability < 0)) {
                            
                            perAnswer = 1.0;
                            orgAnswer = -1.0;
                            
                        } else if ((orgProbability > 0) && (perProbability < 0)) {
                            
                            orgAnswer = 1.0;
                            perAnswer = -1.0;
                            ////////////////////////////////////////////////////////////
                        }
                        
                        //in case they are both positive:
                        else if (orgProbability > perProbability && perProbability >=0) {
                            
                            orgAnswer = 1.0;
                            perAnswer = -1.0;
                            
                        } else if (perProbability > orgProbability && orgProbability >=0) {
                            
                            orgAnswer = -1.0;
                            perAnswer = 1.0;
                            
                        }
                        // if both negative then the answer is no (-1) for both
                        else if ((orgProbability < 0)&&(perProbability < 0)) {
                            
                            orgAnswer = -1.0;
                            perAnswer = -1.0;
                            
                        }// in case they are not both negative and they are equal to eachother
                        else if (perProbability==orgProbability) {
                            // random decision
                            if ((Math.ceil(Math.random()) * 10) % 2 == 0){
                                
                                perAnswer = -1.0;
                                orgAnswer = 1.0;
                                
                            }else {
                                
                                orgAnswer = -1.0;
                                perAnswer = 1.0;
                                
                            }
                            
                        }
                        
                        ////////////////////////////////////////////////////////////
                        //////////////////updating confusion matrixies//////////////
                        /////////////// and possibilities assigned by svms//////////
                        
                        perVecManagerSP.tokenAt(
                                perVecManagerSP.getOffset(
                                )-1).setPredictedProbability(perProbability);
                        
                        if(perAnswer==1) perVecManagerSP.tokenAt(perVecManagerSP.getOffset()-1).setTagType(Constants.PERSON);
                        
                        Auxiliary.updateConfusionMatrix(perConfMatrix,
                                rightperAnswer , perAnswer, output);
                        
                        orgVecManagerSP.tokenAt(
                                orgVecManagerSP.getOffset(
                                )-1).setPredictedProbability(orgProbability);
                        
                        if(orgAnswer==1) orgVecManagerSP.tokenAt(orgVecManagerSP.getOffset()-1).setTagType(Constants.ORGANIZATION);
                        
                        Auxiliary.updateConfusionMatrix(orgConfMatrix,
                                rightorgAnswer , orgAnswer , output);
                        
                        // checking for a sequence of tokens of the same person entity
                        if ((perAnswer  == 1.0) && (distanceFromStartOfPer == -1)) {
                            
                            // in case we found one category then the other's index
                            // from start of entitty must be ensured to -1
                            distanceFromStartOfOrg = -1;
                            distanceFromStartOfPer = 1;
                            String currTok =
                                    perVecManagerSP.tokenAt(
                                    perVecManagerSP.getOffset()-1).getToken();
                            perOut.write((currTok + " ").getBytes());
                            
                        } // if
                        
                        //in case it is categorized as person, but we have also found
                        //other tokens before it that belong to the same entity...
                        else if (perAnswer  == 1) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfOrg = -1;
                            distanceFromStartOfPer++;
                            String currTok = perVecManagerSP.tokenAt(
                                    perVecManagerSP.getOffset() - 1).getToken();
                            perOut.write((currTok + " ").getBytes());
                            
                        } // else if
                        
                        //if it is not categorized as person and there a sequense of
                        //tokens of a person entity before it
                        else if ((perAnswer  == -1) &&
                                (distanceFromStartOfPer != -1)) {
                            
                            distanceFromStartOfPer = -1;
                            perOut.write("\n".getBytes());//change line in file
                            
                        } // else if
                        /////////////////////////////////////////////////////////////
                        
                        
                        // checking for a sequence of tokens of the same organization entity
                        if ((orgAnswer  == 1) && (distanceFromStartOfOrg == -1)) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfPer = -1;
                            distanceFromStartOfOrg = 1;
                            String currTok = orgVecManagerSP.tokenAt(
                                    orgVecManagerSP.getOffset() - 1).getToken();
                            orgOut.write((currTok + " ").getBytes());
                            
                        } // if
                        
                        //in case it is categorized as organization, but we have also found
                        //other tokens before it that belong to the same entity...
                        else if (orgAnswer  == 1) {
                            
                            //in case we found one category then the other's index
                            //from start of entitty must be ensured to -1
                            distanceFromStartOfPer = -1;
                            distanceFromStartOfOrg++;
                            String currTok = orgVecManagerSP.tokenAt(
                                    orgVecManagerSP.getOffset() - 1).getToken();
                            orgOut.write((currTok + " ").getBytes());
                            
                        } // else if
                        
                        //if it is not categorized as organization
                        else if ((orgAnswer  == -1) &&
                                (distanceFromStartOfOrg != -1)) {
                            
                            distanceFromStartOfOrg = -1;
                            orgOut.write("\n".getBytes());
                            
                        } // else if
                        
                    } // while-->for all tokens in text
                    
                    Tokenizer tok = perVecManagerSP.getTokens();
                    // current test file annotated by classifiers
                    File outputAnnotatedFile =
                            new File(Constants.ENTITIES_RECONIZED_DIR_PATH + "/annotated_data",
                            curr.getName()
                            );
                    FileManager.write(tok, outputAnnotatedFile.getCanonicalPath());
                    
                    orgVecManagerSP = null;
                    orgOut.write("\n".getBytes());
                    
                    perVecManagerSP = null;
                    perOut.write("\n".getBytes());
                    
                    
                } // if
                
                
            } // while--> for all files in testing set
            
            output.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
            
            
            ////////////////////////////////////////////////////////////////////////
            ///////////////////////results printing/////////////////////////////////
            output.println("\n\n"+
                    "________________________________________________");
            output.println(Constants.PERSON + " results\n");
            output.println(Constants.PERSON + " training vectors = "+perNumOfVecs);
            Auxiliary.printConfusionMatrix(perConfMatrix,output);
            Auxiliary.printPrecisionRecall(
                    Auxiliary.computePrecisionRecall(perConfMatrix), perConfMatrix.length,
                    output);
            
            output.println("\n\n"+
                    "________________________________________________");
            output.println(Constants.ORGANIZATION + " results\n");
            output.println(Constants.ORGANIZATION + " training vectors = " + orgNumOfVecs);
            Auxiliary.printConfusionMatrix(orgConfMatrix, output);
            Auxiliary.printPrecisionRecall(Auxiliary.computePrecisionRecall(orgConfMatrix), orgConfMatrix.length, output);
            
            perOut.close();
            orgOut.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        
    }// testSP()
    
    public static void testSP(
            TimexMatcher matcher, SentenceSplitter sentSplitter,
            String testFilesPath, String personModelFileNameFP, ListManager personListManagerFP,
            String personModelFileNameSP, ListManager personListManagerSP, String personsRecognizedOutputFile,
            String orgModelFileNameFP, ListManager orgListManagerFP, String orgModelFileNameSP,
            ListManager orgListManagerSP, String orgsRecognizedOutputFile,
            int perNumOfVecs, int orgNumOfVecs, PrintStream output)  {
        
        File testDir = new File(testFilesPath);
        if (!testDir.isDirectory()) return ;
        else  testSP(matcher,sentSplitter, testDir.listFiles(), personModelFileNameFP,
                personListManagerFP, personModelFileNameSP, personListManagerSP, personsRecognizedOutputFile,
                orgModelFileNameFP, orgListManagerFP, orgModelFileNameSP, orgListManagerSP, orgsRecognizedOutputFile, 
                perNumOfVecs, orgNumOfVecs, output);
        
        
        
    }
    
} // end of class Trainer
