/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc;

import ipl.util.NativeTrainer;
import ipl.util.Constants;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import ipl.util.NativeTuner;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.file_manager.LibSVMFileManager;
import ipl.util.Auxiliary;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Arrays;
import libsvm.*;
import ipl.nerc.lists.ListManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * By using this class, you may call methods for training an svm model. Named entity categories suported,
 * are Constants.PERSON and Constants.ORGANIZATION (Constants.PER_ORG not supported directly) .
 *
 * @see  class Constants for constant values such as c and gamma svm parameters, paths to files e.t.c.
 */
public  class Trainer  {
    
    protected TimexMatcher matcher;
    /**Period separator for separating periods in texts*/
    protected SentenceSplitter separator;
    
    
    /**
     * Constructor for Trainer class.
     *
     * @param args
     *      An array of String arguments for the classifier,
     *      read from the commmand line such as described in the following example:
     *      "-c [category] -m [margin]" eg -c PERSON -m 0.5
     */
    public Trainer(){
        
        matcher = new TimexMatcher(Constants.TIMEX_PATERNS_FILE_PATH);
        separator = new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH);
        // path needed for training vectors files. Vectors saved in this file are given to native train method
        new File(Constants.FP_PERSON_TRAIN_VECTORS_FILE).getParentFile().mkdirs();
        
    } // constructor: Trainer
    
    /**
     *
     *
     *
     *
     * @return the date and time paterns matcher used by this Trainer
     */
    public TimexMatcher getMatcher() {
        return matcher;
    }
    
    /**
     * Sets the date and time paterns matcher used by this Trainer
     */
    public void setMatcher(String pathToMatcher) {
        matcher = new TimexMatcher(pathToMatcher);
    }
    
    /**
     * @return the Period Separator used by this Trainer
     */
    public SentenceSplitter getSeparator() {
        return separator;
    }
    
    /**
     * Sets the Period Separator used by this Trainer
     */
    public void setSeparator(String pathToSeparator) {
        separator = new SentenceSplitter(pathToSeparator);
    }// PLSPExperiment()
    
    
    /*
     ***************************************************************************
     *_________________________________________________________________________*
     *_________________________________________________________________________*
     *****************************TRAINING METHODS******************************
     *_________________________________________________________________________*
     *_________________________________________________________________________*
     ***************************************************************************
     */
    
       /**
     * Performs training of this classifier for passive learning first pass.
     *
     * @param trainPoolDir
     *          training files
     * @param modelFilePath
     *          Path to file where trained classifier is loaded and saved
     * @param listManager
     *          ListManager object for managing frequency token-lists
     * @param PrintStream output
     *          A printstream to print results and messages
     *
     * @return The number of training vectors used.
     */
    public int trainPLFP(String category, File[] trainFiles, String modelFilePath,
            ListManager listManager, PrintStream output) {
        
        long start;
        String vectorsFile = (category.equals(Constants.PERSON))?
            Constants.FP_PERSON_TRAIN_VECTORS_FILE:
            Constants.FP_ORGANIZATION_TRAIN_VECTORS_FILE;
        
        int numOfTrainingVectors = 0;
        
        start = System.currentTimeMillis();
        
        LibSVMFileManager outputFileManager =
                new LibSVMFileManager(vectorsFile);
        
        for (File curr : trainFiles) {
            
            if (curr.isFile()) {
                
                String attr[];
                int distanceFromStartOfEntity = -1;
                
               
                FileInputStream fis = null;
                try {
                    fis = new FileInputStream(curr);
                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                }
                byte []buff = new byte[ (int) curr.length()];
                try {
                    fis.read(buff);
                    fis.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
                String text = new String(buff);
                FeaturesManager mav;
                // PERSON
                if (category.compareTo(Constants.PERSON) == 0) {
                    mav = new PersonFeaturesManager(curr.getName(),text, listManager, this.getMatcher(), this.getSeparator());
                } // if
                // ORGANIZATION
                else if (category.compareTo(Constants.ORGANIZATION) == 0) {
                    mav = new OrganizationFeaturesManager(curr.getName(),text, listManager, this.getMatcher(), this.getSeparator());
                } // else if
                
                else {
                    mav = null;
                    output.println("Invalid category");
                    System.exit(1);
                } // else
                
                if(category.equalsIgnoreCase(Constants.PERSON)){
                    attr = new String[Constants.PER_FEAT_NUM_FP];
                }else {//(category.equalsIgnoreCase(Constants.ORGANIZATION))
                    attr = new String[Constants.ORG_FEAT_NUM_FP];
                }
                
                
                
                while ( (attr = mav.nextFeatVec(
                        distanceFromStartOfEntity)) != null) {
                    if (attr.length != 1) {
                        numOfTrainingVectors++;
                        outputFileManager.setData(attr);
                    } // if
                    else if (attr[0].compareTo("1") == 0) {
                        output.println(curr.getName());
                        output.println(mav.tokenAt(mav.getOffset()
                        - 1).getToken());
                    } // else if
                    
                    if ((attr[0].compareTo("1") == 0) &&
                            (distanceFromStartOfEntity == -1)) {
                        
                        distanceFromStartOfEntity = 1;
                    } // if
                    else if (attr[0].compareTo("1") == 0) {
                        distanceFromStartOfEntity++;
                    } // else if
                    else if ((attr[0].compareTo("-1") == 0) &&
                            (distanceFromStartOfEntity != -1)) {
                        
                        distanceFromStartOfEntity = -1;
                    } // else if
                } // while
            } // if
        } // while
        outputFileManager.closeFile();
        
        // retrieving vectors from file
        File file = new File(vectorsFile);
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(file);
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        }
        byte buff[] = new byte[(int) file.length()];
        try {
            fis.read(buff);
            fis.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        String temp = new String(buff);
        
        // finding number of features in each vector
        String temp2 = temp.substring(0, temp.indexOf("\n"));
        StringTokenizer strtok = new StringTokenizer(temp2, " ");
        int max_index = strtok.countTokens() - 1;
        strtok = null;
        
        output.println("training " + category +  "...");
        start = System.currentTimeMillis();
        
        // calling native
        double
                cFP = (category.equalsIgnoreCase(Constants.PERSON)) ? Constants.PER_C_FP : Constants.ORG_C_FP,
                gammaFP = (category.equalsIgnoreCase(Constants.PERSON)) ? Constants.PER_GAMMA_FP : Constants.ORG_GAMMA_FP;
        NativeTrainer.train(
                vectorsFile.toCharArray(), vectorsFile.toCharArray().length,
                modelFilePath.toCharArray(), modelFilePath.toCharArray().length,
                max_index, true, cFP, gammaFP);
        output.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
        
        
        return numOfTrainingVectors;
        
    } // trainPLFP(...)
    
    
    /**
     * Method used for training SVM classifier model, for first pass active learning.
     *
     *
     * @param pathModelFile
     *          Path to file where classifier model is loaded and will be saved again after being retrained.
     *
     * @param fpAmbVecsFilePath
     *          Path to file where active learning training vectors, classified by user, are saved.
     *
     * @param output
     *          A Printstream to print results.
     *
     * @return The number of training vectors used.
     *
     */
    public int trainALFP(String category, String pathModelFile, String fpAmbVecsFilePath, PrintStream output) {
        
        
        long start, end;
        // maximum number of features values in each line
        int max_index = 0;
        // number of training vectors (number of lines)
        int numOfTrainingVectors = 0;
        /*
         * training vectors file. A default file where all training vectors ar saved
         * eg Constants.FP_PERSON_TRAIN_VECTORS_FILE.
         */
        String trainVecsPath = "";
        // vectors from initial (created by passive learning) classifier
        String startTrainVectorsPath = "";
        if(category.equalsIgnoreCase(Constants.PERSON)){
            
            trainVecsPath = Constants.FP_PERSON_TRAIN_VECTORS_FILE;
            startTrainVectorsPath = Constants.FP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
            
        } else if(category.equalsIgnoreCase(Constants.ORGANIZATION)){
            
            trainVecsPath = Constants.FP_ORGANIZATION_TRAIN_VECTORS_FILE;
            startTrainVectorsPath = Constants.FP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
            
        }
        
        try {
            
            /////////////////reading fpAmbVecsFilePath file//////////////////////
            File file = new File(fpAmbVecsFilePath);
            FileInputStream fis = new FileInputStream(file);
            byte buff[] = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            // context of fpAmbVecsFilePath file
            String temp = new String(buff);
            // tokenizes to lines
            StringTokenizer strtok = new StringTokenizer(temp, "\n");
            // couting lines => counting number of feature vectors
            numOfTrainingVectors = strtok.countTokens();
            strtok = null;
            //==============reading fpAmbVecsFilePath file=======================
            
            
            String temp2 = temp.substring(0, temp.indexOf("\n"));
            strtok = new StringTokenizer(temp2, " ");
            // counting number of feature values
            max_index = strtok.countTokens() - 1;
            strtok = null;
            
            ////////////////reading startTrainVectorsPath file////////////////////
            File fnew = new File(startTrainVectorsPath);
            FileInputStream fisnew = new FileInputStream(fnew);
            buff = new byte[ (int) fnew.length()];
            fisnew.read(buff);
            fisnew.close();
            String newtemp = new String(buff);
            strtok = new StringTokenizer(newtemp, "\n");
            newtemp = newtemp.trim();
            
            ////////////////////////////////////////////////////////////////////////////////////////////
            //////// concatenating startTrainVectorsPath and fpAmbVecsFilePath to trainVecsPath ////////
            FileOutputStream fos = new FileOutputStream(trainVecsPath);
            fos.write( (temp.concat(newtemp)).getBytes());
            fos.close();
            numOfTrainingVectors += strtok.countTokens();
            strtok = null;
            
        } //try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
        
        output.println("training...");
        start = System.currentTimeMillis();
        // calling native method for training SVM
        double
                cFP = (category.equalsIgnoreCase(Constants.PERSON)) ? Constants.PER_C_FP : Constants.ORG_C_FP,
                gammaFP = (category.equalsIgnoreCase(Constants.PERSON)) ? Constants.PER_GAMMA_FP : Constants.ORG_GAMMA_FP;
        
        NativeTrainer.train(
                trainVecsPath.toCharArray(),
                trainVecsPath.toCharArray().length,
                pathModelFile.toCharArray(),
                pathModelFile.toCharArray().length,
                max_index, true, cFP, gammaFP);
        
        output.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
        
        return numOfTrainingVectors;
        
        
    } //trainALFP(...)
    
    
    /**
     * Finds decisions made by first pass classifiers, for all tokens in file saved in path "trainFilePath".
     * The hashtable array returned, contains two hashtables: <br>
     * - The first Hashtable contains the "sure decisions" and <br>
     * - The second one contains all decisions from all texts.
     *
     * More Details:<br>
     * 1.In each text, first pass classifiers can be sure enought for some classification decisions they make.
     * For each text, an array of those surelly classified tokens is used in order to know if a token found
     * in a text, was also found again in the same text (in a deferent position), and first pass classifiers
     * where sure enought about classifing it. A <b>sure decision</b>, is a decision for which it's cannonical
     * posibility is grater than the margin parameter (you may use the margin value stored in static variable
     * Constants.SURE_DEC_THRESHOLD = 0.8).<br>
     * 2.Cannonical possibilities of each classification decision, are saved in a Hashtable<String, String>.
     * Strings such as [category] + "#" + trainFilePath + "#" + position Of token text, are used as keys for cannonical possibilities
     * (a double value converted to String format) .
     *
     *
     * @param FPVectorsMangr
     *          A FeaturesManager Object for finding first pass feature values
     * @param model
     *          An svm_model object which represents the model (classifier used) as an object.
     * @param margin
     *          A margin used to define which decisions made by model, are considered sure (must be in [-1,1])
     * @param trainFilePath
     *          Path to training file to find decision lists
     * @param category
     *          Named entity category (see Constants.Organization and Constants.Person)
     * @return The array of the 2 hashtables.
     *              <br> At position 0,  sure decisions Hashtable<String, String[]>.
     *              <br> At position 1, all decisions canonical possibilities Hashtable<String, String>.
     */
    public static Hashtable[] findFPDecisionsForSP(FeaturesManager FPVectorsMangr, svm_model model,
            double margin, String trainFilePath, String category){
        
        try {
            
            if (margin < -1 || margin>1) {
                throw new Exception("Margin for sure decisions out of bounds. " +
                        "Margin \"m\" must be less than 1 and grater than 0");
            }
            
        } catch (Exception e){
            
            e.printStackTrace();
            
            System.err.println("Margin redefined as: " + Constants.SURE_DEC_MARGIN);
            margin = Constants.SURE_DEC_MARGIN;
            
        }
        // will include all sure decisions (canonical posibility > margin)
        // key = [category] + "#" + trainFilePath
        Hashtable<String, String[]> sureDec = new Hashtable<String, String[]>(1, 1.0f) ;
        // key = [category] + "#" + trainFilePath + "#" + positionOfToken
        Hashtable<String, String> allDec = new Hashtable<String, String>(FPVectorsMangr.getTokens().size(), 0.95f) ;
        
        int distanceFromStartOfEntity = -1;
        String[] attributes;
        
        int nr_class = svm.svm_get_nr_class(model);
        int []categories = new int[nr_class];
        svm.svm_get_labels(model, categories);
        double[] prob_estimates = new double[nr_class];
        
        // temporary for creating sure list
        String temp[] = new String[1024];
        int listNum = 0;
        
        
        // constucting array of features of FP
        if(category.equalsIgnoreCase(Constants.PERSON)){
            
            attributes = new String[Constants.PER_FEAT_NUM_FP];
            
        }else {//if(category.equalsIgnoreCase(Constants.ORGANIZATION)){
            
            attributes = new String[Constants.ORG_FEAT_NUM_FP];
            
        }
        
        while ( (attributes = FPVectorsMangr.nextFeatVec(
                distanceFromStartOfEntity)) != null) {
            
            int tokPos = FPVectorsMangr.getOffset() -1;
            
            // answer  and canonical probanility (-1,1) for FP decisions
            double answer, canonicalProbability;
            
            if (attributes.length != 1) {
                
                int numOfAttributes = attributes.length - 1;
                svm_node node[] = new svm_node[numOfAttributes];
                
                for (int i = 1; i <= numOfAttributes; i++) {
                    node[i - 1] = new svm_node();
                    node[i - 1].index = i;
                    node[i - 1].value = Auxiliary.atof(attributes[i]);
                } // for
                
                // PREDICT
                answer = svm.svm_predict_probability(model, node,
                        prob_estimates);
                
                if (categories[0] == -1) {
                    canonicalProbability = prob_estimates[1] - prob_estimates[0];
                } // if
                
                else {
                    canonicalProbability = prob_estimates[0] - prob_estimates[1];
                } // else
                
            } // if
            
            else {// cat by safe rules
                answer = -1;
                canonicalProbability = -1;
            } // else
            
            FPVectorsMangr.tokenAt(tokPos).setPredictedProbability(canonicalProbability);
            
            allDec.put(category + "#" + trainFilePath + "#" + tokPos, String.valueOf(canonicalProbability));
            
            if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
                
                distanceFromStartOfEntity = 1;
                String currTok = FPVectorsMangr.tokenAt(tokPos).getToken();
                
                if (currTok.length() >= 3 && canonicalProbability > margin){
                    temp[listNum++] = currTok.toLowerCase();
                } // if
                
            } // if
            else if (answer == 1) {
                
                distanceFromStartOfEntity++;
                String currTok = FPVectorsMangr.tokenAt(
                        FPVectorsMangr.getOffset() - 1).getToken();
                
                if (currTok.length() >= 3 && canonicalProbability > margin){
                    temp[listNum++] = currTok.toLowerCase();
                } // if
                
            } // else if
            
            else if (answer == -1){
                distanceFromStartOfEntity = -1;
            } // else if
            
        } // while
        
        
        // sorting firtst pass list
        String []list = Auxiliary.getSubList(temp, listNum);
        Arrays.sort(list);
        sureDec.put( category + "#" + trainFilePath, list );
        
        // to be returned
        Hashtable []toReturn = new Hashtable[2];
        toReturn[0] = sureDec;
        toReturn[1] = allDec;
        
        return toReturn;
        
    }// findFPDecisionsForSP
    
    /**
     * The same method as described in findFPDecisionsForSP(..), overloaded such as to be used for two
     * categories at the same time.
     */
    public static Hashtable[] findFPDecisionsForSP(
            FeaturesManager FPPersonVectorsMangr, svm_model personModel,
            FeaturesManager FPOrganizationVectorsMangr, svm_model organizationModel,
            double margin, String fileName){
        
        
        Hashtable []perDecisions = findFPDecisionsForSP(FPPersonVectorsMangr, personModel, margin, fileName, Constants.PERSON);
        Hashtable<String, String[]> sureDec = perDecisions[0];
        Hashtable<String, String> allDec = perDecisions[1];
        
        Hashtable []orgDecisions = findFPDecisionsForSP(FPOrganizationVectorsMangr, organizationModel, margin, fileName, Constants.ORGANIZATION);
        
        
        ////////// concatenating person and organization decisions and sure decisions to hashtables to be returned ////////////////
        Enumeration<String> orgSureKeys = (Enumeration<String>) orgDecisions[0].keys();
        while(orgSureKeys.hasMoreElements()){
            
            String key = orgSureKeys.nextElement();
            sureDec.put(key, (String[])orgDecisions[0].get(key));
            
        }
        
        Enumeration<String> orgAllKeys = (Enumeration<String>)orgDecisions[1].keys();
        while(orgAllKeys.hasMoreElements()){
            
            String key = orgAllKeys.nextElement();
            allDec.put(key, (String)orgDecisions[1].get(key));
            
        }
        
        return new Hashtable[] {sureDec, allDec};
        
    }
    
    /**
     * Performs training of this Classifier for second pass (passive learning only, Person-organization category).
     * Contrary to the other method used, this method does not load decisions from first pass, it finds them. So
     * more time is needed because of finding first pass decisions again.
     *
     * @param categ
     *          Category used in order to check if this method should be used. If category is not
     *          Person_Organization, then a CategoryNotSupportedException is thrown and null is returned
     * @param trainFiles
     *          training files
     * @param match
     *          A TimexMatcher object for date and time entities in texts
     * @param perSep
     *          A PeriodSeparator object for finding periods in texts
     * @param FPPerModelFilePath
     *          Path to file where the first pass training model is loaded for Person category
     * @param FPPerListMangr
     *          ListManager object for first pass and person category
     * @param FPOrgModelFilePath
     *          Path to file where the first pass training model is loaded for Organozation category
     * @param FPOrgListMangr
     *          ListManager object for first pass and person category
     * @param SPPerModelFilePath
     *          Path to file where the second pass training model is saved for Person category
     * @param SPPerListMangr
     *          ListManager object for second pass and person category
     * @param SPOrgModelFilePath
     *          Path to file where the second pass training model is saved for Organozation category
     * @param SPOrgListMangr
     *          ListManager object for second pass and person category
     * @param resultsOutput
     *          A printstream to print results and messages
     *
     * @return An array containing the number of training vectors of each category in each position (0:person, 1:organization).
     *
     */
    public int[] trainPLSP(
            String categ, File[] trainFiles, TimexMatcher match, SentenceSplitter perSep,
            String FPPerModelFilePath, ListManager FPPerListMangr, String FPOrgModelFilePath, ListManager FPOrgListMangr,
            String SPPerModelPath, ListManager SPPerListMangr, String SPOrgModelPath, ListManager SPOrgListMangr,
            PrintStream resultsOutput)  {
        
        try {
            
            long start, end;// time counting
            String perVectorsFile = Constants.SP_PERSON_TRAIN_VECTORS_FILE ;
            String orgVectorsFile = Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE;
            
            int perTrainVecsNum =0, orgTrainVecsNum = 0;
            
            start = System.currentTimeMillis();
            
            LibSVMFileManager outputPerFileManager = new LibSVMFileManager(perVectorsFile);
            LibSVMFileManager outputOrgFileManager = new LibSVMFileManager(orgVectorsFile);
            
            svm_model perModelFP = svm.svm_load_model(FPPerModelFilePath);
            int fp_per_nr_class = svm.svm_get_nr_class(perModelFP);
            int []fp_per_categories = new int[fp_per_nr_class];
            svm.svm_get_labels(perModelFP, fp_per_categories);
            double[] fp_per_prob_estimates = new double[fp_per_nr_class];
            
            svm_model orgModelFP = svm.svm_load_model(FPOrgModelFilePath);
            int fp_org_nr_class = svm.svm_get_nr_class(orgModelFP);
            int []fp_org_categories = new int[fp_org_nr_class];
            svm.svm_get_labels(orgModelFP, fp_org_categories);
            double[] fp_org_prob_estimates = new double[fp_org_nr_class];
            
            
            // for all files in second pass training set
            for (File curr : trainFiles) {
                
                if (curr.isFile()) {
                    
                    String fileName = curr.getName();
                    String perAttr[];
                    int perDistFromStartOfEntity = -1;
                    
                    String orgAttr[];
                    int orgDistFromStartOfEntity = -1;
                    
                    // extracting text
                    FileInputStream fis = new FileInputStream(curr);
                    byte []buff = new byte[ (int) curr.length()];
                    fis.read(buff);
                    fis.close();
                    String text = new String(buff);
                    
                    // managers for First Pass features
                    FeaturesManager FPPerVecMangr;
                    FeaturesManager FPOrgVecMangr;
                    // managers for Second Pass features
                    FeaturesManager SPPerVecMangr;
                    FeaturesManager SPOrgVecMangr;
                    
                    
                    
                    FPPerVecMangr =
                            new PersonFeaturesManager(curr.getName(),
                            text, FPPerListMangr,
                            match, perSep);
                    
                    FPOrgVecMangr =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, FPOrgListMangr,
                            match, perSep);
                    
                    SPPerVecMangr = new PersonFeaturesManager(curr.getName(),
                            text, SPPerListMangr,
                            match, perSep);
                    
                    SPOrgVecMangr = new OrganizationFeaturesManager(curr.getName(),
                            text, SPOrgListMangr,
                            match, perSep);
                    
                    
                    
                    // getting sure decisions list and all decisions cannonical posibilities from fp classifier
                    Hashtable []FPdecisions = findFPDecisionsForSP(FPPerVecMangr, perModelFP, FPOrgVecMangr, orgModelFP, Constants.SURE_DEC_MARGIN, fileName);
                    
                    Hashtable<String, String[]> sureDec = FPdecisions[0];
                    Hashtable<String, String> allDec = FPdecisions[1];
                    
                    perDistFromStartOfEntity = -1;
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////// finding second pass decisions //////////////////
                    ////////////////////////////////////////////////////////////////
                    
                    // for all tokens in text
                    while (
                            (perAttr = SPPerVecMangr.nextFeatVec(
                            perDistFromStartOfEntity, sureDec, allDec, fileName) )!= null
                            
                            &&
                            
                            (orgAttr = SPOrgVecMangr.nextFeatVec(
                            orgDistFromStartOfEntity, sureDec, allDec, fileName)) != null
                            
                            ) {
                        
                        //////////////////////////////// person /////////////////////////////////
                        if (perAttr.length != 1) {
                            
                            perTrainVecsNum++;
                            outputPerFileManager.setData(perAttr);
                            
                        } // if
                        
                        if ((perAttr[0].compareTo("1") == 0) && (
                                perDistFromStartOfEntity == -1)) {
                            
                            perDistFromStartOfEntity = 1;
                            
                        } // if
                        else if (perAttr[0].compareTo("1") == 0) {
                            
                            perDistFromStartOfEntity++;
                            
                        } // else if
                        else if ((perAttr[0].compareTo("-1") == 0) && (perDistFromStartOfEntity != -1)) {
                            
                            perDistFromStartOfEntity = -1;
                            
                        } // else if
                        
                        
                        /////////////////////////// organization /////////////////////////////////
                        if (orgAttr.length != 1) {
                            
                            orgTrainVecsNum++;
                            outputOrgFileManager.setData(orgAttr);
                            
                        } // if
                        
                        if ((orgAttr[0].compareTo("1") == 0) && (
                                orgDistFromStartOfEntity == -1)) {
                            
                            orgDistFromStartOfEntity = 1;
                        } // if
                        else if (orgAttr[0].compareTo("1") == 0) {
                            orgDistFromStartOfEntity++;
                        } // else if
                        else if ((orgAttr[0].compareTo("-1") == 0) && (orgDistFromStartOfEntity != -1)) {
                            
                            orgDistFromStartOfEntity = -1;
                            
                        } // else if
                        
                        
                        
                    } // while
                    
                    FPPerVecMangr = null;
                    SPPerVecMangr = null;
                    FPOrgVecMangr = null;
                    SPOrgVecMangr = null;
                    
                } // if
                
            } // while
            
            outputPerFileManager.closeFile();
            outputOrgFileManager.closeFile();
            
            ////////////
            // TRAIN2 //
            ////////////
            
            //////////////////////////////////////////////////////////////////////////////
            ///////////////////////////////// person /////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            
            // read train file to load number of features (see max_index)
            FileInputStream fis = new FileInputStream(perVectorsFile);
            byte buff[] = new byte[2512];
            fis.read(buff);
            fis.close();
            String temp = new String(buff);
            temp = temp.substring(0, temp.indexOf("\n"));
            StringTokenizer strtok = new StringTokenizer(temp, " ");
            int max_index = strtok.countTokens() - 1;
            
            resultsOutput.println("Training Person...");
            start = System.currentTimeMillis();
            NativeTrainer.train(
                    perVectorsFile.toCharArray(),
                    perVectorsFile.toCharArray().length,
                    SPPerModelPath.toCharArray(),
                    SPPerModelPath.toCharArray().length,
                    max_index, true, Constants.PER_C_SP, Constants.PER_GAMMA_SP);
            resultsOutput.println( "DONE, TIME NEEDED: " + Constants.toTimeString(start));
            
            
            //////////////////////////////////////////////////////////////////////////////
            //////////////////////////// organization ////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            // read train file to load number of features (see max_index)
            fis = new FileInputStream(orgVectorsFile);
            buff = new byte[2512];
            fis.read(buff);
            fis.close();
            temp = new String(buff);
            temp = temp.substring(0, temp.indexOf("\n"));
            strtok = new StringTokenizer(temp, " ");
            max_index = strtok.countTokens() - 1;
            
            resultsOutput.println("Training Organization...");
            start = System.currentTimeMillis();
            NativeTrainer.train(
                    orgVectorsFile.toCharArray(),
                    orgVectorsFile.toCharArray().length,
                    SPOrgModelPath.toCharArray(),
                    SPOrgModelPath.toCharArray().length,
                    max_index, true, Constants.ORG_C_SP, Constants.ORG_GAMMA_SP);
            resultsOutput.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
            
            return new int[] {perTrainVecsNum, orgTrainVecsNum};
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return null;
        
    } // trainPLSP(...)
    
    
   
    
    /**
     * Performs training of this Classifier for second pass (active learning only, categ = Person_Organizaqtion)
     *
     * @param categ,
     * @param PerAmbiguousVecsPath
     * @param spOrgAmbiguousVecsPath
     * @param SPPerModelPath
     * @param SPOrgModelPath
     * @param resultsOutput
     *
     *
     * @return An array containing the number of training vectors of each category in each position (0:person, 1:organization).
     */
    public int[] trainALSP(
            String categ,
            String spPerAmbiguousVecsPath, String spOrgAmbiguousVecsPath,
            String SPPerModelPath, String SPOrgModelPath,
            PrintStream resultsOutput)  {
        
        long start = System.currentTimeMillis();
        
        try {
            
            String trainVecsPath = Constants.SP_PERSON_TRAIN_VECTORS_FILE;
            String startTrainVectorsPath = Constants.SP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
            
            /////////////////////////////////////////////////////////////////////////////////////
            ///////////////////// preparing SP person active learning training vectors /////////////////////////
            /////////////////////////////////////////////////////////////////////////////////////
            
            /////////////////reading spPerAmbiguousVecsPath file/////////////////////////////////
            File file = new File(spPerAmbiguousVecsPath);
            FileInputStream fis = new FileInputStream(file);
            byte buff[] = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            // context of spOrgAmbiguousVecsPath file
            String temp = new String(buff);
            // tokenizes to lines
            StringTokenizer strtok = new StringTokenizer(temp, "\n");
            // couting lines => counting number of feature vectors
            int numOfPerTrainingVectors = strtok.countTokens();
            strtok = null;
            //==============reading spPerAmbiguousVecsPath file=======================
            
            String temp2 = temp.substring(0, temp.indexOf("\n"));
            strtok = new StringTokenizer(temp2, " ");
            // counting number of feature values
            int per_max_index = strtok.countTokens() - 1;
            strtok = null;
            
            ////////////////reading startTrainVectorsPath file////////////////////
            File fnew = new File(startTrainVectorsPath);
            FileInputStream fisnew = new FileInputStream(fnew);
            buff = new byte[ (int) fnew.length()];
            fisnew.read(buff);
            fisnew.close();
            String newtemp = new String(buff);
            strtok = new StringTokenizer(newtemp, "\n");
            newtemp = newtemp.trim();
            numOfPerTrainingVectors += strtok.countTokens();
            
            /////////////////////////////////////////////////////////////////////////////////////////////////
            //////// concatenating startTrainVectorsPath and spPerAmbiguousVecsPath to trainVecsPath ////////
            FileOutputStream fos = new FileOutputStream(trainVecsPath);
            fos.write( (temp.concat(newtemp)).getBytes());
            fos.close();
            strtok = null;
            
            /////////////////////////////////////////////////////////////////////////////////////
            /////////////////// preparing SP Organization active learning training vectors /////////////////////
            /////////////////////////////////////////////////////////////////////////////////////
            trainVecsPath = Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE;
            startTrainVectorsPath = Constants.SP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
            /////////////////reading spOrgAmbiguousVecsPath file//////////////////////
            file = new File(spOrgAmbiguousVecsPath);
            fis = new FileInputStream(file);
            buff = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            // context of spOrgAmbiguousVecsPath file
            temp = new String(buff);
            // tokenizes to lines
            strtok = new StringTokenizer(temp, "\n");
            // couting lines => counting number of feature vectors
            int numOfOrgTrainingVectors = strtok.countTokens();
            strtok = null;
            //==============reading spOrgAmbiguousVecsPath file=======================
            
            temp2 = temp.substring(0, temp.indexOf("\n"));
            strtok = new StringTokenizer(temp2, " ");
            // counting number of feature values
            int org_max_index = strtok.countTokens() - 1;
            strtok = null;
            
            ////////////////reading startTrainVectorsPath file////////////////////
            fnew = new File(startTrainVectorsPath);
            fisnew = new FileInputStream(fnew);
            buff = new byte[ (int) fnew.length()];
            fisnew.read(buff);
            fisnew.close();
            newtemp = new String(buff);
            strtok = new StringTokenizer(newtemp, "\n");
            newtemp = newtemp.trim();
            numOfOrgTrainingVectors += strtok.countTokens();
            
            /////////////////////////////////////////////////////////////////////////////////////////////////
            //////// concatenating startTrainVectorsPath and spOrgAmbiguousVecsPath to trainVecsPath ////////
            fos = new FileOutputStream(trainVecsPath);
            fos.write((temp.concat(newtemp)).getBytes());
            fos.close();
            strtok = null;
            ////////////////////////// active learning training vectors files for Sp ready ////////////////////////////////
            //==============================================================================================
            
            //////////////////////////////////////////////////////////////////////////////
            ///////////////////////////////// person /////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            long startPer = System.currentTimeMillis();
            resultsOutput.println("Trainind person...");
            NativeTrainer.train(
                    Constants.SP_PERSON_TRAIN_VECTORS_FILE.toCharArray(),
                    Constants.SP_PERSON_TRAIN_VECTORS_FILE.toCharArray().length,
                    SPPerModelPath.toCharArray(),
                    SPPerModelPath.toCharArray().length,
                    per_max_index, true,
                    Constants.PER_C_SP,
                    Constants.PER_GAMMA_SP);
            resultsOutput.println( "DONE, TIME NEEDED: " + Constants.toTimeString(startPer));
            
            //////////////////////////////////////////////////////////////////////////////
            //////////////////////////// organization ////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            long startOrg = System.currentTimeMillis();
            resultsOutput.println("Trainind organization...");
            NativeTrainer.train(
                    Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE.toCharArray(),
                    Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE.toCharArray().length,
                    SPOrgModelPath.toCharArray(),
                    SPOrgModelPath.toCharArray().length,
                    org_max_index, true, Constants.ORG_C_SP,
                    Constants.ORG_GAMMA_SP);
            resultsOutput.println( "DONE, TIME NEEDED: " + Constants.toTimeString(startOrg));
            
            System.out.println( "Time summuary for training categories PERSON and ORGANIZATION: " + Constants.toTimeString(start));
            return new int[] {numOfPerTrainingVectors, numOfOrgTrainingVectors};
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return null;
        
    } // trainPLSP(...)
    
    
} // end of class Trainer
