/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning;

import ipl.util.Constants;
import ipl.nerc.active_learning.data.UsedVector;
import ipl.nerc.preprocessing.TextPreprocessor;
import java.util.Hashtable;
import java.util.StringTokenizer;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;
import java.util.ArrayList;
import ipl.nerc.active_learning.data.AmbiguousVector;
import ipl.nerc.active_learning.data.PoolHandler;
import java.io.*;
import ipl.nerc.lists.ListManager;
import ipl.util.Auxiliary;
import ipl.core.Tokenizer;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import ipl.util.file_manager.DirectoryManager;
import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;

public abstract class ActiveLearning {
    
    /**
     *          
     *  .       :
     * "__"  "42nea@e=A&f=17031&m=N24&aa=1.xml"
     * (  isUsedVector(String fname, int pos)) .
     */
    protected Hashtable<String, UsedVector> usedVectors;
    
    /**
     * Pool of candidate AmbiguousVector Objects. We fill in each experiment
     * this pool and then we we choose which of them are going to be placed in the
     * final batch (see field batch[])
     */
    protected ArrayList<AmbiguousVector> ambVecPool;
    
    /**
     *         
     * ,         token
     */
    protected AmbiguousVector[] batch;
    
    protected ListManager perListManager;
    
    protected ListManager orgListManager;
    
    /**
     * M  .       
     *         (batch).     
     *   batchSize       
     * active learning .
     */
    protected int batchSize;
    
    /** */
    protected TimexMatcher matcher;
    
    /** */
    protected SentenceSplitter separator;
    
    /**
     * Flag for using no method for performing active learning.
     * Useful for computing average similarity of pool
     */
    public static int NONE = 0;
    
    /**
     * Flag for using informativeness method for performing active learning
     */
    public static int INFORMATIVENESS = 1;
    
    /**
     * Flag for using informativeness method and then local diversity on batch,
     * for performing active learning
     */
    public static int INFO_LOCDIVER = 2;
    
    /**
     * Flag for using informativeness and representiveness methods and then
     * local diversity on batch, for performing active learning
     */
    public static int INFO_REPR_LOCDIVER = 3;
    
    /**
     * Flag for using informativeness and representiveness methods and then
     * global diversity on batch, for performing active learning
     */
    public static int INFO_LOC_GLOBDIVER = 4;
    
    /**
     * Flag for computing pool of all ambiguous vectors and then excluding
     * those already used.
     *
     * @see method findAndGetBatch
     */
    public static int FIND = 1;
    
    /**
     * Flag for computing pool of all ambiguous vectors and then excluding
     * those already used. The computed pool will be saved for future use.
     *
     * @see method findAndGetBatch
     */
    public static int FIND_SAVE = 2;
    
    /**
     * Flag for computing pool of all ambiguous vectors and then excluding
     * those already used. The computed pool will be saved for future use.
     *
     * @see method findAndGetBatch
     */
    public static int LOAD_SAVE = 3;
    
    /**
     *       
     *   .          
     *            
     *  
     *
     * @param usedVectorsFileName
     *              
     *         
     * @param timexPatternsFileName
     *            (patterns)
     *      
     * @param periodSeparatorModelFileName
     *               
     *        (      )
     * @param listsPath
     *             
     * @param batchSize
     *         
     */
    public ActiveLearning(
            String timexPatternsFileName,
            String periodSeparatorModelFileName,
            int batchSize) {
        
        this.batchSize = batchSize;
        
        this.matcher = new TimexMatcher(timexPatternsFileName);
        this.separator = new SentenceSplitter(periodSeparatorModelFileName);
        ambVecPool = new ArrayList(batchSize);
        
        
    } // ActiveLearning
    
    /**
     * Loads to field usedVector, all vector nodes used in previous active learning experimernts.
     * Those "used vectors" are saved after each experiment in file with path
     * this.usedVectorsFileName. The reason we use this method, is to avoid
     * retraining the svm with the same AmbiguousVector Objects more than once.
     *              pool
     *
     * @param fileName
     *      The path to the file with used vectors
     *
     * @return  A HashTable containing UsedVectorNode Objects, using the String tokens
     *          as keys.
     */
    protected Hashtable<String, UsedVector> loadUsedVectors(String fileName) {
        
        Hashtable<String, UsedVector> usedVectors =
                new Hashtable<String,UsedVector>();
        
        try {
            
            File f = new File(fileName);
            if(!f.exists()){
                
                f.getParentFile().mkdirs();
                f.createNewFile();
                
            }
            FileInputStream fis = new FileInputStream(f);
            byte[] buff = new byte[ (int) f.length()];
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text =
                    TextPreprocessor.removeNoisySpaces(
                    text);
            StringTokenizer strTok = new StringTokenizer(text, " \n");
            int c=0;
            while (strTok.hasMoreTokens()) {
                
                String fname = strTok.nextToken(); // System.out.print((++c) + ": Used Vector filename: " + fname + ", @");
                int pos = Auxiliary.atoi(strTok.nextToken());// System.out.println("pos: " + pos );
                UsedVector temp = new UsedVector(fname, pos);
                String key = Integer.toString(pos) + fname;
                usedVectors.put(key, temp);
                
            } // while
            
            strTok = null;
        }catch (Exception e) {
            e.printStackTrace();
        } // catch
        System.out.println("Number of used vectors loaded: " + usedVectors.size());
        
        return usedVectors;
        
    } // loadUsedVectors
    
    /**
     *           pos 
     *  fname     usedVectors (   )
     *
     * @param fname
     *              
     * @param pos
     *          
     *
     * @returns true         
     *   false
     *
     */
    protected boolean isUsedVector(String fname, int pos) {
        
        String key = Integer.toString(pos) + fname;
        if (getUsedVectors().containsKey(key)) {
            return true;
        } // if
        else {
            return false;
        } // else
        
    } // isUsedVector
    
    /**
     *      batch   AmbiguousVector    
     *  (    svm        batch)
     *       informativeness .
     *          batch   -
     *         .
     *
     * :
     *             batch  arrayList,  
     *     -batch
     *
     * @param distance
     *          SVM         
     *
     *
     * @returns     batch  -1   
     *
     */
    protected int getPosition(double distance) {
        
        double max = 0;
        int pos = -1;
        for (int i = 0; i < batchSize; i++) {
            if (getAmbVecPool().get(i).getMinDistanceHyperplane() > max) {
                max = getAmbVecPool().get(i).getMinDistanceHyperplane();
                pos = i;
            } // if
        } // for
        
        if (distance < max) {
            return pos;
        } // if
        else {
            return -1;
        } // else
        
    } // getPosition
    
    /**
     * Creares and returns an ambiguous vector node (an AmbiguousVector object)
     *
     * @param tok
     *      Tokenizer containing separated tokens of text
     * @param atrr
     *      A String array containing values of attributes
     * @param offset
     *      Index to where we point at(which token) now in text
     * @param fileName
     *      Name of the file containing the next token
     * @param distance
     *      Distance from the svm hyperplane
     * @return an AmbiguousVector object according to parameters given
     *
     */
    protected AmbiguousVector constructAmbiguousVector(
            Tokenizer tok,String attr[],int offset,
            String fileName, double distance) {
        
        int toksize = tok.size();
        
        String []attrVector = attr;
        // +/- 12 tokens from the current one
        final int vicinity = 12;
        int start, end, pos = offset;
        // position in phrase returned
        int posInPhrase;
        
        if (pos - vicinity >= 0) {
            
            start = pos - vicinity;
            posInPhrase = vicinity;
            
        } else{
            
            start = 0;
            posInPhrase = pos;
            
        }
        
        
        
        if (pos + vicinity >= toksize) end = toksize - 1;
        else end = pos + vicinity;
        
        int length = end - start + 1;
        String phrase[] = new String[length];
        for (int i = 0; i < length; i++)
            phrase[i] = tok.get(start + i).getToken();
        
        // retaining previous tokens (usefull for lists)
        final int NumOfPreviousWords = 7;
        String previousWords[] = new String[NumOfPreviousWords];
        for (int i = 0; i < NumOfPreviousWords; i++)
            previousWords[i] = "";
        int j;
        pos = offset;
        if ((pos - NumOfPreviousWords) > 0)  j = pos - NumOfPreviousWords;
        else j = 0;
        
        int z = pos - 1;
        int i = 0;
        while ((z >= j) && (!tok.get(z).getEndOfPeriod()))
            previousWords[i++] = tok.get(z--).getToken().toLowerCase();
        // the one to be returned
        AmbiguousVector node =
                new AmbiguousVector(phrase, posInPhrase, tok.get(offset),
                fileName, previousWords,attrVector, offset, Math.abs(distance));
        
        return node;
        
    } // getNode
    
    /**
     * This method is used when having multiple categories to perfom active learning
     * experiment. Creares and returns an ambiguous vector node (an AmbiguousVector object).
     * Attribute values contained in the returned node, are those of person followed by
     * those of organization in the same String array.
     *
     * @param tok
     *      Tokenizer containing separated tokens of text
     * @param perAtrr
     *      A String array containing values of attributes for category = PERSON
     * @param orgAtrr
     *      A String array containing values of attributes for category = ORGANIZATION
     * @param offset
     *      Index to where we point at(which token) now in text
     * @param fileName
     *      Name of the file containing the next token
     * @param perDistance
     *      Distance from the svm hyperplane for category = PERSON
     * @param orgDistance
     *      Distance from the svm hyperplane for category = ORGANIZATION
     *
     * @return an AmbiguousVector object according to parameters given
     *
     */
    protected AmbiguousVector constructAmbiguousVector(
            Tokenizer tok, String perAttr[], String orgAttr[], int offset,
            String fileName, double perDistance, double orgDistance,
            boolean okByPersonSafeRules, boolean okByOrganiationSafeRules, String pass) {
        
        int toksize = tok.size();
        
        ////////////////////////////////////////////////////////////////////////
        // we will use a single array to store all attribute values for both
        // categories. Firstly for peson attribute values and then for organization
        
        
        
        /////////////////////////////////////////////////////////////////////////////////
        ///concatenating feature values of both categories in one feature String array///
        /////////we include the categories in this array before feature-values///////////
        ///////////////Categories are in attributesConcat[0]=person and//////////////////
        //////////////at attributesConcat[perAttr.length] = organization/////////////////
        /////////////////////////////////////////////////////////////////////////////////
        String []attributesConcat = null;
        
        if (pass.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            attributesConcat =
                    new String[Constants.PER_FEAT_NUM_FP + Constants.ORG_FEAT_NUM_FP + 2];
            for (int i = 0; i < Constants.PER_FEAT_NUM_FP + 1; i++ ){
                
                attributesConcat[i] = perAttr[i];
                
            }
            for (int i = 0; i < Constants.ORG_FEAT_NUM_FP + 1; i++ ){
                
                attributesConcat[i + Constants.PER_FEAT_NUM_FP + 1] = orgAttr[i];
                
            }
            
        }//fp
        else {//if(pass.equalsIgnoreCase(Constants.SECOND_PASS)){
            
            attributesConcat =
                    new String[Constants.PER_FEAT_NUM_SP + Constants.ORG_FEAT_NUM_SP + 2];
            for (int i = 0; i < Constants.PER_FEAT_NUM_SP + 1; i++ ){
                
                attributesConcat[i] = perAttr[i];
                
            }
            for (int i = 0; i < Constants.ORG_FEAT_NUM_SP + 1; i++ ){
                
                attributesConcat[i + Constants.PER_FEAT_NUM_SP + 1] = orgAttr[i];
                
            }
            
        }//sp
        
        //===concatenating feature value of both categories in one array========
        //===we do not include the categories in this array contrary to=========
        //======================================================================
        // +/- 15 tokens from the current one
        final int vicinity = 15;
        int start, end, pos = offset;
        // position in phrase returned
        int posInPhrase;
        
        if (pos - vicinity >= 0) {
            
            start = pos - vicinity;
            posInPhrase = vicinity;
            
        } else{
            
            start = 0;
            posInPhrase = pos;
            
        }
        
        if (pos + vicinity >= toksize) end = toksize - 1;
        else end = pos + vicinity;
        
        int length = end - start + 1;
        String phrase[] = new String[length];
        for (int i = 0; i < length; i++)
            phrase[i] = tok.get(start + i).getToken();
        
        // retaining previous tokens (usefull for lists)
        final int NumOfPreviousWords = 7;
        String previousWords[] = new String[NumOfPreviousWords];
        for (int i = 0; i < NumOfPreviousWords; i++)
            previousWords[i] = "";
        int j;
        pos = offset;
        if ((pos - NumOfPreviousWords) > 0)  j = pos - NumOfPreviousWords;
        else j = 0;
        
        int z = pos - 1;
        int i = 0;
        while ((z >= j) && (!tok.get(z).getEndOfPeriod()))
            previousWords[i++] = tok.get(z--).getToken().toLowerCase();
        
        // the one to be returned
        AmbiguousVector node =
                new AmbiguousVector(phrase, posInPhrase, tok.get(offset),
                fileName, previousWords, attributesConcat, offset,
                Math.abs(perDistance), Math.abs(orgDistance),
                okByPersonSafeRules, okByOrganiationSafeRules);
        
        return node;
        
    } // getAmbiguousNode
    
    /**
     * This method is used for finding ambiguous vectors. A batch of this.bachSize
     * vectors is returned as an array of AmbiguousVector objects. Vectors of batch
     * are selected according to method specified by parameter howFlag.
     *
     *
     * @param poolDir
     *      path to directory where active learning files are stored
     * @param perModelFileName
     *      (pass null value if category is not person
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.PERSON is saved .
     * @param orgModelFileName
     *      (pass null value if category is not organization
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.ORGANIZATION is saved .
     * @param methodFlag
     *      A flag indicating which active learning method to use.
     * @param category
     *      Category of the named entities eg Constants.PER_ORG
     *
     * @param computeAll
     *      A flag used in case of loading a former batch from a file. If computeAll
     *      is false, then distances of vectors from hyperplanes are computed only
     *      for the first this.batchSize * Math.log10(poolSize) vectors of the loaded pool.
     *
     * @see class Constants for default paths and values
     *
     */
    public abstract AmbiguousVector[] findAndGetBatch(
            String usedVectorsFileName, String poolDir,
            String perModelFileName, String orgModelFileName,
            String category, int methodFlag,
            boolean computeAll) throws Exception ;
    
    /**
     * Computes Pool of vectors from text files and saves it as a file, using an Object Stream.
     * Pool is saved as an ArrayList<AmbiguousVector> object File in directory "dirToSave".
     * Vectors included in batch are excluded from pool.
     *
     * @param actLeFilesDir
     *      Path to directory of text files of pool
     *
     * @param dirToSave
     *      Directory to save object file of pool
     *
     * @param expNum
     *      An integer indicating the number of current experiment. This number is
     *      used as the file name of the saved pool.
     *
     * @param perInputModelFileName
     *      Path to file of SVM model for category PERSON
     *
     * @param orgInputModelFileName
     *      Path to file of SVM model for category ORGANIZATION
     *
     * @param methodFlag
     *      flag for method to use for computing batch (eg ActiveLearning.INFORMATIVENESS)
     *
     * @caution
     *      Do not use this method if lists are updated after pool being saved.
     *      Values for features depending on lists are not recomputed. Lists must not
     *      change after saving pool. Alternatevly, if lists are updated, then pool must
     *      be recomputed by calling again method findAndSavePersonOrganization() or findPersonOrganization().
     *
     */
    public AmbiguousVector[] randomBatchFP(
            String actLeFilesDir, File dirToSave, int expNum, String perInputModelFileName,
            String orgInputModelFileName) throws IOException {
        
        
        // pool containing all candidate ambiguous vector nodes
        ambVecPool = new ArrayList<AmbiguousVector>();
        
        // counter for nodes
        int count = 0;
        File curr;
        DirectoryManager poolManager = new DirectoryManager(actLeFilesDir);
        
        svm_model perModel = svm.svm_load_model(perInputModelFileName);
        svm_model orgModel = svm.svm_load_model(orgInputModelFileName);
        
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        
        int []perCategories = new int[per_nr_class];
        int []orgCategories = new int[org_nr_class];
        
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        
        while ( (curr = poolManager.getNextFile()) != null && curr.isFile()) {
            
            String perAttr[];
            int perDistanceFromStartOfEntity = -1;
            
            String orgAttr[];
            int orgDistanceFromStartOfEntity = -1;
            
            ////////////////////////////////////////////////////////////////
            ////////////////reading next training text//////////////////////
            // buffer for next training text file
            byte []buff = new byte[ (int) curr.length()];
            
            try {
                FileInputStream fis = new FileInputStream(curr);
                fis.read(buff);
                fis.close();
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
            String text = new String(buff);
            ////////////////reading next training text//////////////////////
            //==============================================================
            
            FeaturesManager perVectorManager;
            FeaturesManager orgVectorManager;
            
            perVectorManager = new PersonFeaturesManager(curr.getName(),text, getPerListManager(), matcher, separator);
            orgVectorManager = new OrganizationFeaturesManager(curr.getName(),text, getOrgListManager(), matcher, separator);
            
            Tokenizer perTok = perVectorManager.getTokens();
            Tokenizer orgTok = orgVectorManager.getTokens();
            
            fill_pool_loop_label:
                while (// for each available feature vector in text
                        (perAttr = perVectorManager.nextFeatVec(
                        perDistanceFromStartOfEntity)) != null &&
                        (orgAttr = orgVectorManager.nextFeatVec(
                        orgDistanceFromStartOfEntity)) != null){
                    
                    double perAnswer = -1, perCanDist = -1;
                    double orgAnswer = -1, orgCanDist = -1;
                    
                    // in case feature vector for next token was recomputed without following safe rules
                    boolean noPerSafeRules = false;
                    boolean noOrgSafeRules = false;
                    
                    ////////////////////////////////////////////////////////////
                    ////////////// computing feature vectors for ///////////////
                    ///////////////// PERSON and ORGANIZATION //////////////////
                    ////////////////////////////////////////////////////////////
                    
                    // if it is surelly negative for both categories because of
                    // safe rules for each category, so we do not need them
                    if (orgAttr.length == 1 && perAttr.length ==1){
                        
                        perDistanceFromStartOfEntity = -1;
                        orgDistanceFromStartOfEntity = -1;
                        
                        continue fill_pool_loop_label;
                        
                    }else{
                        
                        // recompute features without appling safe rules for person
                        if (perAttr.length == 1){
                            
                            // computing again attributes without safe rules for person
                            perAttr =
                                    perVectorManager.prevFeatVec_noSafeRules(
                                    perDistanceFromStartOfEntity);
                            perDistanceFromStartOfEntity = -1;
                            noPerSafeRules = true;
                            
                        }
                        
                        // recompute without following safe rules for organization
                        if (orgAttr.length == 1){
                            
                            // computing again attributes without safe rules for organizations
                            orgAttr =
                                    orgVectorManager.prevFeatVec_noSafeRules(
                                    orgDistanceFromStartOfEntity);
                            orgDistanceFromStartOfEntity = -1;
                            noOrgSafeRules = true;
                            
                        }
                        //==============================================================
                        //===============computing feature vectors for==================
                        //==================PERSON and ORGANIZATION=====================
                        //==============================================================
                    }// else
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////converting feature vectors to///////////////////
                    ////////svm_nodes and calling svm classifiers to decide/////////
                    ////////////////////////////////////////////////////////////////
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////////////////person//////////////////////////////
                    int perNumOfAttributes = Constants.PER_FEAT_NUM_FP ;
                    svm_node perNodes[] = new svm_node[perNumOfAttributes];
                    
                    for (int i = 0; i < perNumOfAttributes; i++) {
                        
                        perNodes[i] = new svm_node();
                        perNodes[i].index = i+1;
                        perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                        
                    } // for
                    
                    // in case it surelly negative for person category beacause
                    // of person safe rules
                    if (!noPerSafeRules) {
                        
                        // PREDICT
                        perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                        if (perCategories[0] == -1) {
                            perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                        } // if
                        else {
                            perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                        } // else
                        
                    } // if
                    else{
                        
                        perAnswer = -1;
                        perCanDist = -1;
                        
                    }
                    
                    if ((perAnswer == 1) && (perDistanceFromStartOfEntity == -1)) {
                        perDistanceFromStartOfEntity = 1;
                    } // if
                    else if (perAnswer == 1) {
                        perDistanceFromStartOfEntity++;
                    } // else if
                    else {
                        perDistanceFromStartOfEntity = -1;
                    } // else if
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////////////organization///////////////////////////
                    
                    // in case it surelly negative for organization
                    // category beacause of organization safe rules
                    int orgNumOfAttributes = Constants.ORG_FEAT_NUM_FP;
                    svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                    
                    for (int i = 0; i < orgNumOfAttributes; i++) {
                        
                        orgNodes[i] = new svm_node();
                        orgNodes[i].index = i+1;
                        orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                        
                    } // for
                    // PREDICT
                    
                    if (!noOrgSafeRules) {
                        
                        orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                        if (orgCategories[0] == -1) {
                            orgCanDist =
                                    org_prob_estimates[1] - org_prob_estimates[0];
                        } // if
                        else {
                            orgCanDist =
                                    org_prob_estimates[0] - org_prob_estimates[1];
                        } // else
                    } // if
                    else{
                        orgAnswer = -1;
                        orgCanDist = -1;
                        
                    }
                    
                    if ((orgAnswer == 1) && (orgDistanceFromStartOfEntity == -1)) {
                        orgDistanceFromStartOfEntity = 1;
                    } // if
                    else if (orgAnswer == 1) {
                        orgDistanceFromStartOfEntity++;
                    } // else if
                    else {
                        orgDistanceFromStartOfEntity = -1;
                    } // else if
                    
                    perVectorManager.tokenAt(
                            perVectorManager.getOffset() - 1).setPredictedProbability(perCanDist);
                    
                    orgVectorManager.tokenAt(
                            orgVectorManager.getOffset() - 1).setPredictedProbability(orgCanDist);
                    
                    // if it is not a formerly used vector, then add it
                    boolean alreadySelected =
                            isUsedVector(curr.getName(),perVectorManager.getOffset()-1);
                    if (!alreadySelected) {
                        
                        AmbiguousVector nextVec =
                                constructAmbiguousVector(perTok, perAttr, orgAttr,
                                perVectorManager.getOffset() - 1, curr.getName(),
                                Math.abs(perCanDist), Math.abs(orgCanDist),
                                !noPerSafeRules,!noOrgSafeRules, Constants.SECOND_PASS);
                        
                        ambVecPool.add(nextVec);
                        count++;
                        
                    } // if
                    
                } // while : for each vector
                
        } // while : for each file
        
        random();
        System.out.println("Method used: " + " Random selection\n\n");
        
        // removing batch vectors from pool
        int countRemoved = 0;
        for (int iter = 0; iter < this.batch.length; iter++){
            
            if(ambVecPool.remove(batch[iter])) countRemoved++;
            
        }
        
        File pool2Save = new File(dirToSave, "(" + String.valueOf(expNum) + ").pool");
        pool2Save.getParentFile().mkdirs();
        // stream for saving pool after finding it
        ObjectOutputStream poolStr = new ObjectOutputStream(new FileOutputStream(pool2Save));
        poolStr.writeObject(ambVecPool);
        poolStr.close();
        
        File poolSizes = new File(dirToSave, "pool sizes.txt");
        
        if(poolSizes .exists()){
            
            FileReader rmf = new FileReader(poolSizes);
            char []cBuf = new char[(int)poolSizes.length()];
            rmf.read(cBuf);
            rmf.close();
            
            FileWriter fwr = new FileWriter(poolSizes);
            fwr.write(cBuf);
            fwr.write("\n" + " " + expNum + ": " + String.valueOf(ambVecPool.size()));
            fwr.close();
            
        }else{
            
            FileWriter fwr = new FileWriter(poolSizes);
            fwr.write(String.valueOf(ambVecPool.size()));
            fwr.close();
            
        }
        
        System.out.println("Vectors Counted in pool: " + count);
        System.out.println(countRemoved + " vectors removed from pool and added to last batch.\n" +
                ambVecPool.size() + " vectors in pool saved");
        System.out.println("Pool saved at path: \n" + dirToSave);
        
        return getBatch();
        
    }
    
    /**
     * @return
     *      The batch (an AmbiguousVector[] array) containing mostly ambguousVectors
     */
    public AmbiguousVector[] getBatch() {
        
        return batch;
        
    } // findAndGetBatch
    
    protected void random(){
        
        AmbiguousVector tmp[] = (AmbiguousVector[]) ambVecPool.toArray();
        // batchsize may be less than expected (eg in last experiment)
        batchSize = Math.min(tmp.length, batchSize);
        batch = new AmbiguousVector[batchSize];
        // filling batch with nodes, sorted according to informativeness measure
        int i=0;
        int count = 0;
        while(batchSize > count){
            
            batch[i] = tmp[i];
            i = (int)Math.random()*batchSize - 1;
            count++;
        }
        
    }// random
    
    /**
     * Fills the batch according to Informativeness measure. This means
     * according to the "distance" from svm's Hyperplane. It keeps sorted in the batch,
     * those AmbiguousVector Objects that are more close to the SVM's hyperplane.
     * The absolute of the cannonical probability assigned by the svm model is used
     * as a distance measure. As cannonical probability tends to be zero, as closer
     * the vector is to the hyperplane.
     *
     * Those AmbiguousVector Objects have the best informativeness score because
     * the SVM classifier is less sure about classifing them (they are more ambiguous).
     * It is expected to improove quicker the classifier model by retraining it according to
     * those ambiguous vectors.
     */
    protected void informativeness(boolean sortPool){
        
        
        // temporary ascendingly sorted array of ambiguous vectors from ambVecPool
        AmbiguousVector tmp[] = PoolHandler.ascDistSortedArray(ambVecPool);
        // batchsize may be less than expected (eg in last experiment)
        batchSize = Math.min(tmp.length, batchSize);
        batch = new AmbiguousVector[batchSize];
        // filling batch with nodes, sorted according to informativeness measure
        for (int i=0; i < batchSize;i++) {
            
            batch[i] = tmp[i];
            
        }
        
        if(sortPool){
            
            ambVecPool.clear();
            
            for(int i=0; i < tmp.length; i++){
                
                ambVecPool.add(tmp[i]);
                
            }
            
        }
        
    }//informativeness
    
    /**
     *
     * Computes batch as following:
     * Step 1:
     * <p>
     * It sorts the pool of non-used ambiguous vectors by appling the
     * informativeness method.(AmbiguousVector []tmp = infoSortedArr()).
     * <p>
     * Step2:
     * <p>
     * For each next AmbiguousVector (which has higher Informativeness than the
     * next ones in tmp[]), it keeps this node in the batch if and only if it is
     * less similar to any other AmbiguousVector objects allready included in the
     * current batch.<p>
     * Step3:
     * <p>
     * In case it fails to fully fill the batch, it starts filling it again this
     * time using the Informativeness method only.
     *
     *
     * @param similThreshold The average similarity of the whole pool of ative learning
     * files that have not yet been used .
     */
    protected void info_LocalDiversity(
            String category, String passFlag){
        
        double similThreshold = Constants.LOCAL_DIVER_THRESHOLD;
        
        int perFeatNum = 0, orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{// second pass
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        // temporary ascendingly sorted array of ambiguous vectors from ambVecPool
        AmbiguousVector tmp[] = PoolHandler.ascDistSortedArray(getAmbVecPool());
        // in case there are less than batchSize vectors available
        
        batchSize = Math.min(tmp.length,batchSize);
        batch = new AmbiguousVector[batchSize];
        // for every node in tmp (sorted ambVecPool)
        int pool_i;   // current vector in tmp
        int i = 0;    // how many in current batch so far
        
        // for every node in tmp, and untill batch is filled
        for (pool_i = 0; ( i < batchSize ) && ( pool_i < tmp.length ); pool_i++){
            
            AmbiguousVector next = tmp[pool_i];
            
            // true if next is rejected from batch
            boolean rejected = false;
            double simil = 0.0;
            // for every previous node in batch
            for (int j = 0; j < i; j++){
                // we keep next in batch, in case it isn't more similar than average
                // to any other vector already in the current batch  .
                // if break is executed then i remains the same then in next loop
                // node i will be replaced by an other from tmp
                simil = next.cosineSimillarityTo(batch[j], perFeatNum, orgFeatNum);
                if (simil >= similThreshold) {
                    
                    rejected = true ;
                    break;
                    
                }
                
            }// for j
            
            if(!rejected){
                
                // in order not to be reused in
                // case batch not filled because of local diversity
                // see after end of for loop
                tmp[pool_i] = null;
                batch[i++] = next;
                
            }
            
        }// for
        
        
        System.out.println(
                "Batch filled with " + i + " vectors by appling Informativeness and local diversity.\n" +
                "Local Diversity Threshold used = " + similThreshold + ".\n" +
                (( i != batchSize ) ? "Rest of batch will be filled by " +
                "appling informativeness method only\n" : "") +
                "Please wait . . .");
        
        /*
         * in case the batch contains less than batchSize, this means
         * that because of local diversity enforcement, it failed to fully fill the batch
         * so we fill the rest of the batch with best informativenes vectors (not already in batch)
         */
        if( i != batchSize ){
            
            
            int tmp_i = 0;
            
            while (i < batchSize) {
                
                if(tmp[tmp_i]!=null){
                    batch[i++] = tmp[tmp_i];
                    
                }
                
                tmp_i++;
            }
            
        }// if
        
    }//info_LocalDiversity
    
    /**
     * This method combines the foloowing:
     * 1)informativness (see method informativeness)
     * -->local diversity ()         batch
     * 2)representativness (see)
     */
    protected void info_repr_locDiver(
            String category, String passFlag){
        
        // similarty thershold for local diversity
        double similThreshold = Constants.LOCAL_DIVER_THRESHOLD;
        
        int featureNums[] = PoolHandler.getFeatureNums(passFlag, category);
        int perFeatNum = featureNums[0], orgFeatNum = featureNums[1];
        
        AmbiguousVector.InfoRepresentScoreNode infRep[];
        
        ArrayList<AmbiguousVector> subpool = new ArrayList<AmbiguousVector>();
        
        // array of nodes containing informativeness, representativeness. The array returned is
        // ascendingly sorted by a ballanced score of the above two measures, for
        // each AmbiguousVector
        infRep = PoolHandler.informRepresSortedArray(perFeatNum, orgFeatNum, ambVecPool, batchSize);
        
        batchSize = Math.min(infRep.length, batchSize);
        batch = new AmbiguousVector [batchSize];
        
        ////////////////////////////////////////////////////////////////////////
        ///////////////////appling local diversity//////////////////////////////
        
        int infRep_i = infRep.length-1;// current node in infRep array
        int i = 0;// how many in current batch so far
        // for every node in infRep array:
        // we read the array from bottom to top because after appling ascending
        // sorting, the highest information-representativeness scores are placed at
        // bottom of the array
        System.out.println("filling batch (appliing local diversity) . . .");
        int countLDiv = 0;
        while( (i < batchSize) && (infRep_i> -1) ){
            
            AmbiguousVector.InfoRepresentScoreNode next = infRep[infRep_i];
            int j;
            boolean i_rejected = false;
            for (j=0; j<i ; j++){
                // we keep next in batch, in case it isn't more similar than used threshold
                // to any other vector already in the current batch  .
                // if i is dicreased by one then in next loop the lastly added
                // node.parent in bacth, is replaced by an other node.parent
                if(
                        next.getAmbiguousVec().cosineSimillarityTo(
                        infRep[j].getAmbiguousVec(),perFeatNum,orgFeatNum) >= similThreshold) {
                    
                    i_rejected = true;
                    countLDiv ++;
                    break;
                    
                }// if
                
            }// for
            
            if(!i_rejected) {
                
                batch[i++] = next.getAmbiguousVec();
                System.err.println("added @pos = " + (i-1) + "vector with score = " + next.getScore() );
                infRep[infRep_i] = null;
                
                
            }
            
            infRep_i--;
            
        }// while
        
        
        
        /* Remember that batchSize is resized to the number of non-used vectors
         * in pool. So, in case the batch contains less than batchSize, this means
         * that because of local diversity enforcement, it failed to fully fill the batch
         * so we fill the rest we the best - not allready selected - vectors of infRep array...*/
        System.out.println(
                "Batch filled with " + i + " vectors by appling Informativeness, Representativeness and Local diversity.\n" +
                "Number of vectors excluded by local diversity: " + countLDiv);
        
        if(i != batchSize){
            
            System.out.println("Rest of batch will be filled by those vectors having highest \n" +
                    "informativeness-representativeness scores.\n" +
                    "(score = lamda*informativeness + (1-lamda)*representativeness, lamda = " + infRep[0].getLamda()+")");
            
            infRep_i = infRep.length-1;// current node in infRep array
            // i is equal to how many vectors in current batch so far
            while(( i < batchSize ) && ( infRep_i > -1) ){
                
                if (infRep[infRep_i] == null){
                    
                    infRep_i--;
                    
                }else{
                    
                    batch[i++] = infRep[infRep_i].getAmbiguousVec();
                    // System.err.println("added @pos = " + (i-1) + "vector with score = " + infRep[infRep_i].getScore() );
                    infRep_i--;
                    
                }
                
            }// while
            
        }
        
        
    }
    
    /**
     * -->informativness (  svm)
     * -->local diversity ()        batch
     * -->global diversity (        
     * batches     )
     *
     * protected void info_Loc_Glob_Diversity(
     * String category, String passFlag){
     *
     * long start = System.currentTimeMillis();
     *
     * double similThreshold = Constants.LOCAL_DIVER_THRESHOLD;
     * System.out.println("Threshold used: " + similThreshold);
     *
     *
     * int perFeatNum = 0, orgFeatNum = 0;
     * if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
     *
     * if (category.equalsIgnoreCase(Constants.PER_ORG)){
     * perFeatNum = Constants.PER_FEAT_NUM_FP;
     * orgFeatNum = Constants.ORG_FEAT_NUM_FP;
     * }
     *
     * else if (category.equalsIgnoreCase(Constants.PERSON))
     * perFeatNum = Constants.PER_FEAT_NUM_FP;
     *
     * else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
     * orgFeatNum = Constants.ORG_FEAT_NUM_FP;
     *
     *
     * } else{// second pass
     *
     * if (category.equalsIgnoreCase(Constants.PER_ORG)){
     * perFeatNum = Constants.PER_FEAT_NUM_SP;
     * orgFeatNum = Constants.ORG_FEAT_NUM_SP;
     * }
     *
     * else if (category.equalsIgnoreCase(Constants.PERSON))
     * perFeatNum = Constants.PER_FEAT_NUM_SP;
     *
     * else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
     * orgFeatNum = Constants.ORG_FEAT_NUM_SP;
     *
     * }
     *
     * // loading batches from former experiments
     * ArrayList<AmbiguousVector []> formerBatches =
     * PoolHandler.loadFormerObjectBatches();//PoolHandler.loadFormerBatches();
     * // vectors excluded by global diversity
     * ArrayList <AmbiguousVector> excludedByGlob = new ArrayList <AmbiguousVector>();
     * System.out.println("Number of former batches loaded: " + ((formerBatches!=null)?formerBatches.size():"0"));
     *
     * // temporary ascendingly sorted array of ambiguous vectors from ambVecPool
     * AmbiguousVector tmp[] = PoolHandler.ascDistSortedArray(getAmbVecPool());
     * ArrayList<AmbiguousVector> sortedVecs = new ArrayList<AmbiguousVector>();
     * for(int k=0;k<tmp.length;k++){
     * sortedVecs.add(tmp[k]);
     * }
     * // in case there are less than batchSize vectors available
     * batchSize = Math.min(tmp.length,batchSize);
     *
     * ////////////////////////////////////////////////////////////////////////
     * ////////////////////////// local diversity /////////////////////////////
     * int rejByLoc = 0;
     * // for every node in sortedVecs (sorted according to informativeness pool)
     *
     * // temporary batch
     * ArrayList <AmbiguousVector> batchTmp = new ArrayList <AmbiguousVector>();
     *
     * // for every node in sortedVecs, and untill batchTmp  is filled
     * int pool_i;   // current vector in sortedVecs
     * ALL_VECS_LABEL:for (pool_i = 0; ( batchTmp.size() <= batchSize ) && ( pool_i < sortedVecs.size() );
     * pool_i++){
     *
     * // true if next vector rejected by Local Diversity from the batchTmp
     * boolean rejectedByLocDiver = false;
     * double simil = 0.0;
     *
     * // for every node added in batchTmp so far, check . . .
     * for (int j = 0; j < (batchTmp.size()-1) && !rejectedByLocDiver; j++){
     *
     * // we keep next in batchTmp, in case it isn't more similar than threshold
     * // (similThreshold) to any other vector already in the current batchTmp.
     * simil = sortedVecs.get(pool_i).cosineSimillarityTo(batchTmp.get(j), perFeatNum, orgFeatNum);
     *
     * if (simil >= similThreshold) {
     *
     * rejectedByLocDiver = true ;
     * rejByLoc++;
     *
     * }
     *
     * }// for j
     *
     * if(!rejectedByLocDiver ){
     *
     * // in order not to be reused in
     * // case batchTmp not filled because of local diversity
     * // see after end of for loop
     * batchTmp.add(sortedVecs.remove(pool_i)); pool_i--;
     * //System.out.println("add in position " +
     * //      (batchTmp.size()-1) + " vector of tag :" +
     * //    batchTmp.get(batchTmp.size()-1).getTagType());
     *
     * if(batchTmp.size()<3) continue ALL_VECS_LABEL;
     * // System.out.println("Number of vectors excluded by appling local diversity: " + rejByLoc);
     *
     * ////////////////////////////////////////////////////////////////////
     * ////////Global diversity : diversity in all former batches//////////
     *
     * // for every node in sortedVecs, and untill batchTmp is filled
     * GLOB_DIV_LABEL: if( formerBatches != null ){// if former batches exist
     *
     * Iterator < AmbiguousVector[] >  iter = formerBatches.iterator();
     * // for every former batch
     * while(iter.hasNext()){
     *
     * AmbiguousVector[] nextFormerBatch = iter.next();
     *
     * // for every node in nextFormerBatch
     * // nextInNextBatch_i: current vector in nextFormerBatch
     * for (int nextInNextBatch_i = 0 ; nextInNextBatch_i < nextFormerBatch.length ; nextInNextBatch_i++){
     *
     * simil =
     * batchTmp.get(batchTmp.size()-1).cosineSimillarityTo(
     * nextFormerBatch[nextInNextBatch_i], perFeatNum, orgFeatNum);
     *
     * if ( simil > similThreshold) {
     *
     * excludedByGlob.add(batchTmp.remove(batchTmp.size()-1));
     *
     * // System.out.println("Number of vectors excluded by appling global diversity: " + excludedByGlob.size());
     *
     * continue ALL_VECS_LABEL;//break GLOB_DIV_LABEL;
     *
     * }
     *
     * }// for nextInNextBatch_i
     *
     * }// while(iter.hasNext())--> for every former batch
     *
     * ///////////global diversity : diversity in former batches///////////
     * //==================================================================
     *
     *
     *
     * }// if there are former batches
     * else{
     * System.out.println("No former batches exist");
     * }
     *
     *
     *
     * }
     *
     * //===========================local diversidy============================
     * //======================================================================
     *
     * }//for local diversity
     *
     * System.out.println("Number of vectors excluded by appling local diversity: " + rejByLoc);
     * System.out.println("Number of vectors excluded by appling global diversity: " + excludedByGlob.size());
     *
     * /*
     * in case the batchTmp contains less than batchSize, this means
     * that because of local diversity enforcement, it failed to fully fill the batchTmp
     * so we fill the rest of the batchTmp with rest best informativenes vectors.
     *
     * if( batchTmp.size()< batchSize ){
     *
     * System.out.println(
     * "Local and/or Global diversity caused temporary batch to contain less than\n" +
     * batchSize + " ambiguous vectors. \n" +
     * "Rest of batch filled acccording to Informativeness.");
     *
     *
     * int jpos = batchTmp.size()-1;
     *
     * Iterator<AmbiguousVector> sortedVecsIter = sortedVecs.iterator();
     * // filling rest of the batchTmp
     * while (jpos < batchSize && sortedVecsIter.hasNext()) {
     *
     * batchTmp.add(sortedVecsIter.next());
     * sortedVecsIter.remove(); pool_i--;
     *
     * jpos++;
     *
     *
     * }
     *
     * // adding again those excluded by global diversity
     * // in case batch not filled yet
     * if (jpos < batchSize) batchTmp.addAll(excludedByGlob);
     *
     * } // if(i != batchSize)
     *
     *
     * // finilizing permanent batch
     * batch = new AmbiguousVector[batchSize];
     * for( int k = 0; k < batchSize; k++){
     * batch[k] = batchTmp.get(k);
     * }
     *
     * System.out.println("Time needed for filling batch: " + Constants.toDigitalTimeString(start));
     *
     * }
     */
    
    public ArrayList<AmbiguousVector> getAmbVecPool() {
        return ambVecPool;
    }
    
    public ListManager getPerListManager() {
        return perListManager;
    }
    
    public ListManager getOrgListManager() {
        return orgListManager;
    }
    
    public Hashtable<String, UsedVector> getUsedVectors() {
        return usedVectors;
    }
    
    
} // end of class ActiveLearning
