/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning;

import ipl.nerc.active_learning.data.AmbiguousVector;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import java.io.*;
import libsvm.*;
import ipl.util.Constants;
import ipl.nerc.active_learning.data.SimilarityVector;
import ipl.nerc.lists.ListManager;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.Auxiliary;
import ipl.core.Tokenizer;
import java.util.*;

public class ActiveLearningFP extends ActiveLearning {
    
    /**
     * Constructor of ActiveLearningFP
     *
     *
     * @param usedVectorsFileName
     *      Path to file where all ambiguous vectors from former active learning
     *      experiments are saved in order not to be used again.
     * @param timexPatternsFileName
     *      Path to file with date and time saved patterns.
     * @param periodSeparatorModelFileName
     *      Path to file where period seperator model (classifier) is saved 
     * @param listsDirPath
     *      Path to directory with lists files
     * @param batchSize
     *      Size of batch of ambiguous vectors
     * @param category
     *      Category of experiment (@see class Constants e.g. Constants.PERSON)
     */
    public ActiveLearningFP(
            String timexPatternsFileName,
            String periodSeparatorModelFileName, String listsDirPath,
            int batchSize) {
        
        super(
                timexPatternsFileName,
                periodSeparatorModelFileName,batchSize);
        
        perListManager = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, listsDirPath);
        orgListManager = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, listsDirPath);
        
    } // constructor: ActiveLearningFP
    
    /**
     * This method is used for finding ambiguous vectors. A batch of this.bachSize
     * vectors is returned as an array of AmbiguousVector objects. Vectors of batch
     * are selected according to method specified by parameter howFlag.
     *
     *
     * @param poolDir
     *      path to directory where active learning files are stored
     * @param perModelFileName
     *      (pass null value if category is not person
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.PERSON is saved .
     * @param orgModelFileName
     *      (pass null value if category is not organization
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.ORGANIZATION is saved .
     * @param methodFlag
     *      A flag indicating which active learning method to use.
     * @param category
     *      Category of the named entities eg Constants.PER_ORG
     *
     * @param computeAll
     *      A flag used in case of loading a former batch from a file. If computeAll
     *      is false, then distances of vectors from hyperplanes are computed only
     *      for the first this.batchSize * Math.log10(poolSize) vectors of the loaded pool.
     *
     * @see class Constants for default paths and values
     */
    public AmbiguousVector[] findAndGetBatch(
            String usedVectorsFileName,
            String poolDir, String perModelFileName,
            String orgModelFileName, String category, int methodFlag,
            boolean computeAll) throws Exception {
       
        
        usedVectors = loadUsedVectors(usedVectorsFileName);

        if (category.equalsIgnoreCase(Constants.PER_ORG)){
            
                findPersonOrganization(poolDir, perModelFileName, orgModelFileName, methodFlag);
                return getBatch();
           
        }
        else if (category.equalsIgnoreCase(Constants.PERSON)){
            
            findPerson(poolDir, perModelFileName, methodFlag);
            return getBatch();
            
        }
        
        else {// (category.equalsIgnoreCase(Constants.ORGANIZATION)){
            
            findOrganization(poolDir, orgModelFileName, methodFlag);
            return getBatch();
            
        }
        
    } // findAmbiguousVectors
    
    
    /**performs main body of public method "findAmbiguousVectors(...)" when category = PERSON*/
    public void findPerson(String dir, String inputModelFileName,int methodFlag) throws Exception {
        
        // pool containing all candidate ambiguous vector nodes
        ambVecPool = new ArrayList<AmbiguousVector>();
        
        int count = 0;
        File curr;
        
        DirectoryManager testDir = new DirectoryManager(dir);
        
        svm_model model = svm.svm_load_model(inputModelFileName);
        int nr_class = svm.svm_get_nr_class(model);
        int []categories = new int[nr_class];
        svm.svm_get_labels(model, categories);
        double[] prob_estimates = new double[nr_class];
        
        
        while ( (curr = testDir.getNextFile()) != null) {
            if (curr.isFile()) {
                String attr[];
                int distanceFromStartOfEntity = -1;
                
                
                byte []buff = new byte[ (int) curr.length()];
                try {
                    FileInputStream fis = new FileInputStream(curr);
                    fis.read(buff);
                    fis.close();
                } // try
                catch (Exception e) {
                    e.printStackTrace();
                } // catch
                String text = new String(buff);
                FeaturesManager vectorManager;
                
                
                vectorManager = new PersonFeaturesManager(curr.getName(),text, getPerListManager(), matcher, separator);
                
                
                Tokenizer tok = vectorManager.getTokens();
                
                //    tokens  
                while ( (attr = vectorManager.nextFeatVec(distanceFromStartOfEntity)) != null) {
                    double answer, probability;
                    if (attr.length != 1) {
                        int numOfAttributes = Constants.PER_FEAT_NUM_FP;
                        svm_node node[] = new svm_node[numOfAttributes];
                        for (int i = 1; i <= numOfAttributes; i++) {
                            node[i - 1] = new svm_node();
                            node[i - 1].index = i;
                            node[i - 1].value = Auxiliary.atof(attr[i]);
                        } // for
                        // PREDICT
                        answer = svm.svm_predict_probability(model, node, prob_estimates);
                        if (categories[0] == -1) {
                            probability = prob_estimates[1] - prob_estimates[0];
                        } // if
                        else {
                            probability = prob_estimates[0] - prob_estimates[1];
                        } // else
                    } // if
                    else {
                        answer = -1;
                        probability = -1;
                    } // else
                    vectorManager.tokenAt(vectorManager.getOffset() - 1).setPredictedProbability(probability);
                    
                    
                    if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
                        distanceFromStartOfEntity = 1;
                    } // if
                    else if (answer == 1) {
                        distanceFromStartOfEntity++;
                    } // else if
                    else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
                        distanceFromStartOfEntity = -1;
                    } // else if
                    
                    
                    if (!isUsedVector(curr.getName(), vectorManager.getOffset() - 1)) {
                        
                        ambVecPool.add(this.constructAmbiguousVector(tok, attr, vectorManager.getOffset() - 1,
                                curr.getName(), probability));
                        count++;
                    } // if
                    
                } // while
                
            } // if
            
        } // while
        
    } // findPerson
    
    /**performs main body of public method "findAmbiguousVectors(...)" when category = ORGANIZATION*/
    public void findOrganization(String dir, String inputModelFileName,int methodFlag) throws Exception {
        
        // pool containing all candidate ambiguous vector nodes
        ambVecPool = new ArrayList<AmbiguousVector>();
        
        int count = 0;
        File curr;
        
        DirectoryManager testDir = new DirectoryManager(dir);
        
        svm_model model = svm.svm_load_model(inputModelFileName);
        int nr_class = svm.svm_get_nr_class(model);
        int []categories = new int[nr_class];
        svm.svm_get_labels(model, categories);
        double[] prob_estimates = new double[nr_class];
        
        
        while ( (curr = testDir.getNextFile()) != null) {
            if (curr.isFile()) {
                String attr[];
                int distanceFromStartOfEntity = -1;
                
                //   
                byte []buff = new byte[ (int) curr.length()];
                try {
                    FileInputStream fis = new FileInputStream(curr);
                    fis.read(buff);
                    fis.close();
                } // try
                catch (Exception e) {
                    e.printStackTrace();
                } // catch
                String text = new String(buff);
                FeaturesManager vectorManager;
                
                
                vectorManager = new OrganizationFeaturesManager(curr.getName(),text, getOrgListManager(), matcher, separator);
                
                
                Tokenizer tok = vectorManager.getTokens();
                
                
                while ( (attr = vectorManager.nextFeatVec(distanceFromStartOfEntity)) != null) {
                    
                    double answer, probability;
                    
                    if (attr.length != 1) {
                        
                        int numOfAttributes =Constants.ORG_FEAT_NUM_FP;
                        svm_node node[] = new svm_node[numOfAttributes];
                        
                        for (int i = 1; i <= numOfAttributes; i++) {
                            node[i - 1] = new svm_node();
                            node[i - 1].index = i;
                            node[i - 1].value = Auxiliary.atof(attr[i]);
                            
                        } // for
                        
                        // PREDICT
                        answer = svm.svm_predict_probability(model, node, prob_estimates);
                        
                        if (categories[0] == -1) {
                            probability = prob_estimates[1] - prob_estimates[0];
                        } // if
                        else {
                            probability = prob_estimates[0] - prob_estimates[1];
                        } // else
                    } // if
                    else {
                        answer = -1;
                        probability = -1;
                    } // else
                    vectorManager.tokenAt(vectorManager.getOffset() - 1).setPredictedProbability(probability);
                    
                    
                    if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
                        distanceFromStartOfEntity = 1;
                    } // if
                    else if (answer == 1) {
                        distanceFromStartOfEntity++;
                    } // else if
                    else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
                        distanceFromStartOfEntity = -1;
                    } // else if
                    
                    
                    if (!isUsedVector(curr.getName(), vectorManager.getOffset() - 1)) {
                        
                        ambVecPool.add(this.constructAmbiguousVector(tok, attr, vectorManager.getOffset() - 1,
                                curr.getName(), probability));
                        count++;
                    } // if
                    
                } // while
                
            } // if
            
        } // while
        
    } // findOrganization
    
    private void findPersonOrganization(
            String actLeFilesDir, String perInputModelFileName,
            String orgInputModelFileName, int methodFlag) throws Exception {
        
        
        // pool containing all candidate ambiguous vector nodes
        ambVecPool = new ArrayList<AmbiguousVector>();
        
        // counter for nodes
        int count = 0;
        File curr;
        DirectoryManager poolDirManag = new DirectoryManager(actLeFilesDir);
        
        svm_model perModel = svm.svm_load_model(perInputModelFileName);
        svm_model orgModel = svm.svm_load_model(orgInputModelFileName);
        
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        
        int []perCategories = new int[per_nr_class];
        int []orgCategories = new int[org_nr_class];
        
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        
        
        while ( (curr = poolDirManag.getNextFile()) != null && curr.isFile()) {
            
            String perAttr[];
            int perdistFromStartOfEntity = -1;
            
            String orgAttr[];
            int orgDistFromStartOfEntity = -1;
            
            ////////////////////////////////////////////////////////////////
            ////////////////reading next training text//////////////////////
            // buffer for next training text file
            byte []buff = new byte[ (int) curr.length()];
            
            try {
                FileInputStream fis = new FileInputStream(curr);
                fis.read(buff);
                fis.close();
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
            String text = new String(buff);
             ////////////////reading next training text//////////////////////
            //==============================================================
            
            FeaturesManager perVectorManager;
            FeaturesManager orgVectorManager;
            
            perVectorManager = new PersonFeaturesManager(curr.getName(),text, getPerListManager(), matcher, separator);
            orgVectorManager = new OrganizationFeaturesManager(curr.getName(),text, getOrgListManager(), matcher, separator);
            
            Tokenizer perTok = perVectorManager.getTokens();
            Tokenizer orgTok = orgVectorManager.getTokens();
            
            fill_pool_loop_label:
                while (// for each available feature vector in text
                        (perAttr = perVectorManager.nextFeatVec(
                        perdistFromStartOfEntity)) != null &&
                        (orgAttr = orgVectorManager.nextFeatVec(
                        orgDistFromStartOfEntity)) != null){
                    
                    double perAnswer = -1, perCanDist = -1;
                    double orgAnswer = -1, orgCanDist = -1;
                    
                    // in case feature vector for next token was recomputed without following safe rules
                    boolean noPerSafeRules = false;
                    boolean noOrgSafeRules = false;
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////computing feature vectors for//////////////////
                    ////////////////////PERSON and ORGANIZATION/////////////////////
                    ////////////////////////////////////////////////////////////////
                    
                    // if it is surelly negative for both categories because of
                    // safe rules for each category, so we do not need them
                    if (orgAttr.length == 1 && perAttr.length ==1){
                        
                        perdistFromStartOfEntity = -1;
                        orgDistFromStartOfEntity = -1;
                        
                        continue fill_pool_loop_label;
                        
                    }else{
                        
                        // recompute features without appling safe rules for person
                        if (perAttr.length == 1){
                            
                            // computing again attributes without safe rules for person
                            perAttr =
                                    perVectorManager.prevFeatVec_noSafeRules(
                                    perdistFromStartOfEntity);
                            perdistFromStartOfEntity = -1;
                            noPerSafeRules = true;
                            
                        }
                        
                        // recompute without following safe rules for organization
                        if (orgAttr.length == 1){
                            
                            // computing again attributes without safe rules for organizations
                            orgAttr =
                                    orgVectorManager.prevFeatVec_noSafeRules(
                                    orgDistFromStartOfEntity);
                            orgDistFromStartOfEntity = -1;
                            noOrgSafeRules = true;
                            
                        }
                        //==============================================================
                        //===============computing feature vectors for==================
                        //==================PERSON and ORGANIZATION=====================
                        //==============================================================
                    }// else
                    
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////converting feature vectors to///////////////////
                    ////////svm_nodes and calling svm classifiers to decide/////////
                    ////////////////////////////////////////////////////////////////
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////////////////person//////////////////////////////
                    int perNumOfAttributes = Constants.PER_FEAT_NUM_FP ;
                    svm_node perNodes[] = new svm_node[perNumOfAttributes];
                    
                    for (int i = 0; i < perNumOfAttributes; i++) {
                        
                        perNodes[i] = new svm_node();
                        perNodes[i].index = i+1;
                        perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                        
                    } // for
                    
                    // in case it surelly negative for person category beacause
                    // of person safe rules
                    if (!noPerSafeRules) {
                        
                        // PREDICT
                        perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                        if (perCategories[0] == -1) {
                            perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                        } // if
                        else {
                            perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                        } // else
                        
                    } // if
                    else{
                        
                        perAnswer = -1;
                        perCanDist = -1;
                        
                    }
                    
                    if ((perAnswer == 1) && (perdistFromStartOfEntity == -1)) {
                        perdistFromStartOfEntity = 1;
                    } // if
                    else if (perAnswer == 1) {
                        perdistFromStartOfEntity++;
                    } // else if
                    else {
                        perdistFromStartOfEntity = -1;
                    } // else if
                    
                    
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////////////organization///////////////////////////
                    
                    // in case it surelly negative for organization
                    // category beacause of organization safe rules
                    int orgNumOfAttributes = Constants.ORG_FEAT_NUM_FP;
                    svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                    
                    for (int i = 0; i < orgNumOfAttributes; i++) {
                        
                        orgNodes[i] = new svm_node();
                        orgNodes[i].index = i+1;
                        orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                        
                    } // for
                    // PREDICT
                    
                    if (!noOrgSafeRules) {
                        
                        orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                        if (orgCategories[0] == -1) {
                            orgCanDist =
                                    org_prob_estimates[1] - org_prob_estimates[0];
                        } // if
                        else {
                            orgCanDist =
                                    org_prob_estimates[0] - org_prob_estimates[1];
                        } // else
                    } // if
                    else{
                        orgAnswer = -1;
                        orgCanDist = -1;
                        
                    }
                    
                    if ((orgAnswer == 1) && (orgDistFromStartOfEntity == -1)) {
                        orgDistFromStartOfEntity = 1;
                    } // if
                    else if (orgAnswer == 1) {
                        orgDistFromStartOfEntity++;
                    } // else if
                    else {
                        orgDistFromStartOfEntity = -1;
                    } // else if
                    
                    
                    perVectorManager.tokenAt(
                            perVectorManager.getOffset() - 1).setPredictedProbability(perCanDist);
                    
                    orgVectorManager.tokenAt(
                            orgVectorManager.getOffset() - 1).setPredictedProbability(orgCanDist);
                    
                    // if it is not a formerly used vector, then add it
                    boolean alreadySelected =
                            isUsedVector(curr.getName(), perVectorManager.getOffset()-1);
                    if (!alreadySelected) {
                        
                        AmbiguousVector nextVec =
                                constructAmbiguousVector(perTok, perAttr, orgAttr,
                                perVectorManager.getOffset() - 1, curr.getName(),
                                Math.abs(perCanDist), Math.abs(orgCanDist),
                                !noPerSafeRules,!noOrgSafeRules, Constants.FIRST_PASS);
                        
                        ambVecPool.add(nextVec);
                        count++;
                        
                    } // if
                    
                } // while : for each vector
                
        } // while : for each file
        
        if( methodFlag == INFORMATIVENESS ){
            
            informativeness(true);
            System.out.println("Method used: " + "INFORMATIVENESS\n\n");
            
        }
        
        else if( methodFlag == INFO_LOCDIVER ){
            
            info_LocalDiversity( Constants.PER_ORG, Constants.FIRST_PASS);
            System.out.println("Method used: " + "INFORMATIVENESS, LOCAL DIVERSITY\n\n");
            
        }
        
        else if( methodFlag == INFO_REPR_LOCDIVER ){
            
            info_repr_locDiver(Constants.PER_ORG, Constants.FIRST_PASS);
            System.out.println("Method used: " + "INFORMATIVENESS, REPRESENTATIVENESS, LOCAL DIVERSITY\n\n");
            
        }
        
        
        
    }
    
    public ArrayList<SimilarityVector> findPoolForAvgSim(
            String actLeFilesDir, String perInputModelFileName,
            String orgInputModelFileName) throws Exception {
        
        // pool
        ArrayList<SimilarityVector> pool = new ArrayList<SimilarityVector>();
        
        int count = 0;
        File curr;
        DirectoryManager testDir = new DirectoryManager(actLeFilesDir);
        
        svm_model perModel = svm.svm_load_model(perInputModelFileName);
        svm_model orgModel = svm.svm_load_model(orgInputModelFileName);
        
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        
        int []perCategories = new int[per_nr_class];
        int []orgCategories = new int[org_nr_class];
        
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        
        
        while ( (curr = testDir.getNextFile()) != null && curr.isFile()) {
            
            String perAttr[];
            int perdistFromStartOfEntity = -1;
            
            String orgAttr[];
            int orgDistFromStartOfEntity = -1;
            
            ////////////////////////////////////////////////////////////////
            ////////////////reading next training text//////////////////////
            // buffer for next training text file
            byte []buff = new byte[ (int) curr.length()];
            
            try {
                FileInputStream fis = new FileInputStream(curr);
                fis.read(buff);
                fis.close();
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
            String text = new String(buff);
            ////////////////reading next training text//////////////////////
            //==============================================================
            
            FeaturesManager perVectorManager;
            FeaturesManager orgVectorManager;
            
            perVectorManager = new PersonFeaturesManager(curr.getName(),text, getPerListManager(), matcher, separator);
            orgVectorManager = new OrganizationFeaturesManager(curr.getName(),text, getOrgListManager(), matcher, separator);
            
            Tokenizer perTok = perVectorManager.getTokens();
            Tokenizer orgTok = orgVectorManager.getTokens();
            
            fill_pool_loop_label:
                while (// for each available feature vector in text
                        (perAttr = perVectorManager.nextFeatVec(
                        perdistFromStartOfEntity)) != null &&
                        (orgAttr = orgVectorManager.nextFeatVec(
                        orgDistFromStartOfEntity)) != null){
                    
                    double perAnswer = -1, perCanDist = -1;
                    double orgAnswer = -1, orgCanDist = -1;
                    
                    // in case feature vector for next token was recomputed without following safe rules
                    boolean noPerSafeRules = false;
                    boolean noOrgSafeRules = false;
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////computing feature vectors for//////////////////
                    ////////////////////PERSON and ORGANIZATION/////////////////////
                    ////////////////////////////////////////////////////////////////
                    
                    // if it is surelly negative for both categories because of
                    // safe rules for each category
                    if (orgAttr.length == 1 && perAttr.length ==1){
                        
                        perdistFromStartOfEntity = -1;
                        orgDistFromStartOfEntity = -1;
                        
                        continue fill_pool_loop_label;
                        
                    }
                    // recompute features without appling safe rules for person
                    if (perAttr.length == 1){
                        
                        perAttr =
                                perVectorManager.prevFeatVec_noSafeRules(
                                perdistFromStartOfEntity);
                        perdistFromStartOfEntity = -1;
                        noPerSafeRules = true;
                        
                    }
                    
                    // recompute without following safe rules for organization
                    if (orgAttr.length == 1){
                        
                        orgAttr =
                                orgVectorManager.prevFeatVec_noSafeRules(
                                orgDistFromStartOfEntity);
                        orgDistFromStartOfEntity = -1;
                        noOrgSafeRules = true;
                        
                    }
                    
                    //==============================================================
                    //===============computing feature vectors for==================
                    //==================PERSON and ORGANIZATION=====================
                    //==============================================================
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////converting feature vectors to///////////////////
                    ////////// svm_nodes, concatenating feature vectors ////////////
                    //////////////and calling svm classifiers to decide/////////////
                    ////////////////////////////////////////////////////////////////
                    
                    
                    // concatenated featurevector for both categories
                    String []concatFeatVec =
                            new String[
                            Constants.PER_FEAT_NUM_FP + Constants.ORG_FEAT_NUM_FP + 2];
                    
                    ////////////////////////////////////////////////////////////////
                    ////////////////////////////person//////////////////////////////
                    int perNumOfAttributes = Constants.PER_FEAT_NUM_FP ;
                    svm_node perNodes[] = new svm_node[perNumOfAttributes];
                    
                    for (int i = 0; i < perNumOfAttributes; i++) {
                        
                        perNodes[i] = new svm_node();
                        perNodes[i].index = i+1;
                        perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                        // must contain each category before category values
                        // because of the cosineSimilarity Method functions..
                        concatFeatVec[i] = perAttr[i];
                        
                    } // for
                    concatFeatVec[perNumOfAttributes] = perAttr[perNumOfAttributes];
                    
                    // in case it surelly negative for person category beacause
                    // of person safe rules
                    if (!noPerSafeRules) {
                        
                        // PREDICT
                        perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                        if (perCategories[0] == -1) {
                            perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                        } // if
                        else {
                            perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                        } // else
                        
                    } // if
                    else{
                        
                        perAnswer = -1;
                        perCanDist = -1;
                        
                    }
                    
                    if ((perAnswer == 1) && (perdistFromStartOfEntity == -1)) {
                        perdistFromStartOfEntity = 1;
                    } // if
                    else if (perAnswer == 1) {
                        perdistFromStartOfEntity++;
                    } // else if
                    else {
                        perdistFromStartOfEntity = -1;
                    } // else if
                    
                    
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////////////organization///////////////////////////
                    
                    // in case it surelly negative for organization
                    // category beacause of organization safe rules
                    int orgNumOfAttributes = Constants.ORG_FEAT_NUM_FP;
                    svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                    
                    for (int i = 0; i < orgNumOfAttributes; i++) {
                        
                        orgNodes[i] = new svm_node();
                        orgNodes[i].index = i+1;
                        orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                        // must contain each category before category values
                        // because of the cosineSimilarity Method functions..
                        concatFeatVec[perNumOfAttributes+1+i] = orgAttr[i];
                        
                    } // for
                    concatFeatVec[perNumOfAttributes+orgNumOfAttributes+1] = orgAttr[orgNumOfAttributes];
                    
                    
                    if (!noOrgSafeRules) {
                        
                        orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                        if (orgCategories[0] == -1) {
                            orgCanDist =
                                    org_prob_estimates[1] - org_prob_estimates[0];
                        } // if
                        else {
                            orgCanDist =
                                    org_prob_estimates[0] - org_prob_estimates[1];
                        } // else
                    } // if
                    else{
                        orgAnswer = -1;
                        orgCanDist = -1;
                        
                    }
                    
                    if ((orgAnswer == 1) && (orgDistFromStartOfEntity == -1)) {
                        orgDistFromStartOfEntity = 1;
                    } // if
                    else if (orgAnswer == 1) {
                        orgDistFromStartOfEntity++;
                    } // else if
                    else {
                        orgDistFromStartOfEntity = -1;
                    } // else if
                    
                    pool.add(new SimilarityVector(concatFeatVec));
                    count++;
                    
                } // while: for each vector
                
        } // while : for each file
        
        
        return pool;
        
    }
    
    
} // end of class ActiveLearningFP

