/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning;

import ipl.nerc.active_learning.data.AmbiguousVector;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import java.io.*;
import libsvm.*;
import ipl.util.Constants;
import ipl.nerc.active_learning.data.UsedVector;
import ipl.nerc.lists.ListManager;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.Auxiliary;
import ipl.core.Tokenizer;
import ipl.nerc.preprocessing.TextPreprocessor;
import java.util.*;
import javax.swing.JOptionPane;

public class ActiveLearningSP extends ActiveLearning {
    
    /**
     * Hashtable containing sure decisions by SECOND pass classifiers
     */
    private Hashtable<String, String[]> sureDecisions;
    
    /**
     * Hashtable containing all decisions by SECOND pass classifiers
     */
    private Hashtable<String, String> allDecisions;
    
    /**
     * Directory of used vector files from first pass;
     */
    private File FPUsedVectorsTxtFile;
    
    /**
     * Constructor of ActiveLearningSP
     *
     *
     * @param usedVectorsFileName
     *      Path to file where all ambiguous vectors from former active learning
     *      experiments are saved in order not to be used again.
     * @param timexPatternsFileName
     *      Path to file with date and time saved patterns.
     * @param periodSeparatorModelFileName
     *      Path to file where period seperator model (classifier) is saved
     * @param listsDirPath
     *      Path to directory with lists files
     * @param batchSize
     *      Size of batch of ambiguous vectors
     * @param category
     *      Category of experiment (@see class Constants e.g. Constants.PERSON)
     */
    public ActiveLearningSP(
            Hashtable<String, String> allDecisions, Hashtable<String, String[]> sureDecisions,
            File FPUsedVectorsTxtFile, String timexPatternsFileName, String periodSeparatorModelFileName,
            String listsDirPath, int batchSize) {
        
        super( timexPatternsFileName, periodSeparatorModelFileName, batchSize);
        
        this.sureDecisions = sureDecisions;
        this.allDecisions = allDecisions;
        this.FPUsedVectorsTxtFile = FPUsedVectorsTxtFile;
        perListManager = ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, listsDirPath);
        orgListManager = ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, listsDirPath);
        
    } // constructor: ActiveLearningSP
    
    /**
     * This method is used for finding ambiguous vectors. A batch of this.bachSize
     * vectors is returned as an array of AmbiguousVector objects. Vectors of batch
     * are selected according to method specified by parameter howFlag.
     *
     *
     * @param poolDir
     *      path to directory where active learning files are stored
     * @param perModelFileName
     *      (null value if category is not person
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.PERSON is saved .
     * @param orgModelFileName
     *      (null value if category is not organization
     *      or person_organization)
     *      Path to directory where SVM model for category
     *      Constants.ORGANIZATION is saved .
     * @param methodFlag
     *      A flag indicating which active learning method to use.
     * @param category
     *      Category of the named entities eg Constants.PER_ORG
     *
     * @param computeAll
     *      A flag used in case of loading a former batch from a file. If computeAll
     *      is false, then distances of vectors from hyperplanes are computed only
     *      for the SECOND this.batchSize * Math.log10(poolSize) vectors of the loaded pool.
     *
     * @see class Constants for default paths and values
     */
    public AmbiguousVector[] findAndGetBatch(String usedVectorsFileName, String poolDir,
            String perModelFileName, String orgModelFileName, String category, int methodFlag,
            boolean computeAll) throws Exception {
        
         usedVectors = loadUsedVectors(usedVectorsFileName);

         if (category.equalsIgnoreCase(Constants.PER_ORG)){
            
            findPersonOrganization(poolDir, perModelFileName, orgModelFileName, methodFlag);
            return getBatch();
            
         }else return null;
         
    } // findAmbiguousVectors
    
    private void findPersonOrganization(String actLeFilesDir, String perInputModelFileName,
            String orgInputModelFileName, int methodFlag) throws Exception {
        
        
        // pool containing all candidate ambiguous vector nodes
        ambVecPool = new ArrayList<AmbiguousVector>();
        
        // counter for nodes
        int count = 0;
        File curr;
        DirectoryManager testDir = new DirectoryManager(actLeFilesDir);
        
        svm_model perModel = svm.svm_load_model(perInputModelFileName);
        svm_model orgModel = svm.svm_load_model(orgInputModelFileName);
        
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        
        int []perCategories = new int[per_nr_class];
        int []orgCategories = new int[org_nr_class];
        
        svm.svm_get_labels(perModel, perCategories);
        svm.svm_get_labels(orgModel, orgCategories);
        
        double[] per_prob_estimates = new double[per_nr_class];
        double[] org_prob_estimates = new double[org_nr_class];
        
        
        while ( (curr = testDir.getNextFile()) != null && curr.isFile()) {
            
            String perAttr[];
            int perDistFromStartOfEntity = -1;
            
            String orgAttr[];
            int orgDistFromStartOfEntity = -1;
            
            ////////////////////////////////////////////////////////////////
            ////////////////reading next training text//////////////////////
            // buffer for next training text file
            byte []buff = new byte[ (int) curr.length()];
            
            try {
                FileInputStream fis = new FileInputStream(curr);
                fis.read(buff);
                fis.close();
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
            String text = new String(buff);
            ////////////////reading next training text//////////////////////
            //==============================================================
            
            FeaturesManager perVectorManager;
            FeaturesManager orgVectorManager;
            
            perVectorManager = new PersonFeaturesManager(curr.getName(),text, getPerListManager(), matcher, separator);
            orgVectorManager = new OrganizationFeaturesManager(curr.getName(),text, getOrgListManager(), matcher, separator);
            
            Tokenizer perTok = perVectorManager.getTokens();
            Tokenizer orgTok = orgVectorManager.getTokens();
            
            fill_pool_loop_label:
                while (// for each available feature vector in text
                    (perAttr = perVectorManager.nextFeatVec(perDistFromStartOfEntity, sureDecisions, allDecisions, curr.getName())) != null &&
                    (orgAttr = orgVectorManager.nextFeatVec(orgDistFromStartOfEntity, sureDecisions, allDecisions, curr.getName())) !=  null){
                
                // JOptionPane.showMessageDialog(null, "perAttr = "+perAttr.length);
                // JOptionPane.showMessageDialog(null, "orgAttr = "+orgAttr.length);
                
                double perAnswer = -1, perCanDist = -1;
                double orgAnswer = -1, orgCanDist = -1;
                
                // in case feature vector for next token was recomputed without following safe rules
                boolean noPerSafeRules = false;
                boolean noOrgSafeRules = false;
                
                ////////////////////////////////////////////////////////////////
                /////////////////computing feature vectors for//////////////////
                ////////////////////PERSON and ORGANIZATION/////////////////////
                ////////////////////////////////////////////////////////////////
                
                // if it is surelly negative for both categories because of
                // safe rules for each category, so we do not need them
                if (orgAttr.length == 1 && perAttr.length == 1){
                    
                    perDistFromStartOfEntity = -1;
                    orgDistFromStartOfEntity = -1;
                    
                    continue fill_pool_loop_label;
                    
                }else{
                    
                    // recompute features without appling safe rules for person
                    if (perAttr.length == 1){
                        
                        // computing again attributes without safe rules for person
                        perAttr =
                                perVectorManager.prevFeatVec_noSafeRules(
                                perDistFromStartOfEntity, sureDecisions, allDecisions, curr.getName());
                        perDistFromStartOfEntity = -1;
                        noPerSafeRules = true;
                        
                    }
                    
                    // recompute without following safe rules for organization
                    if (orgAttr.length == 1){
                        
                        // computing again attributes without safe rules for organizations
                        orgAttr =
                                orgVectorManager.prevFeatVec_noSafeRules(
                                orgDistFromStartOfEntity, sureDecisions, allDecisions, curr.getName());
                        orgDistFromStartOfEntity = -1;
                        noOrgSafeRules = true;
                        
                    }
                    //==============================================================
                    //===============computing feature vectors for==================
                    //==================PERSON and ORGANIZATION=====================
                    //==============================================================
                }// else
                
                
                ////////////////////////////////////////////////////////////////
                ////////////////converting feature vectors to///////////////////
                ////////svm_nodes and calling svm classifiers to decide/////////
                ////////////////////////////////////////////////////////////////
                
                ////////////////////////////////////////////////////////////////
                ////////////////////////////person//////////////////////////////
                int perNumOfAttributes = Constants.PER_FEAT_NUM_SP ;
                svm_node perNodes[] = new svm_node[perNumOfAttributes];
                
                for (int i = 0; i < perNumOfAttributes; i++) {
                    
                    perNodes[i] = new svm_node();
                    perNodes[i].index = i+1;
                    perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                    
                } // for
                
                // in case it surelly negative for person category beacause
                // of person safe rules
                if (!noPerSafeRules) {
                    
                    // PREDICT
                    perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                    if (perCategories[0] == -1) {
                        perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                    } // if
                    else {
                        perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                    } // else
                    
                } // if
                else{
                    
                    perAnswer = -1;
                    perCanDist = -1;
                    
                }
                
                if ((perAnswer == 1) && (perDistFromStartOfEntity == -1)) {
                    perDistFromStartOfEntity = 1;
                } // if
                else if (perAnswer == 1) {
                    perDistFromStartOfEntity++;
                } // else if
                else {
                    perDistFromStartOfEntity = -1;
                } // else if
                
                
                
                ////////////////////////////////////////////////////////////////
                /////////////////////////organization///////////////////////////
                
                // in case it surelly negative for organization
                // category beacause of organization safe rules
                int orgNumOfAttributes = Constants.ORG_FEAT_NUM_SP;
                svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                
                for (int i = 0; i < orgNumOfAttributes; i++) {
                    
                    orgNodes[i] = new svm_node();
                    orgNodes[i].index = i+1;
                    orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                    
                } // for
                // PREDICT
                
                if (!noOrgSafeRules) {
                    
                    orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                    if (orgCategories[0] == -1) {
                        orgCanDist =
                                org_prob_estimates[1] - org_prob_estimates[0];
                    } // if
                    else {
                        orgCanDist =
                                org_prob_estimates[0] - org_prob_estimates[1];
                    } // else
                } // if
                else{
                    orgAnswer = -1;
                    orgCanDist = -1;
                    
                }
                
                if ((orgAnswer == 1) && (orgDistFromStartOfEntity == -1)) {
                    orgDistFromStartOfEntity = 1;
                } // if
                else if (orgAnswer == 1) {
                    orgDistFromStartOfEntity++;
                } // else if
                else {
                    orgDistFromStartOfEntity = -1;
                } // else if
                
                perVectorManager.tokenAt(
                        perVectorManager.getOffset() - 1).setPredictedProbability(perCanDist);
                
                orgVectorManager.tokenAt(
                        orgVectorManager.getOffset() - 1).setPredictedProbability(orgCanDist);
                
                // if it is not a formerly used vector, then add it
                boolean alreadySelected =
                        isUsedVector(curr.getName(), perVectorManager.getOffset()-1);
                if (!alreadySelected) {
                    
                    AmbiguousVector nextVec =
                            constructAmbiguousVector(perTok, perAttr, orgAttr,
                            perVectorManager.getOffset() - 1, curr.getName(),
                            Math.abs(perCanDist), Math.abs(orgCanDist),
                            !noPerSafeRules, !noOrgSafeRules, Constants.SECOND_PASS);
                    
                    ambVecPool.add(nextVec);
                    count++;
                    
                } // if
                
                } // while : for each vector
            
        } // while : for each file
        
        if( methodFlag == INFORMATIVENESS ){
            
            informativeness(true);
            System.out.println("Method used: INFORMATIVENESS\n\n");
            
        } else if( methodFlag == INFO_LOCDIVER ){
            
            info_LocalDiversity( Constants.PER_ORG, Constants.SECOND_PASS);
            System.out.println("Method used: INFORMATIVENESS, LOCAL DIVERSITY\n\n");
            
        } else if( methodFlag == INFO_REPR_LOCDIVER ){
            
            info_repr_locDiver(Constants.PER_ORG, Constants.SECOND_PASS);
            System.out.println("Method used: INFORMATIVENESS, REPRESENTATIVENESS, LOCAL DIVERSITY\n\n");
            
        }
        /*else if( methodFlag == INFO_LOC_GLOBDIVER ){
         
            info_Loc_Glob_Diversity(Constants.PER_ORG, Constants.SECOND_PASS);
            System.out.println("Method used: INFORMATIVENESS, LOCAL DIVERSITY, GLOBAL DIVERSITY\n\n");
         
        }*/
        
        
        
    }
    
   
    /**
     * @param fileName
     *      The path to the file with used vectors. (First pass used vectors also considered as used)
     *
     * @return  A HashTable containing UsedVectorNode Objects, using the String tokens
     *          as keys.
     */
    protected Hashtable<String, UsedVector> loadUsedVectors(String fileName) {
        
        System.out.print( Constants.SECOND_PASS + ":");
        Hashtable<String, UsedVector> usedVectors = super.loadUsedVectors(fileName);
        int c = 0;
        try {
            
            File f = FPUsedVectorsTxtFile;
            FileInputStream fis = new FileInputStream(f);
            byte[] buff = new byte[ (int) f.length()];
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text =  TextPreprocessor.removeNoisySpaces(
                    text);
            StringTokenizer strTok = new StringTokenizer(text, " \n");
            
            while (strTok.hasMoreTokens()) {
                
                c++;
                String fname = strTok.nextToken(); // System.out.print((++c) + ": Used Vector filename: " + fname + ", @");
                int pos = Auxiliary.atoi(strTok.nextToken());// System.out.println("pos: " + pos );
                UsedVector temp = new UsedVector(fname, pos);
                String key = Integer.toString(pos) + fname;
                usedVectors.put(key, temp);
                
            } // while
            
            strTok = null;
        }catch (Exception e) {
            e.printStackTrace();
        } // catch
        System.out.println( Constants.FIRST_PASS + " used vectors loaded: " + c + "\nTotal number of used vectors: " + usedVectors.size());
        
        return usedVectors;
        
    } // loadUsedVectors
    
} // end of class ActiveLearningSP

