/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import ipl.util.Constants;
import ipl.util.Auxiliary;
import ipl.core.TokenNode;
import java.io.Serializable;
import java.util.*;

/**
 * A feature vector which is ambiguous because the svm classifier is less sure
 * about classifing it. Objects of this class are used for filling batches in
 * active learning experiments for improoving svm classifier model.
 */
public class AmbiguousVector implements Serializable {
    
    /**
     * Values of features of this ambiguous feature vector.
     * Althought the type of those values is double precision, the values
     * are stored in an array in String format.
     */
    protected String []featureVector;
    
    /**
     * An array of tokens of the phrase where the token of this AmbiguousVectoreNode
     * is taken.
     */
    protected String phrase[];
    /**Position of ambiguous token in phrase*/
    private int posInPhrase;
    
    /**
     * The token of this AmbiguousVectoreNode
     *
     * @see tokNode
     */
    protected String token;
    
    /**Type of Tag for token eg PERSON */
    private String tagType;
    
    /**Name of the file where this.token is found*/
    protected String fileName;
    
    /**Tokens before this.token in this.phrase*/
    protected String previousWords[];
    
    /**Position of this.token in text*/
    protected int position;
    
    
    private double distancePerHyperplane;
    private double distanceOrgHyperplane;
    private double distanceSingleHyperplane;
    /**
     * True if person canonical distancePerHyperplane is less than organization distancePerHyperplane
     */
    private boolean perMinDistance;
    
    /**
     * It is possible to compute all features values of a vector without
     * appling safe rules.
     *
     * This is due to computing similarity between concatenated feature vectors
     * (concatenation of person and organization feature vectors),
     * when PERSON safe rules imply that it is a negative vector for person category,
     * but ORGANIZATION safe rules do not imply that it is a negative one for
     * ORGANIZATION category.
     *
     * In such cases it is meaningless to ask user to decide whether this vector
     * is a person or not. So this flag is false when user is not to be asked to
     * classify this ambiguous vector.
     */
    private boolean personSafeRulesApplied;
    
    /**
     * It is possible to compute all features values of a vector without
     * appling safe rules.
     *
     * This is due to computing similarity between concatenated feature vectors
     * (concatenation of ORGANIZATION and PERSON feature vectors),
     * when ORGANIZATION safe rules imply that it is a negative vector for ORGANIZATION category,
     * but PERSON safe rules do not imply that it is a negative one for
     * PERSON category.
     *
     * In such cases it is meaningless to ask user to decide whether this vector
     * is a ORGANIZATION or not. So this flag is false when user is not to be asked to
     * classify this ambiguous vector.
     */
    private boolean organizationSafeRulesApplied;
    
    /**
     * A distancePerHyperplane comparator returned by static method DISTANCE_COMPARATOR()
     */
    private static DistanceComparator DISTANCE_COMPARATOR =
            new AmbiguousVector.DistanceComparator();
    
    /**
     * Constructor AmbiguousVector
     *
     *
     *
     * @param phrase
     *      An array with all tokens of the phrase where token "word" comes from
     * @param word
     *      Token from text
     * @param fileName
     *      Name of the file where token comes from
     * @param previousWords
     *      An array of tokens before token word
     * @param featureVector
     *      An array of feature values in String format
     * @param position
     *      Position of token inside file (start from zero)
     * @param distFromHyperPlane
     *      Cannonical posibility computed by svm predicted probabilities.
     *      This value is canonicalized in range [-1,1]. The absolute value of the
     *      Cannonical possibillity is used as the distancePerHyperplane from svm-hyperplane measure.
     */
    public AmbiguousVector(
            String[] phrase, int posInPhrase, TokenNode tokNode, String fileName,
            String[] previousWords, String []featureVector,
            int position,
            double distPerHyperplane, double distOrgHyperplane,
            boolean okByPerSafeRules, boolean okByOrgSafeRules) {
        
        this.phrase = phrase;
        this.posInPhrase = posInPhrase;
        this.token = tokNode.getToken();
        setTagType(tokNode.getTagType());
        this.fileName = fileName;
        this.previousWords = previousWords;
        this.featureVector = featureVector;
        this.position = position;
        this.personSafeRulesApplied = okByPerSafeRules;
        this.organizationSafeRulesApplied = okByOrgSafeRules;
        this.setDistancePerHyperplane(distPerHyperplane);
        this.setDistanceOrgHyperplane(distOrgHyperplane);
        this.perMinDistance = (getDistancePerHyperplane() == getMinDistanceHyperplane());
        
    } // constructor: AmbiguousVector
    
    
    /**
     * Constructor AmbiguousVector
     *
     *
     * @param phrase
     *      An array with all tokens of the phrase where token "word" comes from
     * @param word
     *      Token from text
     * @param fileName
     *      Name of the file where token comes from
     * @param previousWords
     *      An array of tokens before token word
     * @param featureVector
     *      An array of feature values in String format
     * @param position
     *      Position of token inside file (start from zero)
     * @param distFromHyperPlane
     *      Cannonical posibility computed by svm predicted probabilities.
     *      This value is canonicalized in range [-1,1]. The absolute value of the
     *      Cannonical possibillity is used as the distancePerHyperplane from svm-hyperplane measure.
     */
    public AmbiguousVector(
            String[] phrase, int posInPhrase, TokenNode tokNode, String fileName,
            String[] previousWords, String []featureVector,
            int position, double distFromSingleHyperPlane) {
        
        this.phrase = phrase;
        this.posInPhrase = posInPhrase;
        this.token = tokNode.getToken();
        setTagType(tokNode.getTagType());
        this.fileName = fileName;
        this.previousWords = previousWords;
        this.featureVector = featureVector;
        this.position = position;
        this.setDistanceSingleHyperplane(distFromSingleHyperPlane);
        this.personSafeRulesApplied = true;
        this.organizationSafeRulesApplied = true;
        
    } // constructor: AmbiguousVector
    
    
    /**
     * Constructor AmbiguousVector
     *
     * @param fileName
     *      Name of the file where token comes from
     *
     * @param position
     *      Position of token inside file (start from zero)
     *
     * @param distFromHyperPlane
     *      Cannonical posibility computed by svm predicted probabilities.
     *      This value is canonicalized in range [-1,1]. Cannonical possibillity
     *      is used as the distFromHyperPlane measure for this ambiguous vector
     */
    public AmbiguousVector(
            String fileName,int position, double distFromSingleHyperPlane, String []featureVectorIn) {
        
        this.fileName = fileName;
        this.position = position;
        this.setDistanceSingleHyperplane(distFromSingleHyperPlane);
        this.featureVector = featureVectorIn;
        
    } // constructor: AmbiguousVector
    
    /**
     * @return
     *      An array containing all tokens in the phrase where the token of
     *      this AmbiguousVector bellongs to.
     */
    public String[] getPhrase() {
        return phrase;
    } // getPhrase
    
    /**
     * @return         
     *    
     */
    public String[] getFeatureVector() {
        
        return featureVector;
        
    } // getFeatureVector
    
    /**
     * @return Token of this Ambiguous vector
     */
    public String getToken() {
        return token;
    } // getToken
    
    /**
     * @return Name of the file where token wqs extracted.
     */
    public String getFileName() {
        return fileName;
    } // getFileName
    
    /**
     * @return          
     */
    public String[] getPreviousTokens() {
        
        return previousWords;
        
    } // getPreviousTokens
    
    /**
     * @return    token   .     
     *   0,    1 ...
     */
    public int getPosition() {
        return position;
    } // getPosition
    
    /**
     * informativeness:
     *      Equal to 1.0 - distance_from_Minimum_SVM-hyperplane
     *
     *
     * @return 1.0 - distance_from_Minimum_SVM-hyperplane
     */
    public double informativeness(){
        
        return 1.0 - getMinDistanceHyperplane();
        
    }
    
    
    /**
     * informativeness:
     *      Equal to 1.0 - distance_from_PERSON_SVM-hyperplane
     *
     *
     * @return 1.0 - distance_from_PERSON_SVM-hyperplane
     */
    public double personInformativeness(){
        
        return 1.0 - getDistancePerHyperplane();
        
    }
    
    /**
     * informativeness:
     *      Equal to 1.0 - distance_from_ORGANIZATION_SVM-hyperplane
     *
     *
     * @return 1.0 - distance_from_PERSON_SVM-hyperplane
     */
    public double organizationInformativeness(){
        
        return 1.0 - getDistanceOrgHyperplane();
        
    }
    
    
    public static double[] checkPassFlag(String passFlag) throws Exception{
        
        double perGamma, orgGamma;
        double perFeatNum , orgFeatNum ;
        
        ////////////////////////////////////////////////////////////////////////
        ////////checking passFlag argument and initializing parameters//////////
        if(passFlag.equalsIgnoreCase("fp")){
            
            perGamma = Constants.PER_GAMMA_FP;
            orgGamma = Constants.ORG_GAMMA_FP;
            perFeatNum = Constants.PER_FEAT_NUM_FP;
            orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
        }else if(passFlag.equalsIgnoreCase("sp")){
            
            perGamma = Constants.PER_GAMMA_SP;
            orgGamma = Constants.ORG_GAMMA_SP;
            perFeatNum = Constants.PER_FEAT_NUM_SP;
            orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }else{
            perGamma = -1;
            orgGamma = -1;
            perFeatNum = -1;
            orgFeatNum = -1;
            throw new Exception(
                    "Elligal pass argument - must be either\n" +
                    "\"fp\" for first pass or \n" +
                    "\"sp\" for second pass");
            
        }
        return new double[]{perGamma,orgGamma,perFeatNum,orgFeatNum};
        //=========checking pass argument and initializing parameters===========
        //======================================================================
        
        
    }
    
    public int hashCode(){
        
        return (this.fileName+this.token).hashCode();
        
    }
    
    public InfoRepresentScoreNode infoReprNode(double density){
        
        return new InfoRepresentScoreNode(density);
        
    }
    
    public boolean isPerMinDistance() {
        
        this.perMinDistance = (getDistancePerHyperplane() == getMinDistanceHyperplane());
        return this.perMinDistance;
        
    }
    
    public int getPosInPhrase() {
        return posInPhrase;
    }
    
    public boolean equals(AmbiguousVector b){
        
        return
                (this.fileName.equalsIgnoreCase(b.fileName)
                &&
                this.position == b.position);
        
    }
    
    public double getMinDistanceHyperplane() {
        
        return Math.min(getDistancePerHyperplane(), getDistanceOrgHyperplane());
        
    }
    
    public String getTagType() {
        return tagType;
    }
    
    public void setTagType(String tag) {
        this.tagType = tag ;
    }
    
    public boolean isPersonSafeRulesApplied() {
        return personSafeRulesApplied;
    }
    
    public boolean isOrganizationSafeRulesApplied() {
        return organizationSafeRulesApplied;
    }
    
    public  int[] getFeatureNums(String passFlag,String category){
        
        int perFeatNum = 0,orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{// second pass
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        int[] ret = new int[2];
        ret[0] = perFeatNum;
        ret[1] = orgFeatNum;
        // return new int[]{perFeatNum,orgFeatNum};
        
        return ret;
        
    }
    
    /**
     * Objects of this class contain the informativeness and representativeness
     * of an AmbiguousVector instance.
     * You may get an instance of InfoRepresentScoreNode by calling
     * AmbiguousVectorObject.infoReprNode(double density). This statement  returns
     * an InfoRepresentScoreNode instance which contains:
     * (1) a reference to AmbiguousVectorObject and
     * (2) the cannonical distancePerHyperplane (informativeness) of AmbiguousVectorObject.
     */
    public class InfoRepresentScoreNode implements Comparable<AmbiguousVector.InfoRepresentScoreNode>{
        
        /**
         * The ambiguous vector that created this InfoRepresentScoreNode
         */
        private AmbiguousVector parent;
        
        /**
         * Canceled default construtor - it is private in order not to be used
         */
        private InfoRepresentScoreNode(){}// not to be used
        /**
         * Private constructor used by the AmbiguousVector that creates this
         * node. You may get an instance of InfoRepresentScoreNode by calling
         * AmbiguousVectorObject.infoReprNode(double density). This statement calls
         * this constructor, and the returned InfoRepresentScoreNode instance contains areference to
         * AmbiguousVectorObject and the cannonical distancePerHyperplane (informativeness) of
         * AmbiguousVectorObject.
         */
        private InfoRepresentScoreNode(double densityIn){
            
            parent = AmbiguousVector.this;
            distanceHyperplane = Math.abs(AmbiguousVector.this.getMinDistanceHyperplane());
            density = densityIn;
            setLamda(0.6);
            score = getLamda() * (1.0 - getDistance()) + (1.0 - getLamda()) * getDensity() ;
            
        }
        /**
         * Private constructor used by the AmbiguousVector that creates this
         * node. You may get an instance of InfoRepresentScoreNode by calling
         * AmbiguousVectorObject.infoReprNode(double density). This statement calls
         * this constructor, and the returned InfoRepresentScoreNode instance contains areference to
         * AmbiguousVectorObject and the cannonical distancePerHyperplane (informativeness) of
         * AmbiguousVectorObject.
         */
        private InfoRepresentScoreNode(double densityIn,double lamdaIn){
            
            parent = AmbiguousVector.this;
            distanceHyperplane = Math.abs(AmbiguousVector.this.getMinDistanceHyperplane());
            density = densityIn;
            setLamda(lamdaIn);
            score = getLamda() * (1.0 - getDistance()) + (1.0-getLamda()) * getDensity() ;
            
        }
        /**
         * Parameter lamda usede for computing value of score field
         */
        private double lamda;
        /**
         * Informativeness: Distance from svm hyperplane
         */
        private double distanceHyperplane;
        /**
         * Representativeness: density of this ambiguous vector
         */
        private double density;
        /**
         * Score = lamda * distancePerHyperplane + (1.0-lamda) * density
         */
        private double score;
        
        
        public int compareTo(InfoRepresentScoreNode b){
            
            int scoreThis = (int)(this.getScore()*1000000000);
            int scoreB = (int)(b.getScore()*1000000000);
            
            return scoreThis - scoreB;
            
        }
        
        /**
         *
         *
         * @return The value of distancePerHyperplane field
         */
        public double getDistance() {
            return distanceHyperplane;
        }
        /**
         * @return
         *      The value of density field
         */
        public double getDensity() {
            return density;
        }
        public AmbiguousVector getAmbiguousVec() {
            return parent;
        }
        
        public double getLamda() {
            return lamda;
        }
        
        public double getScore() {
            return score;
        }
        
        public void setLamda(double lamda) {
            this.lamda = lamda;
        }
        
    }
    
    public double getDistancePerHyperplane() {
        return distancePerHyperplane;
    }
    
    public void setDistancePerHyperplane(double distancePerHyperplane) {
        this.distancePerHyperplane = distancePerHyperplane;
    }
    
    public double getDistanceOrgHyperplane() {
        return distanceOrgHyperplane;
    }
    
    public void setDistanceOrgHyperplane(double distanceOrgHyperplane) {
        this.distanceOrgHyperplane = distanceOrgHyperplane;
    }
    
    public double getDistanceSingleHyperplane() {
        return distanceSingleHyperplane;
    }
    
    public void setDistanceSingleHyperplane(double distanceSingleHyperplane) {
        this.distanceSingleHyperplane = distanceSingleHyperplane;
    }
    
    /**
     * The representativeness of this ambiguousVectoreNode is defind to its density.
     * Given a pool of ambiguous vectors, it computes and returns the density of
     * this AmbiguousVetor object. Its density is equal to the average simillarity
     * of this AmbiguousVetor to all the other AmbiguousVector Objects in pool.
     */
    public double representativeness(
            ArrayList<AmbiguousVector> pool, String category, String passFlag) throws Exception{
        
        double sum = 0.0;
        
        int[] featNums = getFeatureNums(passFlag, category);
        int perFeatNum = featNums[0], orgFeatNum = featNums[1];
        
        for (int ipos = 0; ipos < pool.size(); ipos++){
            
            AmbiguousVector next = pool.get(ipos);
            
            if(next != this)
                sum = sum + cosineSimillarityTo(next, perFeatNum, orgFeatNum);
            
        }
        
        return sum/(pool.size()-1);
        
        
        
    }
    /**
     * The representativeness of this ambiguousVectoreNode is defind to its density.
     * Given a pool of ambiguous vectors, it computes and returns the density of
     * this AmbiguousVetor object. Its density is equal to the average simillarity
     * of this AmbiguousVetor to all the other AmbiguousVector Objects in pool.
     */
    public double representativeness(
            ArrayList<AmbiguousVector> pool, int perFeatNum , int orgFeatNum ) {
        
        double sum = 0.0;
        
        for (int ipos = 0; ipos < pool.size(); ipos++){
            
            AmbiguousVector next = pool.get(ipos);
            
            if(next != this)
                sum = sum + cosineSimillarityTo(next, perFeatNum, orgFeatNum);
            
        }
        
        return sum/(pool.size()-1);
        
        
        
    }
    
    /**
     *
     *
     * @return The cosine similarity of this Ambiguous Vector Node X to an other
     * Ambiguous Vector Node  Y .
     *
     *          Cos(x,y) = <x,y>/||x||*||y||
     *
     *          where x = (x1,,xn), y=(y1,,yn)
     *          inner product of x and y: <x,y> = x1*y1++xn*yn
     *          ||x|| = sqrt(<x,x>)
     *
     */
    public double cosineSimillarityTo(AmbiguousVector y,String category, String passFlag){
        
        String []featureVector_X = this.featureVector;
        String []featureVector_Y = y.featureVector;
        
        
        int[] featNums = getFeatureNums(passFlag, category);
        int perFeatNum = featNums[0], orgFeatNum = featNums[1];
        
        return cosineSimillarityTo(y,perFeatNum,orgFeatNum);
        
    }
    
    /**
     *
     *
     * @return The cosine similarity of this Ambiguous Vector Node X to an other
     * Ambiguous Vector Node  Y .
     *
     *          Cos(x,y) = <x,y>/||x||*||y||
     *
     *          where x = (x1,,xn), y=(y1,,yn)
     *          inner product of x and y: <x,y> = x1*y1++xn*yn
     *          ||x|| = sqrt(<x,x>)
     *
     */
    public double cosineSimillarityTo(
            AmbiguousVector y, int perFeatNum, int orgFeatNum){
        
        String []featureVector_X = this.featureVector;
        String []featureVector_Y = y.featureVector;
        
        ////////////////////////////////////////////////////////////////////////
        ///////////////////////<x,y>=x1*y1++xn*yn/////////////////////////////
        
        // <x,y> =
        double cosineSimillarity =
                innerProduct(featureVector_X, featureVector_Y, perFeatNum, orgFeatNum);
        
        // <x,y>/||x||
        // where ||x|| = sqrt(<x,x>)
        cosineSimillarity /=
                Math.sqrt(
                innerProduct(featureVector_X, featureVector_X, perFeatNum, orgFeatNum));
        
        // <x,y>/(||x||*||y||)
        // where ||y|| = sqrt(<y,y>
        cosineSimillarity /=
                Math.sqrt(
                innerProduct(featureVector_Y, featureVector_Y, perFeatNum, orgFeatNum));
        
        return cosineSimillarity;
        
    }
    
    /**
     * Computes and returns the inner product of feature vectors X and Y
     * <x,y> =x1*y1++xn*yn.
     * Features are converted to double values from String representation
     */
    private double innerProduct(
            String []featureVector_X ,String []featureVector_Y,
            int perFeatNum,int orgFeatNum ){
        
        double innerProd = 0.0;
        for (int i = 1;i < perFeatNum + 1; i++){
            // skiping "PERSON" at featureVector[0]
            innerProd += Auxiliary.atof(featureVector_X[i])*Auxiliary.atof(featureVector_Y[i]);
        }
        
        for (int i = perFeatNum + 2; i < orgFeatNum + perFeatNum + 2; i++){
            // skiping "ORGANIZATION" at featureVector[perFeatNum + 1]
            innerProd += Auxiliary.atof(featureVector_X[i])*Auxiliary.atof(featureVector_Y[i]);
        }
        
        return innerProd;
        
    }
    
    /**
     *
     *
     * @return A comparator for this class, that compares Ambiguous vector nodes
     * according to their cannonical distancePerHyperplane from the SVM hyperplane.
     * @see compare(a,b) of DistanceComparator for more
     */
    public static DistanceComparator  distComparator(){
        
        return DISTANCE_COMPARATOR;
        
    }
    
    /**
     * @return The String representation of this object
     */
    public String toString(){
        
        //features: from array to string
        StringBuffer features = new StringBuffer("");
        for(int ipos=1;ipos<featureVector.length;ipos++)
            features.append("\t" + featureVector[ipos]);
        
        String representation=
                features.toString() +
                "#" + token.toString() +
                "#" + fileName.toString() +
                "#" + Integer.toString(position) +
                "#" + Double.toString(getMinDistanceHyperplane());
        
        return representation;
        
    }
    public static AmbiguousVector retrieveFromToString(String toStringRepr){
        
        //features: from array to string
        StringTokenizer tok = new StringTokenizer(toStringRepr, "#");
        
        String features, token, fileName, position, dist;
        
        features = tok.nextToken();
        token = tok.nextToken();
        fileName = tok.nextToken();
        position = tok.nextToken();
        dist = tok.nextToken();
        
        ArrayList<String> feats = new ArrayList<String>();
        
        tok = new StringTokenizer(features, "\t");
        while (tok.hasMoreTokens()){
            
            feats.add(tok.nextToken());
            
        }
        AmbiguousVector toRet = new AmbiguousVector(fileName, Integer.parseInt(position), Double.parseDouble(dist), feats.toArray(new String[feats.size()]) );
        
        return toRet;
        
    }
    /**
     * A DistanceComparator is a comparator for AmbiguousVector objects.It implements
     * the java.util.Comparator interface.An object of this class instanciated and returned by
     * an AmbiguousVector object when DISTANCE_COMPARATOR() is invoked. It compares two AmbiguousVector
     * objects with its compare(AmbiguousVector a ,AmbiguousVector b ) method according to their
     * distances from SVM's Hyperplane. The absolute value of the canonical distancePerHyperplane for
     * a and b is used as a measure of distancePerHyperplane. "Canonical distancePerHyperplane" for each ambiguous vector node
     * equals the posibility computed by SVM for being classified as e.g. PERSON, minus the posibility
     * of not being classified as PERSON.
     *
     *
     *
     * @see ActiveLearningPhase3FirstPass.findAmbiguousVectors .
     * @see method compare(a,b) for more details
     */
    protected static class DistanceComparator implements Comparator<AmbiguousVector> {
        
        private DistanceComparator(){}
        
        /**
         * Having the absolute value of a.distancePerHyperplane subtracted by the absolute of
         * b.distancePerHyperplane, it return an integer equal to the above subtraction multiplied by
         * 10^9.
         *
         *
         *
         * @param a The first AmbiguousVector to be compared to the second b
         * @param b The second AmbiguousVector to be compared to the first a
         * @return An integer equal to the above subtraction multiplied by
         * 10^9.
         */
        public int compare(AmbiguousVector a,AmbiguousVector b){
            
            
            double dif = Math.abs(a.getMinDistanceHyperplane())-Math.abs(b.getMinDistanceHyperplane());
            return (int)(1000000000.0*dif);
            
        }
        
        
        
        
    }
    
} // end of AmbiguousVector

