/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import ipl.core.TokenNode;
import ipl.core.Tokenizer;
import java.io.Serializable;
import libsvm.svm_node;

/**
 * A feature vector which is ambiguous because the svm classifier is less sure
 * about classifing it. Objects of this class are used for filling batches in
 * active learning experiments for improoving svm classifier model.
 */
public class TestSetVector implements Serializable {
    
    /**
     * Values of features of this ambiguous feature vector.
     * Althought the type of those values is double precision, the values
     * are stored in an array in String format.
     */
    private String []perFeatureVector;
    private String []orgFeatureVector;

    /* An array of svm nodes, each node for each feature*/
    private svm_node per_node[];

    /* An array of svm nodes, each node for each feature*/
    private svm_node org_node[];

    /**
     * An array of tokens of the phrase where the token of this AmbiguousVectoreNode
     * is taken.
     */
    private String phrase[];
    /**Position of ambiguous token in phrase*/
    private int posInPhrase;
        
    /**
     * The token of this AmbiguousVectoreNode
     *
     * @see tokNode
     */
    private String token;
    
    /**Type of Tag for token eg PERSON */
    private String tagType;
    
    /**Name of the file where this.token is found*/
    private String fileName;
    
    /**Tokens before this.token in this.phrase*/
    private String previousWords[];
    
    /**Position of this.token in text*/
    private int position;
    
    
    /**
     * Constructor AmbiguousVector 
     * 
     * 
     * @param phrase 
     *      An array with all tokens of the phrase where token "word" comes from
     * @param word 
     *      Token from text
     * @param fileName 
     *      Name of the file where token comes from
     * @param previousWords 
     *      An array of tokens before token word
     * @param featureVector 
     *      An array of feature values in String format
     * @param position
     *      Position of token inside file (start from zero)
    * @param distFromHyperPlane
     *      Cannonical posibility computed by svm predicted probabilities. 
     *      This value is canonicalized in range [-1,1]. The absolute value of the 
     *      Cannonical possibillity is used as the distanceHyperplane from svm-hyperplane measure.
     */
    public  TestSetVector(
            int offset, Tokenizer tok, String fileName, String []perFeatureVector,
            String []orgFeatureVector, svm_node per_node[], svm_node org_node[]) {
        
        
        
        ///////////////////////////////////////////////////////////////////////
        /////////////////computing phrase and previous words///////////////////
        // +/- 15 tokens from the current one
        final int vicinity = 15;
        int start, end, pos = offset;
        // position in phrase returned
        int posInPhrase;
        
        if (pos - vicinity >= 0) {
        
            start = pos - vicinity;
            posInPhrase = vicinity;
            
        }
        else{
            
            start = 0;
            posInPhrase = pos;
           
        }
        
        int toksize = tok.size();
      
        if (pos + vicinity >= toksize) end = toksize - 1;
        else end = pos + vicinity;
        
        int length = end - start + 1;
        String phrase[] = new String[length];
        for (int i = 0; i < length; i++)
            phrase[i] = tok.get(start + i).getToken();
        
        // retaining previous tokens (usefull for lists)
        final int NumOfPreviousWords = 7;
        String previousWords[] = new String[NumOfPreviousWords];
        for (int i = 0; i < NumOfPreviousWords; i++)
            previousWords[i] = "";
        int j;
        pos = offset;
        if ((pos - NumOfPreviousWords) > 0)  j = pos - NumOfPreviousWords;
        else j = 0;
        
        int z = pos - 1;
        int i = 0;
        while ((z >= j) && (!tok.get(z).getEndOfPeriod()))
            previousWords[i++] = tok.get(z--).getToken().toLowerCase();
        
        
        ////////////////////////////////////////////////////////////////////////
        ////=============previouw words and phrase computed=================////
        this.phrase = phrase;
        this.posInPhrase = posInPhrase;
        this.token = tok.get(offset).getToken();
        this.tagType = tok.get(offset).getTagType();
        this.fileName = fileName;
        this.previousWords = previousWords;
        this.perFeatureVector = perFeatureVector;
        this.orgFeatureVector = orgFeatureVector;
        this.per_node = per_node;
        this.org_node = org_node;
        this.position = offset;
        
        
    } // constructor: AmbiguousVector

    public String[] getPerFeatureVector() {
        return perFeatureVector;
    }

    public String[] getOrgFeatureVector() {
        return orgFeatureVector;
    }

    public svm_node[] getPer_node() {
        return per_node;
    }

    public svm_node[] getOrg_node() {
        return org_node;
    }

    public String[] getPhrase() {
        return phrase;
    }

    public int getPosInPhrase() {
        return posInPhrase;
    }

    public String getToken() {
        return token;
    }

    public String getTagType() {
        return tagType;
    }

    public String getFileName() {
        return fileName;
    }

    public String[] getPreviousWords() {
        return previousWords;
    }

    public int getPosition() {
        return position;
    }
    

    
    
} // end of AmbiguousVector

