/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2006  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */



package ipl.nerc.features;

import ipl.util.Constants;
import ipl.core.TokenNode;
import ipl.core.Tokenizer;
import ipl.exceptions.FeatureErrorException;
import ipl.nerc.lists.ListManager;
import ipl.nerc.lists.ListNode;
import java.util.regex.*;
import java.util.Hashtable;
import java.util.Arrays;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;

public abstract class FeaturesManager {
    
    /**Seperates tokens in given text*/
    protected Tokenizer tokenizer;
    
    /**
     * An index to the "currently examined token". "Currently examined token" stands
     * for the token for which feature values are currently being extructed. 
     */
    protected int offset;
    /**A manager for handling lists. Some feature-values depend on these lists*/
    protected ListManager listMan;
    
    
    /**
     * Constructor of a FeaturesManager.
     * 
     * 
     * @param text 
     *      The text containing tokens to compute feature values
     * @param lists 
     *      ListManager for frequency lists that are used for computing some features
     * @param matcher 
     *      A matcher for maching date patterns in text
     * @param period_seperator 
     *      A period seperator (separates texts in periods)
     */
    public FeaturesManager(
            String fileName, String text, ListManager lists, TimexMatcher matcher,
            SentenceSplitter periSeparator) {
        
        this.tokenizer = new Tokenizer(fileName, text, matcher, periSeparator);
        this.offset = 0;
        this.listMan = lists;
        
    } // constructor: FeaturesManager
    
     /**
     * FeaturesManager Constructor for category organization.
     * 
     * 
     * 
     * @param tokens 
     *      A tokenizer object containing all tokens of the original text
     * @param lists 
     *       Manager for frequency lists used for some features.
     */
   public FeaturesManager(Tokenizer tokens, ListManager lists) {
        
        tokenizer = tokens;
        offset = 0;
        this.listMan = lists;
        
    } // constructor: FeaturesManager
    
    /**
     * Sets offset to zero.
     */
    public void reset() {
        this.offset = 0;
    } // reset
    
    /**
     * @return the TokenNode at position pos
     */
    public TokenNode tokenAt(int pos) {
        return tokenizer.get(pos);
    } // tokenAt
    
    /**
     * @return current offset
     */
    public int getOffset() {
        return offset;
    } // getOffset
    
    /**
     * @return The Tokenizer object used for tokenizing text. 
     */
    public Tokenizer getTokens() {
        return tokenizer;
    } // getTokens
    
    /**
     * Checks whether or not, token at position pos of text is written in Greek.
     *
     * @param pos 
     *      Position of token to be examined.
     *
     *@return "1" if token at position pos has Greek characters, else "-1"
     *
     */
    protected String isInGrChar(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String grWord = "\\p{InGreek}*";
        String tok = tokenizer.get(pos).getToken();
        
        if (Pattern.matches(grWord, tok)) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    }
    
    /**
     * Checks whether or not, token at position pos of text is written in Latin characters.
     *
     * @param pos 
     *      Position of token to be examined.
     *
     *@return "1" if token at position pos has Latin characters, else "-1"
     *
     */
   protected String isInLtChar(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String enWord = "[a-zA-Z]*";
        String tok = tokenizer.get(pos).getToken();
        
        if (Pattern.matches(enWord, tok)) {
            return "1";
        } // else if
        else {
            return "-1";
        } // else
        
    }
    
    /**
     * Checks whether token at position pos in text is a number, or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is a number, else "-1"
     */
    protected String isNumber(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String number = "[0-9]*";
        String tok = tokenizer.get(pos).getToken();
        
        if (Pattern.matches(number, tok)) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    } // isNumber
    
    /**
     * Checks whether token at position pos in text is a dot ("."), or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is a dot, else "-1"
     */
    protected String isDot(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.compareTo(".") == 0) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    } // isDot
    
    /**
     * Checks whether token at position pos in text is a comma (","), or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is a comma, else "-1"
     */
    protected String isComma(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.compareTo(",") == 0) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    } // isComma
    
    /**
     * Checks whether token at position pos in text is a slash ("/"), or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is a slash, else "-1"
     */
    protected String isSlash(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.compareTo("/") == 0) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    } // isSlash
    
    /**
     * Checks whether token at position pos in text is a dash ("-"), or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is a dash, else "-1"
     */
    protected String isDash(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.compareTo("-") == 0) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
        
    } // isDash
    
    /**
     * Checks whether token at position pos in text is an end of period , or not.
     *
     * @param pos 
     *      The position in text of the token  to be examined
     *
     * @return "1" if token at position pos is an end of period, else "-1"
     */
    protected String isEndOfPeriod(int pos) {
        
        if ( (pos < 0) || (pos >= this.tokenizer.size())) return "1";
        
        if (tokenizer.get(pos).getEndOfPeriod()) return "1";
        
        return "-1";
        
    } // isEndOfPeriod
    
    /**
     * Computes and returns a property according to length (in characters) of token at posiotion pos.
     * If length of token is greater or equal to 10, then "1" is returned. Otherwise
     * (lentgh / 10) * 2) - 1 is returned.
     *
     * @param pos 
     *      The position in text of the token  being examined by this method.
     *
     * @return The length-property (as defined above) in string
     * representation of the token at position pos .
     */
    protected String getLengthProperty(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        double l = tok.length();
        
        if (l >= 10) {
            return "1";
        } // if
        else {
            return Double.toString(((l / 10) * 2) - 1);
        } // else
    } // getLengthProperty
    
    /**
     * Checks whether or not, the first letter of token at position pos is
     * a capital letter, and the rest of the following letters are in lower case.
     *
     * @param pos The position in text of the token  being examined by this method.
     *
     * @return "1" if the first letter of token at position pos is
     * capital and the rest letters are non-capital. Else "-1" is returned
     */
    protected String isFirstCapital(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if ((tok.substring(0, 1).compareTo(tok.substring(0, 1).toUpperCase()) == 0)
        && (tok.substring(1).compareTo(tok.substring(1).toLowerCase()) == 0)) {
            return "1";
        } // if
        
        return "-1";
    } // firstCapital
    
    /**
     * Checks if all letters of the token at position pos are in upper case.
     *
     * @param 
     *      The position in text of token being examined by this method.
     *
     * @return "1" if all letters of token at position pos are capital, else "-1" is returned .
     *
     */
    protected String isCapitalized(int pos) {
        
        if (!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.compareTo(tok.toUpperCase()) == 0) {
            return "1";
        } // if
        
        int countC = 0;
        int countL = 0;
        String capLetter = "[A-Z-٢]";
        
        for (int i = 1; i < tok.length(); i++) {
            
            if (Pattern.matches(capLetter, String.valueOf(tok.charAt(i)))) {
                countC++;
            }else {
                countL++;
            } // else
            
        } // for
        if ( (countC > 0) && (countL > 0)) {
            return "1";
        } // if
        
        return "-1";
        
    } // isCapitalized
    
    /**
     * Checks if token at position pos ends with a <b>common Greek ending</b> such as :
     *
     *<i>
     * "" , "" , "" ,  "" , "" ,  "" , "" ,  "" , "" ,  "" , "" ,
     * "" , "" ,  "" , "" ,  "" , "" ,  "" , "" ,  "" , ""
     *</i>
     *
     * @param pos The position in text of the token being examined by this method.
     *
     * @return "1" if token at position pos ends with a <b>common Greek ending</b>,
     * else "-1" is returned
     */
    protected String greekEnding(int pos) {
        
        if(!checkPos(pos)) return "-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (
                tok.toLowerCase().endsWith("") || tok.endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") || tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")
                ) {
            
            return "1";
            
        }else {
            return "-1";
        } // else
    } // greekEnding
    
    
    /**
     * Checks whether or not, distanceFromStart is equal to level and it is not equal to "-1"
     *
     *
     * @return "1" or "-1", according to what described above
     */
    protected String distLessThan(int distFromStart, int level) {
        
        if (distFromStart == -1) {
            return "-1";
        } // if
        else if (distFromStart <= level) {
            return "1";
        } // else if
        else {
            return "-1";
        } // else
        
    } // distLessThan
    
    
    /**
     * Checks whether or not, token at current offset position is tagged as categName
     * @see Constants class (eg Constants.PERSON)
     *
     * return true if token is taged as categName, else false
     */
    protected boolean isInside(String categName) {
        
        if (tokenizer.get(this.getOffset()).getTagType().compareTo(categName) == 0) { 
            
            return true;
            
        } else {
            
            return false;
            
        } // else
    } // isInside
   
    /**
     * @param p 
     *      Position of token in text being examined.
     *
     * @return 
     *      The frequency of appearances in this text (how many times), of this token at position p.
     */
    protected int getFrequency(Hashtable<String, ListNode> ht, int p) {
        
        if (ht.containsKey(tokenizer.get(p).getToken().toLowerCase())) {
            
            return ht.get(tokenizer.get(p).getToken().toLowerCase()).getCount();
            
        } // if
        
        return 0;
        
    } // getFrequency
    
    protected String isInPrevList(
            Hashtable<String, ListNode> ht, int level, int dist) {
        
        int end;
        if (this.getOffset() < dist) {
            end = 0;
        } // if
        else {
            end = this.getOffset() - dist;
        } // else
        
        int count = 0;
        int i = this.getOffset() - 1;
        while ((i >= end) && (!tokenizer.get(i).getEndOfPeriod())) {
            count += getFrequency(ht, i);
            i--;
        } // while
        
        if (count < (level / 3)) {
            return "-1.0";
        } // if
        else if (count < 2 * level / 3) {
            return "-0.75";
        } // else if
        else if (count < 3 * level / 3) {
            return "-0.5";
        } // else if
        else if (count < 4 * level / 3) {
            return "-0.25";
        } // else if
        else if (count < 5 * level / 3) {
            return "0.0";
        } // else if
        else if (count < 6 * level / 3) {
            return "0.25";
        } // else if
        else if (count < 7 * level / 3) {
            return "0.5";
        } // else if
        else if (count < 8 * level / 3) {
            return "0.75";
        } // else if
        else {
            return "1.0";
        } // else
    } // isInPrevList
    
    /**
     *    token   pos      
     *   
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     *
     * @return "1"      "-1"
     */
    protected String insideSubtitle(int pos) {
        
        if (checkPos(pos)&&tokenizer.get(pos).isSubtitle()) {
            return "1";
        } // if
        else return "-1";
        
    } // insideSubtitle
    
   
    
    protected String getCategory(String type) {
        
        if (tokenizer.get(this.getOffset()).getTagType().compareTo(type) == 0) {
            
            return "1";
            
        } // if
        else {
            
            return "-1";
            
        } // else
    
    } // getCategory
    
    protected String getCategoryForOffset(String type, int offset) {
        
        if (tokenizer.get(offset).getTagType().compareTo(type) == 0) {
            
            return "1";
            
        } // if
        else {
            
            return "-1";
            
        } // else
    } // getCategory
    
    /**
     * @return "1" if and only if token token at posiotin pos is in plural. A token
     * is considered to be in plural when it ends with:
     *   "","","","","","","",""
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String inPlural(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.endsWith("") || tok.endsWith("") ||
                tok.endsWith("") ||
                tok.endsWith("") || tok.endsWith("") ||
                tok.endsWith("") || tok.endsWith("")
                || tok.endsWith("")
                ) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
    } // inPlural
    
    /**
     *  "1"       token   pos
     *   "-1"
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String isSigmaTeliko(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.toLowerCase().endsWith("")) {
            return "1";
        } // if
        else if (tok.endsWith("")) {
            return "1";
        } // else if
        else {
            return "-1";
        } // else
    } // getEndingProperty
    
    /**
     *
     *
     */
    protected String isInGenetive(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.toLowerCase().endsWith("")) {
            return "1";
        } // if
        else if (tok.toLowerCase().endsWith("")) {
            return "1";
        } // else if
        else {
            return "-1";
        } // else
    } // isInGenetive
    
    /**
     * @return "1" if token at position pos , starts with a common Greek Sirname
     * prefix such as ""("kara"),""("papa") e.t.c.
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String greekNamePrefixProperty(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // if
        
        if (
                (
                tok.toLowerCase().startsWith("") || // , 
                tok.toLowerCase().startsWith("") || // 
                tok.toLowerCase().startsWith("")// 
                )&&
                !tok.toLowerCase().startsWith("") //  
                &&
                !tok.toLowerCase().startsWith("") //  
                ) {
            return "1";
        } // if
        
        else if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // else if
        
        else if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // else if
        
        else if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // else if
        else if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // else if
        else if (tok.toLowerCase().startsWith("") ||
                tok.toLowerCase().startsWith("")) {
            return "1";
        } // else if
        
        else {
            return "-1";
        } // else
    } // greekNamePrefixProperty
    
    /**
     * Checks if token at position pos ends with a common Greek sirname suffix
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String greekNameSuffixProperty(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        } // if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().toLowerCase().endsWith("")) {
            return "1";
        } // else if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        } // else if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        }
        
        else if(
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        }
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith(""))  {
            return "1";
            
        } // else if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith(""))  {
            return "1";
            
        } // else if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith(""))  {
            return "1";
        }//else if
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        } // else if
        
        
        else if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")) {
            return "1";
        } // else if
        
        else {
            return "-1";
        } // else
        
    } // greekNameSuffixProperty
    
    
    /**
     * Checks if token at position pos ends with common Greek adjective suffixes
     * like "-", "-" etc.
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String isAdjective(int pos) {
        
        if(!checkPos(pos))return"-1";
        
        String tok = tokenizer.get(pos).getToken();
        
        if (
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")||
                //////////////////////////////////
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("") ||
                tok.toLowerCase().endsWith("")
                ) return "1";
        
        else return "-1";
        
    } // isAdjective
    
    /**
     * @return "1"  if current token is directly preceded by k. (Mr./Mrs.). Else
     * "-1" is returned
     */
    protected String isMr() {
        
        if ( (this.getOffset() == 0) || (this.getOffset() == 1)) {
            return "-1";
        } // if
        if ((tokenizer.get(this.getOffset() - 1).getToken().compareTo(".") == 0) &&
                (tokenizer.get(this.getOffset() - 2).getToken().compareTo("") == 0)) {
            return "1";
        } // if
        else {
            return "-1";
        } // else
    } // isMr
    
    /**
     *  "1"          dist tokens  
     *   token   ..  . .   "-1"
     *
     * @param dist        ..  .
     */
    protected String areMessrs(int dist) {
        int end, i;
        
        // ..
        if (this.getOffset() < dist) {
            end = 0;
        } // if
        else {
            end = this.getOffset() - dist;
        } // else
        i = this.getOffset() - 4;
        while ((i >= end) && (!tokenizer.get(i).getEndOfPeriod())) {
            for (int j = i; j < this.getOffset(); j++) {
                if ( (tokenizer.get(j).getToken().compareTo("") == 0) &&
                        (tokenizer.get(j + 1).getToken().compareTo(".") == 0) &&
                        (tokenizer.get(j + 2).getToken().compareTo("") == 0) &&
                        (tokenizer.get(j + 3).getToken().compareTo(".") == 0)) {
                    return "1";
                } // if
            } // for
            i--;
        } // while
        
        // .
        if (this.getOffset() < dist) {
            end = 0;
        } // if
        else {
            end = this.getOffset() - dist;
        } // else
        i = this.getOffset() - 2;
        while ((i >= end) && (!tokenizer.get(i).getEndOfPeriod())) {
            for (int j = i; j < this.getOffset(); j++) {
                if ( (tokenizer.get(j).getToken().compareTo("") == 0) &&
                        (tokenizer.get(j + 1).getToken().compareTo(".") == 0)) {
                    return "1";
                } // if
            } // for
            i--;
        } // while
        
        return "-1";
    } // areMessrs
    
    /**
     * Checks if the previous token from the one at position pos, starts with one
     * of the following substrings:
     * "" (e.g. ""=ministry )
     * "" (e.g. ""=stock market) e.t.c.
     * or if it is equal to:
     * "" which is used as a shortcut of "brothers" for
     * companies owned by siblings.
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     */
    protected String presidedByIndicatingTok(int pos){
        
        if (!checkPos(pos)||!checkPos(pos-1)) return "-1";
        
        String previousTok=tokenizer.get(pos-1).getToken();
        if (
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")||
                //=============================================
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")||
                previousTok.equalsIgnoreCase("")
                
                )
            return "1";
        
        else return "-1";
        
    }
    
    protected String isAFOI(int pos){
        
        if (!checkPos(pos)||!checkPos(pos-1)) return "-1";
        
        String previousTok=tokenizer.get(pos-1).getToken();
        if (previousTok.equalsIgnoreCase("")|| previousTok.equalsIgnoreCase("")
        
        )
            return "1";
        
        else return "-1";
        
    }
    
    /**
     * Checks up to 10 tokens after position pos, 10 tokens before pos, and token
     * at pos itself in text. If it finds a token  which is a common greek legal
     * form for companies, it returns a double value (in string format),
     * calculated according to the nearest distance from pos.
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     * @return values from "1.0" to "-1.0" according to distance of "", "" etc
     * from token at position pos.
     *
     * @see isfollowedByOrgLegalForm() for details about Greek CLF
     */
    protected String isCloseToCLF(int pos){
        
        //suposing (just for simplicity reasons) that we just search
        //for distance from token "AE"
        
        if(followedByCLFAt(pos-1).equals("1"))return "0.0";//is AE at pos
        
        if(followedByCLFAt(pos).equals("1"))return "0.1";//is directly followed by A.E. or AE
        if(followedByCLFAt(pos-2).equals("1"))return "-0.1";//is AE at pos-1
        
        if(followedByCLFAt(pos+1).equals("1"))return "0.2";//is AE at pos+2
        if(followedByCLFAt(pos-3).equals("1"))return "-0.2";//is AE at pos-2
        
        if(followedByCLFAt(pos+2).equals("1"))return "0.3";//is AE at pos+3
        if(followedByCLFAt(pos-4).equals("1"))return "-0.3";//is AE at pos-3
        
        if(followedByCLFAt(pos+3).equals("1"))return "0.4";//...
        if(followedByCLFAt(pos-5).equals("1"))return "-0.4";//...
        
        if(followedByCLFAt(pos+4).equals("1"))return "0.5";//...
        if(followedByCLFAt(pos-6).equals("1"))return "-0.5";//...
        
        if(followedByCLFAt(pos+5).equals("1"))return "0.6";//...
        if(followedByCLFAt(pos-7).equals("1"))return "-0.6";//...
        
        if(followedByCLFAt(pos+6).equals("1"))return "0.7";//...
        if(followedByCLFAt(pos-8).equals("1"))return "-0.7";//...
        
        if(followedByCLFAt(pos+7).equals("1"))return "0.8";//...
        if(followedByCLFAt(pos-9).equals("1"))return "-0.8";//...
        
        if(followedByCLFAt(pos+8).equals("1"))return "0.9";//is AE at pos+9
        if(followedByCLFAt(pos-10).equals("1"))return "-0.9";//is AE at pos-9
        
        if(followedByCLFAt(pos+9).equals("1"))return "1.0";//is AE at pos+10
        if(followedByCLFAt(pos-11).equals("1"))return "-1.0";//is AE at pos+10
        
        return "-1.0";//if no AE found near token at pos
        
    }
    
    /**
     * Checks if token at position pos is followed by tokens indicating a greek common
     * legal form of a company (CLF-company legal form) according to Greek Legal System.
     * A very common example of these series of tokens is ".." which may also
     * be found as "AE". It also checks for common tokens such as ""
     * following Greek company names (e.g. "   AE" which can
     * be transalated in English  as "Ioannou and Sons SA").
     * This method is private because it is not directly used for a feature. It is used
     * from method isCloseToCLF for getting a value of a feature.
     *
     * @param pos The position of
     * token in text being examined by this method.
     *
     * @see isCloseToCLF() method of this class.
     */
    private String followedByCLFAt(int pos){
        
        if(!checkPos(pos))return "-1";
        
        //an array of tokens following the one at position=pos
        //in case there is no token to be inserted in index ind this array
        //then nextTokens[ind]="noTokFound"
        String []nextTokens=new String[12];
        for (int ind=0;ind<12;ind++){
            
            if(checkPos(pos+ind+1))
                
                nextTokens[ind]=tokenizer.get(pos+ind+1).getToken();
            
            else  //if illegal position out of bounds
                
                nextTokens[ind]="noTokFound";
            
        }//for
        
        ////////////////////////////////////////////
        ////////////////AE or A.E.//////////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        ////////////////E or E...///////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                &&nextTokens[4].equals("") && nextTokens[5].equals(".")
                )
            return "1";
        
        ////////////////////////////////////////////
        //////////////// or ...///////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                &&nextTokens[4].equals("") && nextTokens[5].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        ////////////// or ....//////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                &&nextTokens[4].equals("") && nextTokens[5].equals(".")
                &&nextTokens[6].equals("") && nextTokens[7].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        ////////////// or ....//////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                &&nextTokens[4].equals("") && nextTokens[5].equals(".")
                &&nextTokens[6].equals("") && nextTokens[7].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        ///////////////// or ../////////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        ///////////////// or ../////////////////
        if(nextTokens[0].equals(""))
            return "1";
        if (
                nextTokens[0].equals("") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("") && nextTokens[3].equals(".")
                )
            return "1";
        ////////////////////////////////////////////
        /////////////////SA or S.A./////////////////
        if(nextTokens[0].equals("SA"))
            return "1";
        
        if (
                nextTokens[0].equals("S") && nextTokens[1].equals(".")
                &&nextTokens[2].equals("A") && nextTokens[3].equals(".")
                )
            return "1";
        /////////////////////////////////////////////
        //////////////////////////////////////////
        if(
                nextTokens[0].equalsIgnoreCase("")
                )
            return "1";
        
        ////////////////////////////////////////////
        ///////////////// OR ///////////////
        if(
                nextTokens[0].equalsIgnoreCase("")||
                nextTokens[0].equalsIgnoreCase("")
                )
            return "1";
        
        
        //IF NONE OF THE ABOVE
        return "-1";
        
    }
    
    
    
    ////////////////////////////////////////////////////////////////////////////
    //////////////////////// Second Pass features //////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    
     /**
     *  "1"    token   position    words
     *   "-1"
     * @param words   
     * @param position    token
     *
     * @return   "1"  "-1"       position    
     *              words, 
     */
    protected String isInList(String[] words, int position) {
        
        if (!checkPos(position)) return "-1.0";
        
        if (words.length == 0) {
            return "-1.0";
        } // if
        
        String temp, token = tokenizer.get(position).getToken().toLowerCase();
        int pos = Arrays.binarySearch(words, token);
        //   token   
        if (pos >= 0) {
            return "1";
        } // if
        
        //   token          
        //   .         
        //       token    .
        pos = java.lang.Math.abs(pos) - 1;
        
        //    
        float res1 = -1;
        if (pos > 0) {
            temp = words[pos - 1];
            int length;
            if (temp.length() < token.length()) {
                length = temp.length();
            } // if
            else {
                length = token.length();
            } // else
            int i = 0;
            boolean end = false;
            while ((i < length) && !end) {
                if (token.charAt(i) != temp.charAt(i)) {
                    end = true;
                } // if
                else {
                    i++;
                } // else
            } // while
            if (i != 0) {
                res1 = (float) ((2.0 * (float) i / temp.length()) - 1.0);
            } // if
        } // if
        
        //    
        float res2 = -1;
        if (pos < words.length) {
            temp = words[pos];
            int length;
            if (temp.length() < token.length()) {
                length = temp.length();
            } // if
            else {
                length = token.length();
            } // else
            int i = 0;
            boolean end = false;
            while ((i < length) && !end) {
                if (token.charAt(i) != temp.charAt(i)) {
                    end = true;
                } // if
                else {
                    i++;
                } // else
            } // while
            if (i != 0) {
                res2 = (float) ((2.0 * (float) i / temp.length()) - 1.0);
            } // if
        } // if
        
        return Float.toString(Math.max(res1, res2));
        
    } // isInList
    
     /**
     * @param  pos
     *      Position of token under examination in file text fileName (starting from 0).
     * @param allFpDecisions
     *      A hashtable containing decisions of FP classifiers. Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text
     * @return First pass probability assigned by person svm for token at position pos in text file fileName.
     */
    protected String personFPProbability(int pos, Hashtable<String, String> allFpDecisions, String fileName) {
        
        if (checkPos(pos)){
            
            String key = Constants.PERSON + "#" + fileName + "#" + pos;
            String probability = allFpDecisions.get(key);
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String personFPProbability(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            return probability;
            
        } else return "-1";
        
    } // getTokenFirstPassProbability
    
    /**
     * @param  pos
     *      Position of token under examination in file text fileName (starting from 0).
     * @param allFpDecisions
     *      A hashtable containing decisions of FP classifiers. Hashtable keys must be like:<br>
     *      Constants.ORGANIZATION + "#" + Name_of_text_file + "#" + position_in_text
     * @return First pass probability assigned by person svm for token at position pos in text file fileName.
     */
    protected String organizationFPProbability(int pos, Hashtable<String, String> allFpDecisions, String fileName) {
        
        if (checkPos(pos)){
            
            // Constants.PERSON + "#" + currFile.getName() + "#" + (perFPVectors.getOffset() - 1)
            String key = Constants.ORGANIZATION + "#" + fileName + "#" + pos;
            String probability = allFpDecisions.get(key);
            
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String organizationFPProbability(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
            
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
           
            return probability;
            
        } else return "-1";
        
    } // getTokenFirstPassProbability
    
    protected String entityFPProbability(int pos, String[] allFpDecisions) {
        
        if (checkPos(pos)){
            
            String probability = allFpDecisions[pos];
            
            return probability;
            
        } else return "-1";
        
    } // getTokenFirstPassProbability
    
    /**
     * @param pos
     *      Position of token in text.
     * @param allFpDecisions
     *      All decisions of first pass classifiers in a hashtable. The hashtable uses
     *      as keys values such as Constants.ORGANIZATION + "#" + fileName + "#" + pos.
     * @param fileName
     *      Name of the text file containing this token.
     *
     * @return
     * A double from -1.0 to 1.0, indicating the distance from the "nearest
     * organization" before token at position pos. <br>
     * (Nearest Organization: If cannonical possibility
     * assigned by first pass classifier for category organizatnion is greater than 0.75,
     * then token is concidered as organization).
     *
     *
     */
    public String distanceFromPrevOrg(int pos, Hashtable<String, String> allFpDecisions, String fileName, double margin){
        
        if(!checkPos(pos)) return "-1.0";
        
        int i =  0;
        while (i < 11){
            
            if(!checkPos(pos - i)) return "-1.0";
            String key = Constants.ORGANIZATION + "#" + fileName + "#" + (pos - i);
            String probability = allFpDecisions.get(key);
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String distanceFromPrevOrg(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
                
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            double num = Double.parseDouble(probability);
            if(num > margin) return String.valueOf(1.0 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
    
    /**
     * @param pos
     *      Position of token in text.
     * @param allFpDecisions
     *      All decisions of first pass classifiers in a hashtable. The hashtable uses
     *      as keys values such as Constants.PERSON + "#" + fileName + "#" + pos.
     * @param fileName
     *      Name of the text file containing this token.
     *
     * @return
     * A double from -1.0 to 1.0, indicating the distance from the "nearest
     * person name" before token at position pos. <br>
     * (Nearest person name: If cannonical possibility
     * assigned by first pass classifier for category PERSON is greater than 0.75,
     * then token is concidered as PERSON).
     *
     *
     */
    public String distanceFromPrevPer(int pos, Hashtable<String, String> allFpDecisions, String fileName, double margin ){
        
        if(!checkPos(pos)) return "-1";
        
        int i =  0;
        while (i < 11){
            
            if(!checkPos(pos - i)) return "-1.0";
            String key = Constants.PERSON + "#" + fileName + "#" + (pos - i);
            String probability = allFpDecisions.get(key);
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String distanceFromPrevPer(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
                
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            double num = Double.parseDouble(probability);
           
            if(num > margin) return String.valueOf(1.2 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
    
    public String distanceFromPrevEntity(int pos, String[] allFpDecisions,double margin ){
        
        if(!checkPos(pos)) return "-1";
        
        int i =  0;
        while (i < 11){
            
            if(!checkPos(pos - i)) return "-1.0";
            String probability = allFpDecisions[pos];
            
            double num = Double.parseDouble(probability);
           
            if(num > margin) return String.valueOf(1.2 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
    
    /**
     * @param pos
     *      Position of token in text.
     * @param allFpDecisions
     *      All decisions of first pass classifiers in a hashtable. The hashtable uses
     *      as keys values such as Constants.ORGANIZATION + "#" + fileName + "#" + pos.
     * @param fileName
     *      Name of the text file containing this token.
     *
     * @return
     * A double from -1.0 to 1.0, indicating the distance from the "nearest
     * organization" after token at position pos. <br>
     * (Nearest Organization: If cannonical possibility
     * assigned by first pass classifier for category organizatnion is greater than 0.75,
     * then token is concidered as organization).
     *
     *
     */
    public String distanceFromFollowingOrg(int pos, Hashtable<String, String> allFpDecisions, String fileName, double margin){
        
        if(!checkPos(pos)) return "-1.0";
        
        int i =  1;
        while (i < 11){
            
            if(!checkPos(pos + i)) return "-1.0";
            String key = Constants.ORGANIZATION + "#" + fileName + "#" + (pos + i);
            String probability = allFpDecisions.get(key);
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String distanceFromFollowingOrg(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
                
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            double num = Double.parseDouble(probability);
           
            if(num > margin) return String.valueOf(1.2 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
    
    /**
     * @param pos
     *      Position of token in text.
     * @param allFpDecisions
     *      All decisions of first pass classifiers in a hashtable. The hashtable uses
     *      as keys values such as Constants.PERSON + "#" + fileName + "#" + pos.
     * @param fileName
     *      Name of the text file containing this token.
     *
     * @return
     * A double from -1.0 to 1.0, indicating the distance from the "nearest
     * person name" after token at position pos. <br>
     * (Nearest person name: If cannonical possibility
     * assigned by first pass classifier for category PERSON is greater than 0.75,
     * then token is concidered as PERSON).
     *
     *
     */
    public String distanceFromFollowingPer(int pos, Hashtable<String, String> allFpDecisions, String fileName, double margin){
        
        if(!checkPos(pos))return "-1";
        
        int i =  1;
        while (i < 11){
            
            if(!checkPos(pos + i)) return "-1.0";
            String key = Constants.PERSON + "#" + fileName + "#" + (pos + i);
            String probability = allFpDecisions.get(key);
             try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String distanceFromFollowingPer(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
                
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            double num = Double.parseDouble(probability);
            
            if(num > margin) return String.valueOf(1.2 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
    
    public String distanceFromFollowingEntity(int pos, String[] allFpDecisions, double margin){
        
        if(!checkPos(pos)) return "-1.0";
        
        int i =  1;
        while (i < 11){
            
            if(!checkPos(pos + i)) return "-1.0";
            String probability = allFpDecisions[pos];
            try {
                
                if (probability == null) {
                    throw new FeatureErrorException("Null previous pass probability: Hashtable of probabilities assigned by\n" +
                            "first pass classifiers, contains null probality values. Please check if active learning files pool matches the one used for \n" +
                            "constructing hashtable allFpDecisions (parameter of method protected String distanceFromFollowingOrg(...)), or recompute previous pass decisions\n" +
                            "hashtable for this active learning files pool");
                }
                
            } catch (FeatureErrorException ex) {
                ex.printStackTrace();
            }
            double num = Double.parseDouble(probability);
           
            if(num > margin) return String.valueOf(1.2 - 0.2*i);
            i++;
            
        }
        
        return "-1.0";
        
    }
   
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    /**
     * checks if position pos is illegal. Illigal means that pos may either be
     * negative, greater than the position of the last token of the text, or there
     * is an end-of-period between the curently indexed token (token at offset) and
     * token at pos.
     *
     * @param pos The position to be checked
     *
     * @return true if pos found legal, else false
     */
    private  boolean checkPos(int pos){
        
        //pos out of bounds
        if ( (pos < 0) || (pos >= this.tokenizer.size())) {
            return false;
        } // if
        
        //in case pos<offset and between pos and offset there is an-end-of-period
        //then token at pos cannot be related to token at offset
        if (pos < this.getOffset()) {
            
            for (int i = pos; i < this.getOffset(); i++) {
                
                if (tokenizer.get(i).getEndOfPeriod()) {
                    
                    return false;
                    
                } // if
            } // for
        } else
            
            //in case pos>offset and between pos and offset there is an end of period
            //then token at pos cannot be related to token at offset
            if (pos > this.getOffset()) {
            
            for (int i = this.getOffset(); i <= pos; i++) {
                
                if (tokenizer.get(i).getEndOfPeriod()) {
                    
                    return false;
                    
                } // if
            } // for
            } // else if
        //if nothing of the above, then everything is ok
        return true;
        
    }//checkPos(int)
   
    /*
     *
     *Getting feature vectors
     *
     */
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of feature values for the next token (token at position offset in text). Each subclass specializes in a
     * specific category such as organizations. Each subclass may use only a subset
     * of features described by methods of this class in order to return the feature-array.
     * (to be used for first pass only).
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @return An array of feature values for the next token.
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     *
     */
    public String[] nextFeatVec(int distFromStart){
        
         
        return  nextFeatVec(distFromStart, null, null, null);
        
    }
    
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of features values for the token at position posIntext. Each subclass specializes in a
     * specific category such as organizations. Each subclass may use only a subset
     * of features described by methods of this class in order to return the feature-array.
     * (to be used for first pass only).
     *
     * @param posInText
     *      Position of token in text (starting from 0).
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @return An array of feature values for the token.
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     *
     */
    public String[] featureVectorAt(int posInText, int distFromStart){
        
        return  featureVectorAt(posInText, distFromStart, null, null, null);
        
    }
    
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of features values for the previous token (token at offset-1). Each subclass specializes in a
     * specific category such as organizations. Each subclass may use only a subset
     * of features described by methods of this class in order to return the feature-array.
     * (to be used for first pass only).
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @return An array of feature values for the previous token in text,
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     *
     */
    public String[] prevFeatVec_noSafeRules(int distFromStart){
        
        return  prevFeatVec_noSafeRules(distFromStart, null, null, null);
        
    }
    
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of features values for the next token (token at offset). Each subclass specializes in a
     * specific category such as organizations. Each subclass may use only a subset
     * of features described by methods of this class in order to return the feature-array.
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @param sureDec
     *      (Null if first pass)
     *      A hashtable containing sure decisions of first pass classifiers (eg decisions with cannoncal posibility>0.8).
     *      Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text eg PERSON#text01.txt#231
     *
     * @param allFpDecisions
     *      (Null if first pass)
     *      All decisions of first pass classifiers (cannonical possibilties assgigned)in a hashtable. The hashtable uses
     *      key values such as Constants.PERSON + "#" + fileName + "#" + pos.
     *
     * @param filename
     *      Name of the file of the text.
     *  
     * @return An array of feature values for the next token in text,
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     */
    public abstract String[] nextFeatVec(
            int distFromStart, Hashtable<String, String[]> sureDec, Hashtable<String, String> allFpDecisions, String filename);
    
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of features values for the previous token (token at offset-1), without appling 
     * safe rules. Each subclass specializes in a specific category such as organizations.
     * Each subclass may use only a subset of features described by methods of this class,
     * in order to return the feature-array.
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @param sureDec
     *      (Null if first pass)
     *      A hashtable containing sure decisions of first pass classifiers (eg decisions with cannoncal posibility>0.8).
     *      Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text eg PERSON#text01.txt#231
     *
     * @param allFpDecisions
     *      (Null if first pass)
     *      All decisions of first pass classifiers (cannonical possibilties assgigned)in a hashtable. The hashtable uses
     *      key values such as Constants.PERSON + "#" + fileName + "#" + pos.
     *
     * @param filename
     *      Name of the file of the text.
     *  
     * @return An array of feature values for the previous token in text,
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     */
    public  abstract String[] prevFeatVec_noSafeRules(
            int distFromStart, Hashtable<String, String[]> sureDec, Hashtable<String, String> allFpDecisions, String filename);
    
    /**
     * Abstract method overiden by subclasses in order to contruct and return
     * the array of features values for token at position posInText. 
     * Each subclass specializes in a specific category such as organizations.
     * Each subclass may use only a subset of features described by methods of this class,
     * in order to return the feature-array.
     * 
     * @param posInText
     *      Position of token in text (starting from 0).      
     *     
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     * 
     * @param sureDec
     *      (Null if first pass)
     *      A hashtable containing sure decisions of first pass classifiers (eg decisions with cannoncal posibility>0.8).
     *      Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text eg PERSON#text01.txt#231
     * 
     * @param allFpDecisions
     *      (Null if first pass)
     *      All decisions of first pass classifiers (cannonical possibilties assgigned)in a hashtable. The hashtable uses
     *      key values such as Constants.PERSON + "#" + fileName + "#" + pos.
     * 
     * @param filename
     *      Name of the file of the text.
     * 
     * @return An array of feature values for token at position "posInText" in text,
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     */
    public abstract String[] featureVectorAt(
            int posInText, int distFromStart, Hashtable<String, String[]> sureDec, Hashtable<String, String> allFpDecisions, String filename);
    
} // end of class FeaturesManager
