/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.features;

import ipl.util.Constants;
import ipl.core.Tokenizer;
import ipl.splitter.SentenceSplitter;
import ipl.util.Auxiliary;
import ipl.nerc.lists.ListManager;
import ipl.nerc.regex.timex.TimexMatcher;
import java.util.Hashtable;

/*
 *- apostash apo allo proswpo/organismo
 *- listes(opws kai gia kathe kathgoria) me none (Arxiepiskopos , pro8hpourgos, tsampions ling ktl)
 *- yparxei idiothta gia " ?
 *
 */
public class PersonFeaturesManager extends FeaturesManager {
    
    
    /**
     * Constructor of a FeaturesManager for category Person.
     * 
     * 
     * @param text 
     *      The text containing tokens to compute feature values
     * @param lists 
     *      ListManager for frequency lists that are used for computing some features
     * @param matcher 
     *      A matcher for maching date patterns in text
     * @param period_seperator 
     *      A period seperator (separates texts in periods)
     */
    public PersonFeaturesManager(
            String fileName, String text, ListManager lists, TimexMatcher matcher,
            SentenceSplitter period_separator) {
        
        super(fileName, text, lists, matcher, period_separator);
        
    } // constructor: PersonFeaturesManager
 
       
    /**
     * FeaturesManager Constructor for category Person.
     * 
     * 
     * 
     * @param tokens 
     *      A tokenizer object containing all tokens of the original text
     * @param lists 
     *       Manager for frequency lists used for some features.
     */
    public PersonFeaturesManager(Tokenizer tokens, ListManager lists) {
        super(tokens, lists);
    } // constructor: PersonFeaturesManager
    
       
    /**
     * 
     * Contructs and returns the array of feature values for the next token (token at position offset in text).
     * Features are computed for category Person.
     * (to be used for first pass only).
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @return An array of feature values for the next token.
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     *
     */
    public String[] nextFeatVec(
            int distFromStart, Hashtable<String, String[]> sureDec,
            Hashtable<String, String> allDec, String fileName) {
        
        String perFPSureList[], orgFPSureList[];
        
        // if there are no more tokens then return null
        if (getOffset() == tokenizer.size()) {
            return null; 
        } // if
        
        String category = getCategory(Constants.PERSON);
        String currentToken = tokenizer.get(getOffset()).getToken();
        
        ////////////////////////////////////////////////////////////////////////
        //////////skiping examination of this token at position pos/////////////
        ////////////////////////////////////////////////////////////////////////
        
        //skip dates and time by returning just the following:
        if (isInside(Constants.DATE) || isInside(Constants.TIME)) {
            
            offset++; 
            return (new String[] {category});
            
        } // if
        
        // if distFromStart == -1 or in other words we have not found an other
        // part of entity of this category before and close to this token. So we skip it
        // in the following cases:
        if (distFromStart == -1) {
            
            //if it isn't Greek nor English word skip it
            if ((isInGrChar(getOffset()).compareTo("1") != 0) &&
                    (isInLtChar(getOffset()).compareTo("1") != 0)) {
                offset++;
                return (new String[] {category});
            } // if
            
            if ( (isFirstCapital(getOffset()).compareTo("1") != 0) &&
                    (isCapitalized(getOffset()).compareTo("1") != 0)) {
                offset++;
                return (new String[] {category});
            } // if
            
            //if it is a Greek verb then skip it
            if (isInGrChar(getOffset()).compareTo("1") == 0) {
                if (currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("")) {
                    offset++;
                    return (new String[] {category});
                } // if
                
                //if it is stop-word(inside safe list) skip it
                 if (listMan.getSafeList().containsKey(currentToken.toLowerCase())) {
                    offset++;
                    return (new String[] {category});
                } // if
            } // if
        } // if
        //=================================================================================
        ////////cases of skiping examination of current token at position pos//////////////
        //=================================================================================
        
        /**
         * table of feature value to be returned
         */
        String[] toReturn = new String[256];
        //enought size to ensure no out of bounds exception to be thrown
        
        int attributeNum = 0;
        toReturn[attributeNum++] = category;
        
        
        //for a window of 5 tokens (this means the token before the previous one,
        //the previous token, the current token, the next token and the one after the next)
        // fill in those string feature values (see each feature in comments)
        String isgrw,//is
                isenw, //is greek word
                isn,//is a number
                isd,//is a dot "."
                isc,//is a comma ","
                
                isDash,//is token at pos a dash?
                
                isSlash,//is token at pos a slash?
                isa,//is an adjective
                iseop,//is it end of period (not just a common dot)
                len,//length
                fcap,//is first letter capital
                cap,//all letters in capital
                
                sigmaTel,//ends with "sigma teliko" -->""
                
                isIngen,//is in genetive
                grSuffix,//has a commmon greek sirname suffix
                grPrefix,//has a commmon greek sirname prefix
                grEnding, //common Greek last character?
                inp,//is it in plural?
                
                isAfoi,//precided by eg ""
                
                indicatingTok,//tokens such as "..." etc
                
                distCLF;//a numeric feature representing how distant a token
        //is from e.g. AE, E e.t.c.(only for current token)
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-3 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 3);
        isenw = isInLtChar(getOffset() - 3);
        isd = isDot(getOffset() - 3);
        isn = isNumber(getOffset() - 3);
        isc = isComma(getOffset() - 3);
        iseop = isEndOfPeriod(getOffset() - 3);
        len = getLengthProperty(getOffset() - 3);
        isSlash = isSlash(getOffset() - 3);
        isDash = isDash(getOffset() - 3);
        isAfoi=isAFOI(getOffset() - 3);
        indicatingTok=presidedByIndicatingTok(getOffset() - 3);
        distCLF=isCloseToCLF(getOffset() - 3);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 3);
            isa = isAdjective(getOffset() - 3);
            fcap = isFirstCapital(getOffset() - 3);
            cap = isCapitalized(getOffset() - 3);
            grSuffix = greekNameSuffixProperty(getOffset() - 3);
            grPrefix = greekNamePrefixProperty(getOffset() - 3);
            grEnding = greekEnding(getOffset() - 3);
            sigmaTel = isSigmaTeliko(getOffset() - 3);
            isIngen=isInGenetive(getOffset() - 3);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 3);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        // in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            // toReturn[attributeNum++] =
               //     personFPProbability(offset - 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() - 3, allDec, fileName);
            
            // toReturn[attributeNum++] =
                //    isInList(perFPSureList, offset - 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset - 3);
            
            
            int forPos = getOffset() - 3;
            
             toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
              //      distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-2 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 2);
        isenw = isInLtChar(getOffset() - 2);
        isd = isDot(getOffset() - 2);
        isn = isNumber(getOffset() - 2);
        isc = isComma(getOffset() - 2);
        iseop = isEndOfPeriod(getOffset() - 2);
        len = getLengthProperty(getOffset() - 2);
        isSlash = isSlash(getOffset() - 2);
        isDash = isDash(getOffset() - 2);
        isAfoi=isAFOI(getOffset() - 2);
        indicatingTok=presidedByIndicatingTok(getOffset() - 2);
        distCLF=isCloseToCLF(getOffset() - 2);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 2);
            isa = isAdjective(getOffset() - 2);
            fcap = isFirstCapital(getOffset() - 2);
            cap = isCapitalized(getOffset() - 2);
            grSuffix = greekNameSuffixProperty(getOffset() - 2);
            grPrefix = greekNamePrefixProperty(getOffset() - 2);
            grEnding = greekEnding(getOffset() - 2);
            sigmaTel = isSigmaTeliko(getOffset() - 2);
            isIngen=isInGenetive(getOffset() - 2);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 2);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        //toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        
        //in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset() - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() - 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() - 2);
            
            
            int forPos = getOffset() - 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance -1////////////////////
        //////////////////from current token (previous token)///////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 1);
        isenw = isInLtChar(getOffset() - 1);
        isd = isDot(getOffset() - 1);
        isn = isNumber(getOffset() - 1);
        isc = isComma(getOffset() - 1);
        iseop = isEndOfPeriod(getOffset() - 1);
        len = getLengthProperty(getOffset() - 1);
        isSlash = isSlash(getOffset() - 1);
        isDash = isDash(getOffset() - 1);
        isAfoi=isAFOI(getOffset() - 1);
        indicatingTok=presidedByIndicatingTok(getOffset() - 1);
        distCLF=isCloseToCLF(getOffset() - 1);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 1);
            isa = isAdjective(getOffset() - 1);
            fcap = isFirstCapital(getOffset() - 1);
            cap = isCapitalized(getOffset() - 1);
            grSuffix = greekNameSuffixProperty(getOffset() - 1);
            grPrefix = greekNamePrefixProperty(getOffset() - 1);
            grEnding = greekEnding(getOffset() - 1);
            sigmaTel = isSigmaTeliko(getOffset() - 1);
            isIngen=isInGenetive(getOffset() - 1);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 1);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset() - 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() - 1, allDec, fileName);
            
            // toReturn[attributeNum++] =
               //     isInList(perFPSureList, offset - 1);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() - 1);
            
            int forPos = getOffset() - 1;
            
            // toReturn[attributeNum++] =
                 //   distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        //////////////current token feature values (distance = 0 )//////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset());
        isenw = isInLtChar(getOffset());
        isd = isDot(getOffset());
        isn = isNumber(getOffset());
        isc = isComma(getOffset());
        iseop = isEndOfPeriod(getOffset());
        len = getLengthProperty(getOffset());
        isSlash = isSlash(getOffset());
        isDash = isDash(getOffset());
        isAfoi=isAFOI(getOffset());
        indicatingTok=presidedByIndicatingTok(getOffset());
        distCLF=isCloseToCLF(getOffset());
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset());
            isa = isAdjective(getOffset());
            fcap = isFirstCapital(getOffset());
            cap = isCapitalized(getOffset());
            grSuffix = greekNameSuffixProperty(getOffset());
            grPrefix = greekNamePrefixProperty(getOffset());
            grEnding = greekEnding(getOffset());
            sigmaTel = isSigmaTeliko(getOffset());
            isIngen=isInGenetive(getOffset());
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        // toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), getOffset());
        toReturn[attributeNum++] = insideSubtitle(getOffset());
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        // toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset(), allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset(), allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset());
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset);
            
            int forPos = getOffset();
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +1////////////////////
        ////////////////////from current token (next token)/////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() + 1 );
        isenw = isInLtChar(getOffset() + 1 );
        isd = isDot(getOffset() + 1 );
        isn = isNumber(getOffset() + 1 );
        isc = isComma(getOffset() + 1 );
        iseop = isEndOfPeriod(getOffset() + 1 );
        len = getLengthProperty(getOffset() + 1 );
        isSlash = isSlash(getOffset() + 1 );
        isDash = isDash(getOffset() + 1 );
        isAfoi=isAFOI(getOffset() + 1 );
        indicatingTok=presidedByIndicatingTok(getOffset() + 1 );
        distCLF=isCloseToCLF(getOffset() + 1 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() + 1 );
            isa = isAdjective(getOffset() + 1 );
            fcap = isFirstCapital(getOffset() + 1 );
            cap = isCapitalized(getOffset() + 1 );
            grSuffix = greekNameSuffixProperty(getOffset() + 1 );
            grPrefix = greekNamePrefixProperty(getOffset() + 1 );
            grEnding = greekEnding(getOffset() + 1 );
            sigmaTel = isSigmaTeliko(getOffset() + 1 );
            isIngen=isInGenetive(getOffset() + 1 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() + 1);
        toReturn[attributeNum++] = insideSubtitle(getOffset() + 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset() + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 1);
                    
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() + 1);
            
            int forPos = getOffset() + 1;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +2////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() +2 );
        isenw = isInLtChar(getOffset() +2 );
        isd = isDot(getOffset() +2 );
        isn = isNumber(getOffset() +2 );
        isc = isComma(getOffset() +2 );
        iseop = isEndOfPeriod(getOffset() +2 );
        len = getLengthProperty(getOffset() +2 );
        isSlash = isSlash(getOffset() +2 );
        isDash = isDash(getOffset() +2 );
        isAfoi=isAFOI(getOffset() +2 );
        indicatingTok=presidedByIndicatingTok(getOffset() +2 );
        distCLF=isCloseToCLF(getOffset() +2 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() +2 );
            isa = isAdjective(getOffset() +2 );
            fcap = isFirstCapital(getOffset() +2 );
            cap = isCapitalized(getOffset() +2 );
            grSuffix = greekNameSuffixProperty(getOffset() +2 );
            grPrefix = greekNamePrefixProperty(getOffset() +2 );
            grEnding = greekEnding(getOffset() +2 );
            sigmaTel = isSigmaTeliko(getOffset() +2 );
            isIngen=isInGenetive(getOffset() +2 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        //toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),offset +2);
        toReturn[attributeNum++] = insideSubtitle(getOffset() +2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        //toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset() + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() + 2);
            
            int forPos = getOffset() + 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +3////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() + 3 );
        isenw = isInLtChar(getOffset() + 3 );
        isd = isDot(getOffset() + 3 );
        isn = isNumber(getOffset() + 3 );
        isc = isComma(getOffset() + 3 );
        iseop = isEndOfPeriod(getOffset() + 3 );
        len = getLengthProperty(getOffset() + 3 );
        isSlash = isSlash(getOffset() + 3 );
        isDash = isDash(getOffset() + 3 );
        isAfoi=isAFOI(getOffset() + 3 );
        indicatingTok=presidedByIndicatingTok(getOffset() + 3 );
        distCLF=isCloseToCLF(getOffset() + 3 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() + 3 );
            isa = isAdjective(getOffset() + 3 );
            fcap = isFirstCapital(getOffset() + 3 );
            cap = isCapitalized(getOffset() + 3 );
            grSuffix = greekNameSuffixProperty(getOffset() + 3 );
            grPrefix = greekNamePrefixProperty(getOffset() + 3 );
            grEnding = greekEnding(getOffset() + 3 );
            sigmaTel = isSigmaTeliko(getOffset() + 3 );
            isIngen=isInGenetive(getOffset() + 3 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), getOffset() + 3);
        toReturn[attributeNum++] = insideSubtitle(getOffset() + 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        // toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(getOffset() + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(getOffset() + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset + 3);
            
            int forPos = getOffset() + 3;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        toReturn[attributeNum++] = isMr();
        toReturn[attributeNum++] = areMessrs(10);
        
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(0, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(0, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(1, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(2, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(3, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(4, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(5, Constants.PERSON), 1);
        
//        toReturn[attributeNum++] = isInPrevList(
//                listMan.getRepeatedWordsList(0, Finals.ORGANIZATION),
//                listMan.getAvgOfRepeatedWordsList(0, Finals.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(1, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(2, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(3, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(4, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(5, Constants.ORGANIZATION), 1);
        
        toReturn[attributeNum++] = distLessThan(distFromStart, 1);
        toReturn[attributeNum++] = distLessThan(distFromStart, 2);
        toReturn[attributeNum++] = distLessThan(distFromStart, 3);
        toReturn[attributeNum++] = distLessThan(distFromStart, 4);
        toReturn[attributeNum++] = distLessThan(distFromStart, 5);
        toReturn[attributeNum++] = distLessThan(distFromStart, 6);
        toReturn[attributeNum++] = distLessThan(distFromStart, 7);
        
        offset++;//proceed to next token
      
        return Auxiliary.getSubList(toReturn, attributeNum);
            
    } // getNextFeatureVector
    
    
     public String[] nextFeatVec(int distFromStart, String[] perFPSureList, String[] orgFPSureList,
            String[] allFpPerDecisionsArr, String[] allFpOrgDecisionsArr) {
                
        // if there are no more tokens then return null
        if (getOffset() == tokenizer.size()) {
            return null; 
        } // if
        
        String category = getCategory(Constants.PERSON);
        String currentToken = tokenizer.get(getOffset()).getToken();
        
        ////////////////////////////////////////////////////////////////////////
        //////////skiping examination of this token at position pos/////////////
        ////////////////////////////////////////////////////////////////////////
        
        //skip dates and time by returning just the following:
        if (isInside(Constants.DATE) || isInside(Constants.TIME)) {
            
            offset++; 
            return (new String[] {category});
            
        } // if
        
        // if distFromStart == -1 or in other words we have not found an other
        // part of entity of this category before and close to this token. So we skip it
        // in the following cases:
        if (distFromStart == -1) {
            
            //if it isn't Greek nor English word skip it
            if ((isInGrChar(getOffset()).compareTo("1") != 0) &&
                    (isInLtChar(getOffset()).compareTo("1") != 0)) {
                offset++;
                return (new String[] {category});
            } // if
            
            if ( (isFirstCapital(getOffset()).compareTo("1") != 0) &&
                    (isCapitalized(getOffset()).compareTo("1") != 0)) {
                offset++;
                return (new String[] {category});
            } // if
            
            //if it is a Greek verb then skip it
            if (isInGrChar(getOffset()).compareTo("1") == 0) {
                if (currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("")) {
                    offset++;
                    return (new String[] {category});
                } // if
                
                //if it is stop-word(inside safe list) skip it
                 if (listMan.getSafeList().containsKey(currentToken.toLowerCase())) {
                    offset++;
                    return (new String[] {category});
                } // if
            } // if
        } // if
        //=================================================================================
        ////////cases of skiping examination of current token at position pos//////////////
        //=================================================================================
        
        /**
         * table of feature value to be returned
         */
        String[] toReturn = new String[256];
        //enought size to ensure no out of bounds exception to be thrown
        
        int attributeNum = 0;
        toReturn[attributeNum++] = category;
        
        
        //for a window of 5 tokens (this means the token before the previous one,
        //the previous token, the current token, the next token and the one after the next)
        // fill in those string feature values (see each feature in comments)
        String isgrw,//is
                isenw, //is greek word
                isn,//is a number
                isd,//is a dot "."
                isc,//is a comma ","
                
                isDash,//is token at pos a dash?
                
                isSlash,//is token at pos a slash?
                isa,//is an adjective
                iseop,//is it end of period (not just a common dot)
                len,//length
                fcap,//is first letter capital
                cap,//all letters in capital
                
                sigmaTel,//ends with "sigma teliko" -->""
                
                isIngen,//is in genetive
                grSuffix,//has a commmon greek sirname suffix
                grPrefix,//has a commmon greek sirname prefix
                grEnding, //common Greek last character?
                inp,//is it in plural?
                
                isAfoi,//precided by eg ""
                
                indicatingTok,//tokens such as "..." etc
                
                distCLF;//a numeric feature representing how distant a token
        //is from e.g. AE, E e.t.c.(only for current token)
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-3 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 3);
        isenw = isInLtChar(getOffset() - 3);
        isd = isDot(getOffset() - 3);
        isn = isNumber(getOffset() - 3);
        isc = isComma(getOffset() - 3);
        iseop = isEndOfPeriod(getOffset() - 3);
        len = getLengthProperty(getOffset() - 3);
        isSlash = isSlash(getOffset() - 3);
        isDash = isDash(getOffset() - 3);
        isAfoi=isAFOI(getOffset() - 3);
        indicatingTok=presidedByIndicatingTok(getOffset() - 3);
        distCLF=isCloseToCLF(getOffset() - 3);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 3);
            isa = isAdjective(getOffset() - 3);
            fcap = isFirstCapital(getOffset() - 3);
            cap = isCapitalized(getOffset() - 3);
            grSuffix = greekNameSuffixProperty(getOffset() - 3);
            grPrefix = greekNamePrefixProperty(getOffset() - 3);
            grEnding = greekEnding(getOffset() - 3);
            sigmaTel = isSigmaTeliko(getOffset() - 3);
            isIngen=isInGenetive(getOffset() - 3);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 3);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        // in case we are in second pass experiment
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null) {
            
            // toReturn[attributeNum++] =
               //     entityFPProbability(offset - 3, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() - 3, allFpOrgDecisionsArr);
            
            // toReturn[attributeNum++] =
                //    isInList(perFPSureList, offset - 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset - 3);
            
            
            int forPos = getOffset() - 3;
            
             toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
              //      distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-2 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 2);
        isenw = isInLtChar(getOffset() - 2);
        isd = isDot(getOffset() - 2);
        isn = isNumber(getOffset() - 2);
        isc = isComma(getOffset() - 2);
        iseop = isEndOfPeriod(getOffset() - 2);
        len = getLengthProperty(getOffset() - 2);
        isSlash = isSlash(getOffset() - 2);
        isDash = isDash(getOffset() - 2);
        isAfoi=isAFOI(getOffset() - 2);
        indicatingTok=presidedByIndicatingTok(getOffset() - 2);
        distCLF=isCloseToCLF(getOffset() - 2);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 2);
            isa = isAdjective(getOffset() - 2);
            fcap = isFirstCapital(getOffset() - 2);
            cap = isCapitalized(getOffset() - 2);
            grSuffix = greekNameSuffixProperty(getOffset() - 2);
            grPrefix = greekNamePrefixProperty(getOffset() - 2);
            grEnding = greekEnding(getOffset() - 2);
            sigmaTel = isSigmaTeliko(getOffset() - 2);
            isIngen=isInGenetive(getOffset() - 2);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 2);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        //toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        
        //in case we are in second pass experiment
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null){    
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() - 2, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() - 2, allFpOrgDecisionsArr);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() - 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() - 2);
            
            
            int forPos = getOffset() - 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance -1////////////////////
        //////////////////from current token (previous token)///////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset() - 1);
        isenw = isInLtChar(getOffset() - 1);
        isd = isDot(getOffset() - 1);
        isn = isNumber(getOffset() - 1);
        isc = isComma(getOffset() - 1);
        iseop = isEndOfPeriod(getOffset() - 1);
        len = getLengthProperty(getOffset() - 1);
        isSlash = isSlash(getOffset() - 1);
        isDash = isDash(getOffset() - 1);
        isAfoi=isAFOI(getOffset() - 1);
        indicatingTok=presidedByIndicatingTok(getOffset() - 1);
        distCLF=isCloseToCLF(getOffset() - 1);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() - 1);
            isa = isAdjective(getOffset() - 1);
            fcap = isFirstCapital(getOffset() - 1);
            cap = isCapitalized(getOffset() - 1);
            grSuffix = greekNameSuffixProperty(getOffset() - 1);
            grPrefix = greekNamePrefixProperty(getOffset() - 1);
            grEnding = greekEnding(getOffset() - 1);
            sigmaTel = isSigmaTeliko(getOffset() - 1);
            isIngen=isInGenetive(getOffset() - 1);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() - 1);
        toReturn[attributeNum++] = insideSubtitle(getOffset() - 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null) {
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() - 1, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() - 1, allFpOrgDecisionsArr);
            
            // toReturn[attributeNum++] =
               //     isInList(perFPSureList, offset - 1);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() - 1);
            
            int forPos = getOffset() - 1;
            
            // toReturn[attributeNum++] =
                 //   distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        //////////////current token feature values (distance = 0 )//////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(getOffset());
        isenw = isInLtChar(getOffset());
        isd = isDot(getOffset());
        isn = isNumber(getOffset());
        isc = isComma(getOffset());
        iseop = isEndOfPeriod(getOffset());
        len = getLengthProperty(getOffset());
        isSlash = isSlash(getOffset());
        isDash = isDash(getOffset());
        isAfoi=isAFOI(getOffset());
        indicatingTok=presidedByIndicatingTok(getOffset());
        distCLF=isCloseToCLF(getOffset());
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset());
            isa = isAdjective(getOffset());
            fcap = isFirstCapital(getOffset());
            cap = isCapitalized(getOffset());
            grSuffix = greekNameSuffixProperty(getOffset());
            grPrefix = greekNamePrefixProperty(getOffset());
            grEnding = greekEnding(getOffset());
            sigmaTel = isSigmaTeliko(getOffset());
            isIngen=isInGenetive(getOffset());
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        // toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), getOffset());
        toReturn[attributeNum++] = insideSubtitle(getOffset());
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        // toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null) {
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset(), allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset(), allFpOrgDecisionsArr);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset());
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset);
            
            int forPos = getOffset();
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +1////////////////////
        ////////////////////from current token (next token)/////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() + 1 );
        isenw = isInLtChar(getOffset() + 1 );
        isd = isDot(getOffset() + 1 );
        isn = isNumber(getOffset() + 1 );
        isc = isComma(getOffset() + 1 );
        iseop = isEndOfPeriod(getOffset() + 1 );
        len = getLengthProperty(getOffset() + 1 );
        isSlash = isSlash(getOffset() + 1 );
        isDash = isDash(getOffset() + 1 );
        isAfoi=isAFOI(getOffset() + 1 );
        indicatingTok=presidedByIndicatingTok(getOffset() + 1 );
        distCLF=isCloseToCLF(getOffset() + 1 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() + 1 );
            isa = isAdjective(getOffset() + 1 );
            fcap = isFirstCapital(getOffset() + 1 );
            cap = isCapitalized(getOffset() + 1 );
            grSuffix = greekNameSuffixProperty(getOffset() + 1 );
            grPrefix = greekNamePrefixProperty(getOffset() + 1 );
            grEnding = greekEnding(getOffset() + 1 );
            sigmaTel = isSigmaTeliko(getOffset() + 1 );
            isIngen=isInGenetive(getOffset() + 1 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),getOffset() + 1);
        toReturn[attributeNum++] = insideSubtitle(getOffset() + 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null) {
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 1, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 1, allFpOrgDecisionsArr);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 1);
                    
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() + 1);
            
            int forPos = getOffset() + 1;
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +2////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() +2 );
        isenw = isInLtChar(getOffset() +2 );
        isd = isDot(getOffset() +2 );
        isn = isNumber(getOffset() +2 );
        isc = isComma(getOffset() +2 );
        iseop = isEndOfPeriod(getOffset() +2 );
        len = getLengthProperty(getOffset() +2 );
        isSlash = isSlash(getOffset() +2 );
        isDash = isDash(getOffset() +2 );
        isAfoi=isAFOI(getOffset() +2 );
        indicatingTok=presidedByIndicatingTok(getOffset() +2 );
        distCLF=isCloseToCLF(getOffset() +2 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() +2 );
            isa = isAdjective(getOffset() +2 );
            fcap = isFirstCapital(getOffset() +2 );
            cap = isCapitalized(getOffset() +2 );
            grSuffix = greekNameSuffixProperty(getOffset() +2 );
            grPrefix = greekNamePrefixProperty(getOffset() +2 );
            grEnding = greekEnding(getOffset() +2 );
            sigmaTel = isSigmaTeliko(getOffset() +2 );
            isIngen=isInGenetive(getOffset() +2 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        //toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),offset +2);
        toReturn[attributeNum++] = insideSubtitle(getOffset() +2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        //toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null){
                        
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 2, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 2, allFpOrgDecisionsArr);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, getOffset() + 2);
            
            int forPos = getOffset() + 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +3////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(getOffset() + 3 );
        isenw = isInLtChar(getOffset() + 3 );
        isd = isDot(getOffset() + 3 );
        isn = isNumber(getOffset() + 3 );
        isc = isComma(getOffset() + 3 );
        iseop = isEndOfPeriod(getOffset() + 3 );
        len = getLengthProperty(getOffset() + 3 );
        isSlash = isSlash(getOffset() + 3 );
        isDash = isDash(getOffset() + 3 );
        isAfoi=isAFOI(getOffset() + 3 );
        indicatingTok=presidedByIndicatingTok(getOffset() + 3 );
        distCLF=isCloseToCLF(getOffset() + 3 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(getOffset() + 3 );
            isa = isAdjective(getOffset() + 3 );
            fcap = isFirstCapital(getOffset() + 3 );
            cap = isCapitalized(getOffset() + 3 );
            grSuffix = greekNameSuffixProperty(getOffset() + 3 );
            grPrefix = greekNamePrefixProperty(getOffset() + 3 );
            grEnding = greekEnding(getOffset() + 3 );
            sigmaTel = isSigmaTeliko(getOffset() + 3 );
            isIngen=isInGenetive(getOffset() + 3 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), getOffset() + 3);
        toReturn[attributeNum++] = insideSubtitle(getOffset() + 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        // toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( perFPSureList != null   && allFpPerDecisionsArr != null && orgFPSureList != null   && allFpOrgDecisionsArr != null) {
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 3, allFpPerDecisionsArr);
            
            toReturn[attributeNum++] =
                    entityFPProbability(getOffset() + 3, allFpOrgDecisionsArr);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, getOffset() + 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, offset + 3);
            
            int forPos = getOffset() + 3;
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpPerDecisionsArr, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingEntity(forPos, allFpOrgDecisionsArr, 0.8);
            
        } // if
        
        toReturn[attributeNum++] = isMr();
        toReturn[attributeNum++] = areMessrs(10);
        
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(0, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(0, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(1, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(2, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(3, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(4, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(5, Constants.PERSON), 1);
        
//        toReturn[attributeNum++] = isInPrevList(
//                listMan.getRepeatedWordsList(0, Finals.ORGANIZATION),
//                listMan.getAvgOfRepeatedWordsList(0, Finals.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(1, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(2, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(3, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(4, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(5, Constants.ORGANIZATION), 1);
        
        toReturn[attributeNum++] = distLessThan(distFromStart, 1);
        toReturn[attributeNum++] = distLessThan(distFromStart, 2);
        toReturn[attributeNum++] = distLessThan(distFromStart, 3);
        toReturn[attributeNum++] = distLessThan(distFromStart, 4);
        toReturn[attributeNum++] = distLessThan(distFromStart, 5);
        toReturn[attributeNum++] = distLessThan(distFromStart, 6);
        toReturn[attributeNum++] = distLessThan(distFromStart, 7);
        
        offset++;//proceed to next token
      
        return Auxiliary.getSubList(toReturn, attributeNum);
            
    } // getNextFeatureVector
    
    
    /**
     * 
     * Contructs and returns the array of feature values for the previous token (token at position offset-1 in text),
     * without appling safe rules. Features are computed for category Person.
     * (to be used for first pass only).
     *
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     *
     * @param sureDec
     *      (Null if first pass)
     *      A hashtable containing sure decisions of first pass classifiers (eg decisions with cannoncal posibility>0.8).
     *      Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text eg PERSON#text01.txt#231
     *
     * @param allDec
     *      (Null if first pass)
     *      All decisions of first pass classifiers (cannonical possibilties assgigned)in a hashtable. The hashtable uses
     *      key values such as Constants.PERSON + "#" + fileName + "#" + pos.
     *
     * @param filename
     *      Name of the file of the text.
     * 
     * @return An array of feature values for the next token.
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     *
     */
    public String[] prevFeatVec_noSafeRules(
            int distFromStart, Hashtable<String, String[]> sureDec, Hashtable<String, String> allDec,
            String fileName) {
        
        String perFPSureList[], orgFPSureList[];
        
        // we are searching for the previous token
        int prevOffset = this.getOffset() -1;
        
        
        // if there are no more tokens then return null
        if (prevOffset  == tokenizer.size()) {
            System.err.println("end of tokenizer"); 
        } // if
        
        String category = getCategoryForOffset(Constants.PERSON, prevOffset);
        String currentToken = tokenizer.get(prevOffset ).getToken();
        
        
        
        if (prevOffset < 0) {
            return null;
        } // if
        
        
        /**
         * table of feature value to be returned
         */
        String[] toReturn = new String[256];
        //enought size to ensure no out of bounds exception to be thrown
        
        int attributeNum = 0;
        
        toReturn[attributeNum++] = category;
        
        //for a window of 5 tokens (this means the token before the previous one,
        //the previous token, the current token, the next token and the one after the next)
        // fill in those string feature values (see each feature in comments)
        String isgrw,//is
                isenw, //is greek word
                isn,//is a number
                isd,//is a dot "."
                isc,//is a comma ","
                
                isDash,//is token at pos a dash?
                
                isSlash,//is token at pos a slash?
                isa,//is an adjective
                iseop,//is it end of period (not just a common dot)
                len,//length
                fcap,//is first letter capital
                cap,//all letters in capital
                
                sigmaTel,//ends with "sigma teliko" -->""
                
                isIngen,//is in genetive
                grSuffix,//has a commmon greek sirname suffix
                grPrefix,//has a commmon greek sirname prefix
                grEnding, //common Greek last character?
                inp,//is it in plural?
                
                isAfoi,//precided by eg ""
                
                indicatingTok,//tokens such as "..." etc
                
                distCLF;//a numeric feature representing how distant a token
        //is from e.g. AE, E e.t.c.(only for current token)
        
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-3 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(prevOffset - 3);
        isenw = isInLtChar(prevOffset - 3);
        isd = isDot(prevOffset - 3);
        isn = isNumber(prevOffset - 3);
        isc = isComma(prevOffset - 3);
        iseop = isEndOfPeriod(prevOffset - 3);
        len = getLengthProperty(prevOffset - 3);
        isSlash = isSlash(prevOffset - 3);
        isDash = isDash(prevOffset - 3);
        isAfoi=isAFOI(prevOffset - 3);
        indicatingTok=presidedByIndicatingTok(prevOffset - 3);
        distCLF=isCloseToCLF(prevOffset - 3);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset - 3);
            isa = isAdjective(prevOffset - 3);
            fcap = isFirstCapital(prevOffset - 3);
            cap = isCapitalized(prevOffset - 3);
            grSuffix = greekNameSuffixProperty(prevOffset - 3);
            grPrefix = greekNamePrefixProperty(prevOffset - 3);
            grEnding = greekEnding(prevOffset - 3);
            sigmaTel = isSigmaTeliko(prevOffset - 3);
            isIngen=isInGenetive(prevOffset - 3);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset - 3);
        toReturn[attributeNum++] = insideSubtitle(prevOffset - 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        // in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            // toReturn[attributeNum++] =
               //     personFPProbability(prevOffset - 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset - 3, allDec, fileName);
            
            // toReturn[attributeNum++] =
                //    isInList(perFPSureList, prevOffset - 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, prevOffset - 3);
            
            
            int forPos = prevOffset - 3;
            
             toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
              //      distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-2 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(prevOffset - 2);
        isenw = isInLtChar(prevOffset - 2);
        isd = isDot(prevOffset - 2);
        isn = isNumber(prevOffset - 2);
        isc = isComma(prevOffset - 2);
        iseop = isEndOfPeriod(prevOffset - 2);
        len = getLengthProperty(prevOffset - 2);
        isSlash = isSlash(prevOffset - 2);
        isDash = isDash(prevOffset - 2);
        isAfoi=isAFOI(prevOffset - 2);
        indicatingTok=presidedByIndicatingTok(prevOffset - 2);
        distCLF=isCloseToCLF(prevOffset - 2);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset - 2);
            isa = isAdjective(prevOffset - 2);
            fcap = isFirstCapital(prevOffset - 2);
            cap = isCapitalized(prevOffset - 2);
            grSuffix = greekNameSuffixProperty(prevOffset - 2);
            grPrefix = greekNamePrefixProperty(prevOffset - 2);
            grEnding = greekEnding(prevOffset - 2);
            sigmaTel = isSigmaTeliko(prevOffset - 2);
            isIngen=isInGenetive(prevOffset - 2);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset - 2);
        toReturn[attributeNum++] = insideSubtitle(prevOffset - 2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        //toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        
        //in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, prevOffset - 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, prevOffset - 2);
            
            
            int forPos = prevOffset - 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance -1////////////////////
        //////////////////from current token (previous token)///////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(prevOffset - 1);
        isenw = isInLtChar(prevOffset - 1);
        isd = isDot(prevOffset - 1);
        isn = isNumber(prevOffset - 1);
        isc = isComma(prevOffset - 1);
        iseop = isEndOfPeriod(prevOffset - 1);
        len = getLengthProperty(prevOffset - 1);
        isSlash = isSlash(prevOffset - 1);
        isDash = isDash(prevOffset - 1);
        isAfoi=isAFOI(prevOffset - 1);
        indicatingTok=presidedByIndicatingTok(prevOffset - 1);
        distCLF=isCloseToCLF(prevOffset - 1);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset - 1);
            isa = isAdjective(prevOffset - 1);
            fcap = isFirstCapital(prevOffset - 1);
            cap = isCapitalized(prevOffset - 1);
            grSuffix = greekNameSuffixProperty(prevOffset - 1);
            grPrefix = greekNamePrefixProperty(prevOffset - 1);
            grEnding = greekEnding(prevOffset - 1);
            sigmaTel = isSigmaTeliko(prevOffset - 1);
            isIngen=isInGenetive(prevOffset - 1);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset - 1);
        toReturn[attributeNum++] = insideSubtitle(prevOffset - 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset - 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset - 1, allDec, fileName);
            
            // toReturn[attributeNum++] =
               //     isInList(perFPSureList, prevOffset - 1);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, prevOffset - 1);
            
            int forPos = prevOffset - 1;
            
            // toReturn[attributeNum++] =
                 //   distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        //////////////current token feature values (distance = 0 )//////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(prevOffset );
        isenw = isInLtChar(prevOffset );
        isd = isDot(prevOffset );
        isn = isNumber(prevOffset );
        isc = isComma(prevOffset );
        iseop = isEndOfPeriod(prevOffset );
        len = getLengthProperty(prevOffset );
        isSlash = isSlash(prevOffset );
        isDash = isDash(prevOffset );
        isAfoi=isAFOI(prevOffset );
        indicatingTok=presidedByIndicatingTok(prevOffset );
        distCLF=isCloseToCLF(prevOffset );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset );
            isa = isAdjective(prevOffset );
            fcap = isFirstCapital(prevOffset );
            cap = isCapitalized(prevOffset );
            grSuffix = greekNameSuffixProperty(prevOffset );
            grPrefix = greekNamePrefixProperty(prevOffset );
            grEnding = greekEnding(prevOffset );
            sigmaTel = isSigmaTeliko(prevOffset );
            isIngen=isInGenetive(prevOffset );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        // toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset);
        toReturn[attributeNum++] = insideSubtitle(prevOffset);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        // toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, prevOffset);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, prevOffset);
            
            int forPos = prevOffset;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +1////////////////////
        ////////////////////from current token (next token)/////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(prevOffset + 1 );
        isenw = isInLtChar(prevOffset + 1 );
        isd = isDot(prevOffset + 1 );
        isn = isNumber(prevOffset + 1 );
        isc = isComma(prevOffset + 1 );
        iseop = isEndOfPeriod(prevOffset + 1 );
        len = getLengthProperty(prevOffset + 1 );
        isSlash = isSlash(prevOffset + 1 );
        isDash = isDash(prevOffset + 1 );
        isAfoi=isAFOI(prevOffset + 1 );
        indicatingTok=presidedByIndicatingTok(prevOffset + 1 );
        distCLF=isCloseToCLF(prevOffset + 1 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset + 1 );
            isa = isAdjective(prevOffset + 1 );
            fcap = isFirstCapital(prevOffset + 1 );
            cap = isCapitalized(prevOffset + 1 );
            grSuffix = greekNameSuffixProperty(prevOffset + 1 );
            grPrefix = greekNamePrefixProperty(prevOffset + 1 );
            grEnding = greekEnding(prevOffset + 1 );
            sigmaTel = isSigmaTeliko(prevOffset + 1 );
            isIngen=isInGenetive(prevOffset + 1 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset + 1);
        toReturn[attributeNum++] = insideSubtitle(prevOffset + 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, prevOffset + 1);
                    
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, prevOffset + 1);
            
            int forPos = prevOffset + 1;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +2////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(prevOffset +2 );
        isenw = isInLtChar(prevOffset +2 );
        isd = isDot(prevOffset +2 );
        isn = isNumber(prevOffset +2 );
        isc = isComma(prevOffset +2 );
        iseop = isEndOfPeriod(prevOffset +2 );
        len = getLengthProperty(prevOffset +2 );
        isSlash = isSlash(prevOffset +2 );
        isDash = isDash(prevOffset +2 );
        isAfoi=isAFOI(prevOffset +2 );
        indicatingTok=presidedByIndicatingTok(prevOffset +2 );
        distCLF=isCloseToCLF(prevOffset +2 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset +2 );
            isa = isAdjective(prevOffset +2 );
            fcap = isFirstCapital(prevOffset +2 );
            cap = isCapitalized(prevOffset +2 );
            grSuffix = greekNameSuffixProperty(prevOffset +2 );
            grPrefix = greekNamePrefixProperty(prevOffset +2 );
            grEnding = greekEnding(prevOffset +2 );
            sigmaTel = isSigmaTeliko(prevOffset +2 );
            isIngen=isInGenetive(prevOffset +2 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        //toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),prevOffset +2);
        toReturn[attributeNum++] = insideSubtitle(prevOffset +2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        //toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, prevOffset + 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, prevOffset + 2);
            
            int forPos = prevOffset + 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +3////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(prevOffset + 3 );
        isenw = isInLtChar(prevOffset + 3 );
        isd = isDot(prevOffset + 3 );
        isn = isNumber(prevOffset + 3 );
        isc = isComma(prevOffset + 3 );
        iseop = isEndOfPeriod(prevOffset + 3 );
        len = getLengthProperty(prevOffset + 3 );
        isSlash = isSlash(prevOffset + 3 );
        isDash = isDash(prevOffset + 3 );
        isAfoi=isAFOI(prevOffset + 3 );
        indicatingTok=presidedByIndicatingTok(prevOffset + 3 );
        distCLF=isCloseToCLF(prevOffset + 3 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(prevOffset + 3 );
            isa = isAdjective(prevOffset + 3 );
            fcap = isFirstCapital(prevOffset + 3 );
            cap = isCapitalized(prevOffset + 3 );
            grSuffix = greekNameSuffixProperty(prevOffset + 3 );
            grPrefix = greekNamePrefixProperty(prevOffset + 3 );
            grEnding = greekEnding(prevOffset + 3 );
            sigmaTel = isSigmaTeliko(prevOffset + 3 );
            isIngen=isInGenetive(prevOffset + 3 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), prevOffset + 3);
        toReturn[attributeNum++] = insideSubtitle(prevOffset + 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        // toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(prevOffset + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(prevOffset + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, prevOffset + 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, prevOffset + 3);
            
            int forPos = prevOffset + 3;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        toReturn[attributeNum++] = isMr();
        toReturn[attributeNum++] = areMessrs(10);
        
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(0, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(0, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(1, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(2, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(3, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(4, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(5, Constants.PERSON), 1);
        
//        toReturn[attributeNum++] = isInPrevList(
//                listMan.getRepeatedWordsList(0, Finals.ORGANIZATION),
//                listMan.getAvgOfRepeatedWordsList(0, Finals.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(1, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(2, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(3, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(4, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(5, Constants.ORGANIZATION), 1);
        
        toReturn[attributeNum++] = distLessThan(distFromStart, 1);
        toReturn[attributeNum++] = distLessThan(distFromStart, 2);
        toReturn[attributeNum++] = distLessThan(distFromStart, 3);
        toReturn[attributeNum++] = distLessThan(distFromStart, 4);
        toReturn[attributeNum++] = distLessThan(distFromStart, 5);
        toReturn[attributeNum++] = distLessThan(distFromStart, 6);
        toReturn[attributeNum++] = distLessThan(distFromStart, 7);
        
        return Auxiliary.getSubList(toReturn, attributeNum);
        
        
    } // prevFeatVec_noSafeRules
    
    /**
     * Contructs and returns the array of features values for token at position posInText, for category Person. 
     * 
     * @param posInText
     *      Position of token in text (starting from 0).      
     *     
     * @param distFromStart
     *      Distance from the first token of the current named entity found (eg Giorgos Papas). If the previous token 
     *      does not belong to an entity name, then the value of this parameter must be -1. If the current token
     *      is the second in the raw token of an entity name (eg "Papas"), then the value of this parameter must be 1, e.t.c...
     *      This parameter is useful for computing features.
     * 
     * @param sureDec
     *      (Null if first pass)
     *      A hashtable containing sure decisions of first pass classifiers (eg decisions with cannoncal posibility>0.8).
     *      Hashtable keys must be like:<br>
     *      Constants.PERSON + "#" + Name_of_text_file + "#" + position_in_text eg PERSON#text01.txt#231
     * 
     * @param allDec
     *      (Null if first pass)
     *      All decisions of first pass classifiers (cannonical possibilties assgigned)in a hashtable. The hashtable uses
     *      key values such as Constants.PERSON + "#" + fileName + "#" + pos.
     * 
     * @param filename
     *      Name of the file of the text.
     * 
     * @return An array of feature values for token at position "posInText" in text,
     * Feature values are represented by String values of "1" or "-1" (boolean features), and "-1.0" to "1.0" (numeric features).
     */
   public String[] featureVectorAt(
            int posInTxt, int distFromStart, Hashtable<String, String[]> sureDec,
            Hashtable<String, String> allDec, String fileName) {
        
        String perFPSureList[], orgFPSureList[];
        
        // if there are no more tokens then return null
        if (posInTxt == tokenizer.size()) {
            return null;
        } // if
        
        String category = getCategory(Constants.PERSON);
        String currentToken = tokenizer.get(posInTxt).getToken();
        
        ////////////////////////////////////////////////////////////////////////
        //////////skiping examination of this token at position pos/////////////
        ////////////////////////////////////////////////////////////////////////
        
        //skip dates and time by returning just the following:
        if (isInside(Constants.DATE) || isInside(Constants.TIME)) {
            
            posInTxt++;
            return (new String[] {category});
            
        } // if
        
        // if distFromStart == -1 or in other words we have not found an other
        // part of entity of this category before and close to this token. So we skip it
        // in the following cases:
        if (distFromStart == -1) {
            
            //if it isn't Greek nor English word skip it
            if ((isInGrChar(posInTxt).compareTo("1") != 0) &&
                    (isInLtChar(posInTxt).compareTo("1") != 0)) {
                posInTxt++;
                return (new String[] {category});
            } // if
            
            if ( (isFirstCapital(posInTxt).compareTo("1") != 0) &&
                    (isCapitalized(posInTxt).compareTo("1") != 0)) {
                posInTxt++;
                return (new String[] {category});
            } // if
            
            //if it is a Greek verb then skip it
            if (isInGrChar(posInTxt).compareTo("1") == 0) {
                if (currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("") ||
                        currentToken.endsWith("") || currentToken.endsWith("")) {
                    posInTxt++;
                    return (new String[] {category});
                } // if
                
                //if it is stop-word(inside safe list) skip it
                if (listMan.getSafeList().containsKey(currentToken.toLowerCase())) {
                    posInTxt++;
                    return (new String[] {category});
                } // if
            } // if
        } // if
        //=================================================================================
        ////////cases of skiping examination of current token at position pos//////////////
        //=================================================================================
        
        /**
         * table of feature value to be returned
         */
        String[] toReturn= new String[256];
        //enought size to ensure no out of bounds exception to be thrown
        
        int attributeNum = 0;
        toReturn[attributeNum++] = category;
        
        
        //for a window of 5 tokens (this means the token before the previous one,
        //the previous token, the current token, the next token and the one after the next)
        // fill in those string feature values (see each feature in comments)
        String isgrw,//is
                isenw, //is greek word
                isn,//is a number
                isd,//is a dot "."
                isc,//is a comma ","
                
                isDash,//is token at pos a dash?
                
                isSlash,//is token at pos a slash?
                isa,//is an adjective
                iseop,//is it end of period (not just a common dot)
                len,//length
                fcap,//is first letter capital
                cap,//all letters in capital
                
                sigmaTel,//ends with "sigma teliko" -->""
                
                isIngen,//is in genetive
                grSuffix,//has a commmon greek sirname suffix
                grPrefix,//has a commmon greek sirname prefix
                grEnding, //common Greek last character?
                inp,//is it in plural?
                
                isAfoi,//precided by eg ""
                
                indicatingTok,//tokens such as "..." etc
                
                distCLF;//a numeric feature representing how distant a token
        //is from e.g. AE, E e.t.c.(only for current token)
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-3 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(posInTxt - 3);
        isenw = isInLtChar(posInTxt - 3);
        isd = isDot(posInTxt - 3);
        isn = isNumber(posInTxt - 3);
        isc = isComma(posInTxt - 3);
        iseop = isEndOfPeriod(posInTxt - 3);
        len = getLengthProperty(posInTxt - 3);
        isSlash = isSlash(posInTxt - 3);
        isDash = isDash(posInTxt - 3);
        isAfoi=isAFOI(posInTxt - 3);
        indicatingTok=presidedByIndicatingTok(posInTxt - 3);
        distCLF=isCloseToCLF(posInTxt - 3);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt - 3);
            isa = isAdjective(posInTxt - 3);
            fcap = isFirstCapital(posInTxt - 3);
            cap = isCapitalized(posInTxt - 3);
            grSuffix = greekNameSuffixProperty(posInTxt - 3);
            grPrefix = greekNamePrefixProperty(posInTxt - 3);
            grEnding = greekEnding(posInTxt - 3);
            sigmaTel = isSigmaTeliko(posInTxt - 3);
            isIngen=isInGenetive(posInTxt - 3);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt - 3);
        toReturn[attributeNum++] = insideSubtitle(posInTxt - 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        // in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            // toReturn[attributeNum++] =
               //     personFPProbability(posInTxt - 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt - 3, allDec, fileName);
            
            // toReturn[attributeNum++] =
                //    isInList(perFPSureList, posInTxt - 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, posInTxt - 3);
            
            
            int forPos = posInTxt - 3;
            
             toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
              //      distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////// feature values for pre-previous//////////////////////
        ////////////////token (distance=-2 from current token)//////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(posInTxt - 2);
        isenw = isInLtChar(posInTxt - 2);
        isd = isDot(posInTxt - 2);
        isn = isNumber(posInTxt - 2);
        isc = isComma(posInTxt - 2);
        iseop = isEndOfPeriod(posInTxt - 2);
        len = getLengthProperty(posInTxt - 2);
        isSlash = isSlash(posInTxt - 2);
        isDash = isDash(posInTxt - 2);
        isAfoi=isAFOI(posInTxt - 2);
        indicatingTok=presidedByIndicatingTok(posInTxt - 2);
        distCLF=isCloseToCLF(posInTxt - 2);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt - 2);
            isa = isAdjective(posInTxt - 2);
            fcap = isFirstCapital(posInTxt - 2);
            cap = isCapitalized(posInTxt - 2);
            grSuffix = greekNameSuffixProperty(posInTxt - 2);
            grPrefix = greekNamePrefixProperty(posInTxt - 2);
            grEnding = greekEnding(posInTxt - 2);
            sigmaTel = isSigmaTeliko(posInTxt - 2);
            isIngen=isInGenetive(posInTxt - 2);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt - 2);
        toReturn[attributeNum++] = insideSubtitle(posInTxt - 2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        //toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        
        //in case we are in second pass experiment
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt - 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, posInTxt - 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, posInTxt - 2);
            
            
            int forPos = posInTxt - 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance -1////////////////////
        //////////////////from current token (previous token)///////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(posInTxt - 1);
        isenw = isInLtChar(posInTxt - 1);
        isd = isDot(posInTxt - 1);
        isn = isNumber(posInTxt - 1);
        isc = isComma(posInTxt - 1);
        iseop = isEndOfPeriod(posInTxt - 1);
        len = getLengthProperty(posInTxt - 1);
        isSlash = isSlash(posInTxt - 1);
        isDash = isDash(posInTxt - 1);
        isAfoi=isAFOI(posInTxt - 1);
        indicatingTok=presidedByIndicatingTok(posInTxt - 1);
        distCLF=isCloseToCLF(posInTxt - 1);
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt - 1);
            isa = isAdjective(posInTxt - 1);
            fcap = isFirstCapital(posInTxt - 1);
            cap = isCapitalized(posInTxt - 1);
            grSuffix = greekNameSuffixProperty(posInTxt - 1);
            grPrefix = greekNamePrefixProperty(posInTxt - 1);
            grEnding = greekEnding(posInTxt - 1);
            sigmaTel = isSigmaTeliko(posInTxt - 1);
            isIngen=isInGenetive(posInTxt - 1);
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt - 1);
        toReturn[attributeNum++] = insideSubtitle(posInTxt - 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt - 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt - 1, allDec, fileName);
            
            // toReturn[attributeNum++] =
               //     isInList(perFPSureList, posInTxt - 1);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, posInTxt - 1);
            
            int forPos = posInTxt - 1;
            
            // toReturn[attributeNum++] =
                 //   distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        //////////////current token feature values (distance = 0 )//////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        isgrw = isInGrChar(posInTxt );
        isenw = isInLtChar(posInTxt );
        isd = isDot(posInTxt );
        isn = isNumber(posInTxt );
        isc = isComma(posInTxt );
        iseop = isEndOfPeriod(posInTxt );
        len = getLengthProperty(posInTxt );
        isSlash = isSlash(posInTxt );
        isDash = isDash(posInTxt );
        isAfoi=isAFOI(posInTxt );
        indicatingTok=presidedByIndicatingTok(posInTxt );
        distCLF=isCloseToCLF(posInTxt );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt );
            isa = isAdjective(posInTxt );
            fcap = isFirstCapital(posInTxt );
            cap = isCapitalized(posInTxt );
            grSuffix = greekNameSuffixProperty(posInTxt );
            grPrefix = greekNamePrefixProperty(posInTxt );
            grEnding = greekEnding(posInTxt );
            sigmaTel = isSigmaTeliko(posInTxt );
            isIngen=isInGenetive(posInTxt );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        // toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt);
        toReturn[attributeNum++] = insideSubtitle(posInTxt);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        // toReturn[attributeNum++] = isAfoi;
        toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, posInTxt);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, posInTxt);
            
            int forPos = posInTxt;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            // toReturn[attributeNum++] =
               //     distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +1////////////////////
        ////////////////////from current token (next token)/////////////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(posInTxt + 1 );
        isenw = isInLtChar(posInTxt + 1 );
        isd = isDot(posInTxt + 1 );
        isn = isNumber(posInTxt + 1 );
        isc = isComma(posInTxt + 1 );
        iseop = isEndOfPeriod(posInTxt + 1 );
        len = getLengthProperty(posInTxt + 1 );
        isSlash = isSlash(posInTxt + 1 );
        isDash = isDash(posInTxt + 1 );
        isAfoi=isAFOI(posInTxt + 1 );
        indicatingTok=presidedByIndicatingTok(posInTxt + 1 );
        distCLF=isCloseToCLF(posInTxt + 1 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt + 1 );
            isa = isAdjective(posInTxt + 1 );
            fcap = isFirstCapital(posInTxt + 1 );
            cap = isCapitalized(posInTxt + 1 );
            grSuffix = greekNameSuffixProperty(posInTxt + 1 );
            grPrefix = greekNamePrefixProperty(posInTxt + 1 );
            grEnding = greekEnding(posInTxt + 1 );
            sigmaTel = isSigmaTeliko(posInTxt + 1 );
            isIngen=isInGenetive(posInTxt + 1 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt + 1);
        toReturn[attributeNum++] = insideSubtitle(posInTxt + 1);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt + 1, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, posInTxt + 1);
                    
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, posInTxt + 1);
            
            int forPos = posInTxt + 1;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        //=====================================================================//
        //=====================================================================//
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +2////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(posInTxt +2 );
        isenw = isInLtChar(posInTxt +2 );
        isd = isDot(posInTxt +2 );
        isn = isNumber(posInTxt +2 );
        isc = isComma(posInTxt +2 );
        iseop = isEndOfPeriod(posInTxt +2 );
        len = getLengthProperty(posInTxt +2 );
        isSlash = isSlash(posInTxt +2 );
        isDash = isDash(posInTxt +2 );
        isAfoi=isAFOI(posInTxt +2 );
        indicatingTok=presidedByIndicatingTok(posInTxt +2 );
        distCLF=isCloseToCLF(posInTxt +2 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt +2 );
            isa = isAdjective(posInTxt +2 );
            fcap = isFirstCapital(posInTxt +2 );
            cap = isCapitalized(posInTxt +2 );
            grSuffix = greekNameSuffixProperty(posInTxt +2 );
            grPrefix = greekNamePrefixProperty(posInTxt +2 );
            grEnding = greekEnding(posInTxt +2 );
            sigmaTel = isSigmaTeliko(posInTxt +2 );
            isIngen=isInGenetive(posInTxt +2 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        //toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(),posInTxt +2);
        toReturn[attributeNum++] = insideSubtitle(posInTxt +2);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        //toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        //toReturn[attributeNum++] = distCLF;
        toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null ) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt + 2, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, posInTxt + 2);
            
            toReturn[attributeNum++] =
                    isInList(orgFPSureList, posInTxt + 2);
            
            int forPos = posInTxt + 2;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        ///////////////////// feature values for distance +3////////////////////
        //////////////from current token (token after next token)///////////////
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        
        isgrw = isInGrChar(posInTxt + 3 );
        isenw = isInLtChar(posInTxt + 3 );
        isd = isDot(posInTxt + 3 );
        isn = isNumber(posInTxt + 3 );
        isc = isComma(posInTxt + 3 );
        iseop = isEndOfPeriod(posInTxt + 3 );
        len = getLengthProperty(posInTxt + 3 );
        isSlash = isSlash(posInTxt + 3 );
        isDash = isDash(posInTxt + 3 );
        isAfoi=isAFOI(posInTxt + 3 );
        indicatingTok=presidedByIndicatingTok(posInTxt + 3 );
        distCLF=isCloseToCLF(posInTxt + 3 );
        
        //if it is a grrek word then fill in the values of following features
        //some features like "is in plural" have no mining for none Greek words
        if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
            inp = inPlural(posInTxt + 3 );
            isa = isAdjective(posInTxt + 3 );
            fcap = isFirstCapital(posInTxt + 3 );
            cap = isCapitalized(posInTxt + 3 );
            grSuffix = greekNameSuffixProperty(posInTxt + 3 );
            grPrefix = greekNamePrefixProperty(posInTxt + 3 );
            grEnding = greekEnding(posInTxt + 3 );
            sigmaTel = isSigmaTeliko(posInTxt + 3 );
            isIngen=isInGenetive(posInTxt + 3 );
        } // if
        
        else {
            inp = "-1";
            isa = "-1";
            fcap = "-1";
            cap = "-1";
            grSuffix = "-1";
            grPrefix = "-1";
            grEnding = "-1";
            sigmaTel = "-1";
            isIngen = "-1";
        } // else
        toReturn[attributeNum++] = iseop;
        toReturn[attributeNum++] = len;
        toReturn[attributeNum++] = fcap;
        toReturn[attributeNum++] = cap;
        toReturn[attributeNum++] = grEnding;
        toReturn[attributeNum++] = isgrw;
        toReturn[attributeNum++] = isenw;
        toReturn[attributeNum++] = isn;
        toReturn[attributeNum++] = inp;
        toReturn[attributeNum++] = isd;
        toReturn[attributeNum++] = isc;
        toReturn[attributeNum++] = isa;
        toReturn[attributeNum++] = grPrefix;
        toReturn[attributeNum++] = grSuffix;
        toReturn[attributeNum++] = isInList(listMan.getListOfPersonNames(), posInTxt + 3);
        toReturn[attributeNum++] = insideSubtitle(posInTxt + 3);
        toReturn[attributeNum++] = isIngen;
        toReturn[attributeNum++] = sigmaTel;
        toReturn[attributeNum++] = indicatingTok;
        //toReturn[attributeNum++] = isAfoi;
        // toReturn[attributeNum++] = distCLF;
        // toReturn[attributeNum++] = isSlash;
        toReturn[attributeNum++] = isDash;
        
        
        //in case we are at second pass use two more features
        //related to results from first pass
        if ( sureDec != null   && allDec != null && fileName != null) {
            
            perFPSureList = sureDec.get(Constants.PERSON + "#" + fileName );
            orgFPSureList = sureDec.get(Constants.ORGANIZATION + "#" + fileName);
            
            toReturn[attributeNum++] =
                    personFPProbability(posInTxt + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    organizationFPProbability(posInTxt + 3, allDec, fileName);
            
            toReturn[attributeNum++] =
                    isInList(perFPSureList, posInTxt + 3);
            
            // toReturn[attributeNum++] =
               //     isInList(orgFPSureList, posInTxt + 3);
            
            int forPos = posInTxt + 3;
            
            toReturn[attributeNum++] =
                    distanceFromPrevPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromPrevOrg(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingPer(forPos, allDec, fileName, 0.8);
            
            toReturn[attributeNum++] =
                    distanceFromFollowingOrg(forPos, allDec, fileName, 0.8);
            
        } // if
        
        
        toReturn[attributeNum++] = isMr();
        toReturn[attributeNum++] = areMessrs(10);
        
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(0, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(0, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(1, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(2, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(3, Constants.PERSON), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(4, Constants.PERSON), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.PERSON),
                listMan.getAvgOfRepeatedWordsList(5, Constants.PERSON), 1);
        
//        toReturn[attributeNum++] = isInPrevList(
//                listMan.getRepeatedWordsList(0, Finals.ORGANIZATION),
//                listMan.getAvgOfRepeatedWordsList(0, Finals.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(1, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(1, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(2, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(2, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(3, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(3, Constants.ORGANIZATION), 1);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(4, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(4, Constants.ORGANIZATION), 7);
        toReturn[attributeNum++] = isInPrevList(
                listMan.getRepeatedWordsList(5, Constants.ORGANIZATION),
                listMan.getAvgOfRepeatedWordsList(5, Constants.ORGANIZATION), 1);
        
        toReturn[attributeNum++] = distLessThan(distFromStart, 1);
        toReturn[attributeNum++] = distLessThan(distFromStart, 2);
        toReturn[attributeNum++] = distLessThan(distFromStart, 3);
        toReturn[attributeNum++] = distLessThan(distFromStart, 4);
        toReturn[attributeNum++] = distLessThan(distFromStart, 5);
        toReturn[attributeNum++] = distLessThan(distFromStart, 6);
        toReturn[attributeNum++] = distLessThan(distFromStart, 7);
        
        posInTxt++;//proceed to next token
        
     
        return Auxiliary.getSubList(toReturn, attributeNum);
        
        
    } // getNextFeatureVector
    
} // end of class PersonFeaturesManager
