/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.regex.timex;

import java.io.*;
import java.util.ArrayList;

import ipl.nerc.regex.PatternNode;
import ipl.core.Tokenizer;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.Auxiliary;
import ipl.util.Constants;

/**
 * <p>Description: Creates patterns for temporal expressions from training
 * data.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class TimexTrainer extends TimexRegex {

  public TimexTrainer(String args[]) {
    super();

    if (args.length == 0) {
      printParametersAndExit("Invalid parameters");
    } // if

    int t = 0;
    String d = "";
    String o = "";
    for (int i = 0; i < args.length; i += 2) {
      char param = args[i].charAt(1);
      switch (param) {
        case 't':
          if (args.length <= i + 1) {
            printParametersAndExit("Invalid threshold");
          } // if
          t = Auxiliary.atoi(args[i + 1]);
          if (t < 0){
            printParametersAndExit("Invalid threshold");
          } // if
          break;
        case 'd':
          if (args.length <= i + 1) {
            printParametersAndExit("Invalid directory name");
          } // if
          d = args[i + 1];
          if (!(new File(d)).isDirectory()) {
            printParametersAndExit("Invalid directory name");
          } // if
          break;
        case 'o':
          if (args.length <= i + 1) {
            printParametersAndExit("Invalid output filename");
          } // if
          o = args[i + 1];
          break;
        default:
          printParametersAndExit("Invalid parameters");
          break;
      } // switch
    } // for

    if (d.length() == 0) {
      printParametersAndExit("You must give the directory name (-d)");
    } // if

    if (o.length() == 0) {
      printParametersAndExit("You must give the timex patterns output filename (-o)");
    } // if

    collectDateTimeExpressions(d);
    sort();
    //    args[0]   
    args[0] = Integer.toString(t);
    args[1] = o;
  } // constructor: TimexTrainer

  public TimexTrainer(String dirName) {
    super();
    collectDateTimeExpressions(dirName);
    sort();
  } // constructor: TimexTrainer

  public TimexTrainer() {
    super();
  } // constructor: TimexTrainer

  private void printParametersAndExit(String str) {
    System.err.println(str);
    System.err.println();
    System.err.println();
    System.err.println("java ipl.nerc.regex.timex.TimexTrainer -d <train> -o <patterns> [-t <threshold>]");
    System.err.println();
    System.err.println("-d");
    System.err.println("\tThe directory name of training data");
    System.err.println("-o");
    System.err.println("\tThe timex patterns output filename");
    System.err.println("-t");
    System.err.println("\tThe threshold\t(default: 0)");
    System.err.println();
    System.exit(1);
  } // printParametersAndExit

  public ArrayList<PatternNode> getRegexList() {
    return regexList;
  } // getRegexList

  public String[] generalization(ArrayList<String> expression) {
    int size = expression.size();
    String[] general_expr = new String[size];

    for (int i = 0; i < size; i++) {
      // SEPARATOR
      if (matchSeparator(expression.get(i))) {
        general_expr[i] = SEPARATOR;
      } // else if
      // NUMBER
      else if (matchNumber(expression.get(i))) {
        int length = expression.get(i).length();
        if (length == 4) {
          general_expr[i] = "([1-2][0-9][0-9][0-9])";
        } // if
        else {
          general_expr[i] = "[0-9]{" + length + "}";
        } // else
      } // else if
      // ARTICLE
      else if (matchArticle(expression.get(i))) {
        general_expr[i] = ARTICLE;
      } // else if
      // DAY
      else if (matchDay(expression.get(i))) {
        general_expr[i] = DAY;
      } // else if
      // MONTH
      else if (matchMonth(expression.get(i))) {
        general_expr[i] = MONTH;
      } // else if
      // PART OF THE DAY
      else if (matchPartOfTheDay(expression.get(i))) {
        general_expr[i] = PART_OF_THE_DAY;
      } // else if
      // SEASON
      else if (matchSeason(expression.get(i))) {
        general_expr[i] = SEASON;
      } // else if
      // SPECIAL DAY
      else if (matchSpecialDay(expression.get(i))) {
        general_expr[i] = SPECIAL_DAY;
      } // else if
      // SPELL
      else if (matchSpell(expression.get(i))) {
        general_expr[i] = SPELL;
      } // else if
      // ORDINAL NUMBER
      else if (matchOrdinalNumber(expression.get(i))) {
        general_expr[i] = ORDINAL_NUMBER;
      } // else if
      // ENDING
      else if (matchEnding(expression.get(i))) {
        general_expr[i] = ENDING;
      } // else if
      // ABBREVIATION
      else if (matchAbbreviation(expression.get(i))) {
        general_expr[i] = ABBREVIATION;
      } // else if
      // ATTRIBUTIVE ADJECTIVE
      else if (matchAttributiveAdjective(expression.get(i))) {
        general_expr[i] = ATTRIBUTIVE_ADJECTIVE;
      } // else if
      // RELATIVE
      else if (matchRelative(expression.get(i))) {
        general_expr[i] = RELATIVE;
      } // else if
      /************************************************************************/
      //         
      //        .
      /************************************************************************/
      else if ((expression.get(i).compareTo("(") == 0) || (expression.get(i).compareTo(")") == 0)) {
        general_expr[i] = "\\" + expression.get(i);
      } // else if
      else {
        general_expr[i] = expression.get(i);
      } // else
    } // for

    return general_expr;
  } // generalization

  private boolean equal(String[] regex1, String[] regex2) {
    if (regex1.length != regex2.length) {
      return false;
    } // if

    int length = regex1.length;
    for (int i = 0; i < length; i++) {
      if (regex1[i].compareTo(regex2[i]) != 0) {
        return false;
      } // if
    } // for

    return true;
  } // equal

  public int exists(String[] regex) {
    if (regexList.isEmpty()) {
      return -1;
    } // if

    int length = regexList.size();
    for (int i = 0; i < length; i++) {
      if (equal(regex, regexList.get(i).get_expression())) {
        return i;
      } // if
    } // for

    return -1;
  } // exists

  public void collectDateTimeExpressions(String dirName) {
    String tagTypes[] = {Constants.DATE, Constants.TIME};
    int count = 0;

    byte[] buff;
    String text;
    FileInputStream fis;
    File file;
    DirectoryManager dirMan = new DirectoryManager(dirName);

    Tokenizer tok;

    while ( (file = dirMan.getNextFile()) != null) {
      if (file.isFile()) {
        buff = new byte[ (int) file.length()];
        try {
          fis = new FileInputStream(file);
          fis.read(buff);
          fis.close();
        } // try
        catch (IOException e) {
          e.printStackTrace();
        } // catch
        text = new String(buff);
        tok = new Tokenizer(file.getName(), text, null, null);
        for (int i = 0; i < tok.size(); i++) {
          for (int j = 0; j < tagTypes.length; j++) {
            if (tok.get(i).getTagType().compareTo(tagTypes[j]) == 0) {
              count++;
              boolean end = false;
              ArrayList<String> expression = new ArrayList<String>();
              while (!end) {
                expression.add(tok.get(i).getToken());
                boolean isLast = false;
                if (i < tok.size() - 1) {
                  if (tok.get(i + 1).getTagType().compareTo(tagTypes[j]) != 0) {
                    isLast = true;
                  } // if
                } // if
                else {
                  isLast = true;
                } // else
                if (isLast) {
                  end = true;
                } // if
                else {
                  i++;
                } // else
              } // while
              String[] general_expr = generalization(expression);

              int pos;
              //  
              if ((pos = exists(general_expr)) == -1) {
                regexList.add(new PatternNode(general_expr));
              } // if
              // 
              else {
                regexList.get(pos).increase();
              } // else
            } // if
          } // for
        } // for
      } // if
    } // while
    System.out.println("date expressions: " + count);
  } // collectDateTimeExpressions

  public void combineNumericalExpressions() {
    int count = 0;
    for (int i = 0; i < regexList.size() - 1; i++) {
      for (int j = i + 1; j < regexList.size(); j++) {
        if (regexList.get(i).get_expression().length == regexList.get(j).get_expression().length) {
          String[] expr1 = regexList.get(i).get_expression();
          String[] expr2 = regexList.get(j).get_expression();

          int k = 0;
          int length = expr1.length;
          String[] new_expr = new String[length];
          boolean stop = false;
          while ((k < length) && !stop) {
            if (expr1[k].compareTo(expr2[k]) != 0) {
              if (expr1[k].startsWith("[0-9]") && expr2[k].startsWith("[0-9]")) {
                new_expr[k] = "(" + expr1[k] + "|" + expr2[k] + ")";
              } // if
              else {
                stop = true;
              } // else
            } // if
            else {
              new_expr[k] = expr1[k];
            } // else
            k++;
          } // while

          if (!stop) {
            count++;
            int c = regexList.get(i).getCount() + regexList.get(j).getCount();
            PatternNode node = new PatternNode(new_expr, c);
            regexList.remove(j);
            regexList.remove(i);
            regexList.add(i, node);
          } // if
        } // if
      } // for
    } // for
    System.out.println("Number of combinations: " + count);
  } // combineNumericalExpressions

  public void removeRare(int threshold) {
    for (int i = 0; i < regexList.size(); i++) {
      if (regexList.get(i).getCount() < threshold) {
        regexList.remove(i);
        i--;
      } // if
    } // for
  } // removeRare

  public void saveToFile(String fileName) {
    try {
      FileOutputStream fos = new FileOutputStream(fileName);
      int length = regexList.size();
      for (int i = 0; i < length; i++) {
        fos.write((regexList.get(i).getCount() + "\n").getBytes());
        String[] expr = regexList.get(i).get_expression();
        for (int j = 0; j < expr.length; j++) {
          fos.write((expr[j] + " ").getBytes());
        } // for
        fos.write("\n".getBytes());
      } // for
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch
  } // saveToFile

  public int getSize() {
    return regexList.size();
  } // getSize
/*
  public static void main(String args[]) throws IOException {
    TimexTrainer finder = new TimexTrainer(args);
    int cut = Auxiliary.atoi(args[0]);
    finder.combineNumericalExpressions();
    if (cut > 0) {
      finder.removeRare(cut);
    } // if
    System.out.println("Final regular expressions: " + finder.regexList.size());
    finder.saveToFile(args[1]);
  } // end of main
*/
} // end of class TimexTrainer
